@@ -158,18 +158,21 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 	float *mjpt, *prept, *curpt;
 	int *mpjpt;
 #endif
-	static TLS float mi, *m;
-	static TLS int **ijp;
-	static TLS int mpi, *mp;
-	static TLS float *w1, *w2;
-	static TLS float *match;
-	static TLS float *initverticalw;    /* kufuu sureba iranai */
-	static TLS float *lastverticalw;    /* kufuu sureba iranai */
-	static TLS char **mseq1;
-	static TLS char **mseq2;
-	static TLS char **mseq;
-	static TLS int **intwork;
-	static TLS float **floatwork;
+	static TLS float mi = 0.0;
+	static TLS float *m = NULL;
+	static TLS int **ijp = NULL;
+	static TLS int mpi = 0;
+	static TLS int *mp = NULL;
+	static TLS float *w1 = NULL;
+	static TLS float *w2 = NULL;
+	static TLS float *match = NULL;
+	static TLS float *initverticalw = NULL;    /* kufuu sureba iranai */
+	static TLS float *lastverticalw = NULL;    /* kufuu sureba iranai */
+	static TLS char **mseq1 = NULL;
+	static TLS char **mseq2 = NULL;
+	static TLS char **mseq = NULL;
+	static TLS int **intwork = NULL;
+	static TLS float **floatwork = NULL;
 	static TLS int orlgth1 = 0, orlgth2 = 0;
 
 	if( seq1 == NULL )
@@ -178,23 +181,23 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 		{
 			orlgth1 = 0;
 			orlgth2 = 0;
-			free( mseq1 );
-			free( mseq2 );
-			FreeFloatVec( w1 );
-			FreeFloatVec( w2 );
-			FreeFloatVec( match );
-			FreeFloatVec( initverticalw );
-			FreeFloatVec( lastverticalw );
+			if( mseq1 ) free( mseq1 ); mseq1 = NULL;
+			if( mseq2 ) free( mseq2 ); mseq2 = NULL;
+			if( w1 ) FreeFloatVec( w1 ); w1 = NULL;
+			if( w2 ) FreeFloatVec( w2 ); w2 = NULL;
+			if( match ) FreeFloatVec( match ); match = NULL;
+			if( initverticalw ) FreeFloatVec( initverticalw ); initverticalw = NULL;
+			if( lastverticalw ) FreeFloatVec( lastverticalw ); lastverticalw = NULL;
 
-			FreeFloatVec( m );
-			FreeIntVec( mp );
+			if( m ) FreeFloatVec( m ); m = NULL;
+			if( mp ) FreeIntVec( mp ); mp = NULL;
 
-			FreeCharMtx( mseq );
+			if( mseq ) FreeCharMtx( mseq ); mseq = NULL;
 
 
 
-			FreeFloatMtx( floatwork );
-			FreeIntMtx( intwork );
+			if( floatwork ) FreeFloatMtx( floatwork ); floatwork = NULL;
+			if( intwork ) FreeIntMtx( intwork ); intwork = NULL;
 		}
 		return( 0.0 );
 	}
@@ -202,10 +205,12 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
 
+#if 0
 	if( lgth1 <= 0 || lgth2 <= 0 )
 	{
 		fprintf( stderr, "WARNING (g11): lgth1=%d, lgth2=%d\n", lgth1, lgth2 );
 	}
+#endif
 
 #if 1
 	if( lgth1 == 0 && lgth2 == 0 )
@@ -214,7 +219,7 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 	if( lgth1 == 0 )
 	{
 		seq1[0][lgth2] = 0;
-		while( lgth2 ) seq1[0][--lgth2] = '-';
+		while( lgth2 ) seq1[0][--lgth2] = *newgapstr;
 //		fprintf( stderr, "seq1[0] = %s\n", seq1[0] );
 		return( 0.0 );
 	}
@@ -222,7 +227,7 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 	if( lgth2 == 0 )
 	{
 		seq2[0][lgth1] = 0;
-		while( lgth1 ) seq2[0][--lgth1] = '-';
+		while( lgth1 ) seq2[0][--lgth1] = *newgapstr;
 //		fprintf( stderr, "seq2[0] = %s\n", seq2[0] );
 		return( 0.0 );
 	}
@@ -282,8 +287,8 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 		mseq = AllocateCharMtx( njob, ll1+ll2 );
 
 
-		floatwork = AllocateFloatMtx( 26, MAX( ll1, ll2 )+2 ); 
-		intwork = AllocateIntMtx( 26, MAX( ll1, ll2 )+2 ); 
+		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
+		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 
 #if DEBUG
 		fprintf( stderr, "succeeded\n" );
@@ -551,10 +556,12 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 
 
 
+#if 0
 	if( lgth1 <= 0 || lgth2 <= 0 )
 	{
 		fprintf( stderr, "WARNING (g11): lgth1=%d, lgth2=%d\n", lgth1, lgth2 );
 	}
+#endif
 
 	if( lgth1 > orlgth1 || lgth2 > orlgth2 )
 	{
@@ -595,8 +602,8 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 
 
 
-		floatwork = AllocateFloatMtx( 26, MAX( ll1, ll2 )+2 ); 
-		intwork = AllocateIntMtx( 26, MAX( ll1, ll2 )+2 ); 
+		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
+		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 
 #if DEBUG
 		fprintf( stderr, "succeeded\n" );
