@@ -94,7 +94,7 @@ static void match_ribosum( float *match, float **cpmx1, float **cpmx2, int i1, i
 static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int lgth2, float **floatwork, int **intwork, int initialize )
 {
 	int j, k, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
 	int count = 0;
@@ -102,13 +102,15 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 	float **cpmxpdpt;
 	int **cpmxpdnpt;
 	int cpkd;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 
 	if( initialize )
 	{
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[j][l] )
 				{
@@ -121,10 +123,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		}
 	}
 
-	for( l=0; l<26; l++ )
+	for( l=0; l<nalphabets; l++ )
 	{
 		scarr[l] = 0.0;
-		for( k=0; k<26; k++ )
+		for( k=0; k<nalphabets; k++ )
 		{
 			scarr[l] += (n_dis[k][l]-RNAthr) * cpmx1[i1][k];
 		}
@@ -165,13 +167,14 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		cpmxpdpt++;
 	}
 #endif
+	free( scarr );
 }
 
 #if 0
 static void match_add( float *match, float **cpmx1, float **cpmx2, int i1, int lgth2, float **floatwork, int **intwork, int initialize )
 {
 	int j, k, l;
-	float scarr[26];
+	float scarr[nalphabets];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
 	int count = 0;
@@ -186,7 +189,7 @@ static void match_add( float *match, float **cpmx1, float **cpmx2, int i1, int l
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[j][l] )
 				{
@@ -199,10 +202,10 @@ static void match_add( float *match, float **cpmx1, float **cpmx2, int i1, int l
 		}
 	}
 
-	for( l=0; l<26; l++ )
+	for( l=0; l<nalphabets; l++ )
 	{
 		scarr[l] = 0.0;
-		for( k=0; k<26; k++ )
+		for( k=0; k<nalphabets; k++ )
 		{
 			scarr[l] += n_dis[k][l] * cpmx1[i1][k];
 		}
@@ -511,8 +514,8 @@ static float MSalign2m2m_rec( int icyc, int jcyc, double *eff1, double *eff2, ch
 	mp = AllocateIntVec( ll2+2 );
 	gaps = AllocateCharVec( MAX( ll1, ll2 ) + 2 );
 
-	floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 26 ); 
-	intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 26 ); 
+	floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
+	intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets ); 
 
 #if DEBUG
 	fprintf( stderr, "succeeded\n" );
@@ -1243,11 +1246,10 @@ static float MSalign2m2m_rec( int icyc, int jcyc, double *eff1, double *eff2, ch
 static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, char **mseq1, char **mseq2, int depth, float **gapinfo, float **map )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
-	int alnlen;
 	float value = 0.0;
 	register int i, j;
 	char **aseq1, **aseq2;
-	int ll1, ll2, l, len;
+	int ll1, ll2;
 	int lasti, lastj, imid, jmid=0;
 	float wm = 0.0;   /* int ?????? */
 	float g;
@@ -1412,8 +1414,8 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	mp = AllocateIntVec( ll2+2 );
 	gaps = AllocateCharVec( MAX( ll1, ll2 ) + 2 );
 
-	floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 26 ); 
-	intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 26 ); 
+	floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
+	intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets ); 
 
 #if DEBUG
 	fprintf( stderr, "succeeded\n" );
@@ -2140,130 +2142,6 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	FreeFloatMtx( WMMTX2 );
 #endif
 
-	return( value );
-
-//	fprintf( stderr, "==== calling myself (first)\n" );
-
-#if 0
-		fprintf( stderr, "seq1[0] = %s\n", seq1[0] );
-		fprintf( stderr, "seq2[0] = %s\n", seq2[0] );
-#endif
-	value = MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist, ist+jumpi, jst, jst+jumpj, alloclen, aseq1, aseq2, depth, gapinfo, map );	
-#if 0
-		fprintf( stderr, "aseq1[0] = %s\n", aseq1[0] );
-		fprintf( stderr, "aseq2[0] = %s\n", aseq2[0] );
-#endif
-#if MEMSAVE
-#else
-	for( i=0; i<icyc; i++ ) strcpy( mseq1[i], aseq1[i] );
-	for( i=0; i<jcyc; i++ ) strcpy( mseq2[i], aseq2[i] );
-#endif
-
-//	fprintf( stderr, "====(f) aseq1[0] (%d) = %s (%d-%d)\n", depth, aseq1[0], ist, ien );
-//	fprintf( stderr, "====(f) aseq2[0] (%d) = %s (%d-%d)\n", depth, aseq2[0], jst, jen );
-
-	len = strlen( mseq1[0] );
-//	fprintf( stderr, "len = %d\n", len );
-	l = jmid - jumpj - 1;
-//	fprintf( stderr, "l=%d\n", l );
-	if( l > 0 )
-	{
-		for( i=0; i<l; i++ ) gaps[i] = '-'; gaps[i] = 0;
-		for( i=0; i<icyc; i++ ) 
-		{
-			strcat( mseq1[i], gaps );
-			mseq1[i][len+l] = 0;
-		}
-		for( j=0; j<jcyc; j++ )
-		{
-			strncat( mseq2[j], seq2[j]+jst+jumpj+1, l );
-			mseq2[j][len+l] = 0;
-		}
-//		fprintf( stderr, "penalizing (2) .. %f(%d), %f(%d)\n", ogcp2[jumpj+1], jumpj+1, fgcp2[jmid-1], jmid-1 );
-		value +=  ( ogcp2[jumpj+1] + fgcp2[jmid-1] );
-//		value += fpenalty;
-	}
-	len = strlen( mseq1[0] );
-	l = imid - jumpi - 1;
-//	fprintf( stderr, "l=%d\n", l );
-	if( l > 0 )
-	{
-		for( i=0; i<l; i++ ) gaps[i] = '-'; gaps[i] = 0;
-		for( i=0; i<icyc; i++ )
-		{
-			strncat( mseq1[i], seq1[i]+ist+jumpi+1, l );
-			mseq1[i][len+l] = 0;
-		}
-		for( j=0; j<jcyc; j++ ) 
-		{
-			strcat( mseq2[j], gaps );
-			mseq2[j][len+l] = 0;
-		}
-
-//		for( i=0; i<lgth1; i++ ) fprintf( stderr, "ogcp1[%d] = %f\n", i, ogcp1[i] );
-//		for( i=0; i<lgth1; i++ ) fprintf( stderr, "fgcp1[%d] = %f\n", i, fgcp1[i] );
-
-
-//		fprintf( stderr, "penalizing (1) .. ogcp1[%d] = %f, fgcp1[%d] = %f\n", jumpi+1, ogcp1[jumpi+1], imid-1, fgcp1[imid-1] );
-		value += ( ogcp1[jumpi+1] + fgcp1[imid-1] );
-//		value += fpenalty;
-	}
-#if 0
-	for( i=0; i<icyc; i++ ) fprintf( stderr, "after gapfill mseq1[%d]=%s\n", i, mseq1[i] );
-	for( i=0; i<jcyc; i++ ) fprintf( stderr, "after gapfill mseq2[%d]=%s\n", i, mseq2[i] );
-#endif
-
-//	fprintf( stderr, "==== calling myself (second)\n" );
-
-#if MEMSAVE
-	alnlen = strlen( aseq1[0] );
-	for( i=0; i<icyc; i++ ) aseq1[i] += alnlen;
-	for( i=0; i<jcyc; i++ ) aseq2[i] += alnlen;
-#endif
-
-#if 0
-		fprintf( stderr, "seq1[0] = %s\n", seq1[0] );
-		fprintf( stderr, "seq2[0] = %s\n", seq2[0] );
-#endif
-	value += MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist+imid, ien, jst+jmid, jen, alloclen, aseq1, aseq2, depth, gapinfo, map );	
-#if 0
-		fprintf( stderr, "aseq1[0] = %s\n", aseq1[0] );
-		fprintf( stderr, "aseq2[0] = %s\n", aseq2[0] );
-#endif
-
-
-
-#if DEBUG
-	if( value - maxwm > 1 || maxwm - value > 1 )
-	{
-		fprintf( stderr, "WARNING value  = %f, but maxwm = %f\n", value, maxwm );
-		for( i=0; i<icyc; i++ )
-		{
-			fprintf( stderr, ">1-%d\n%s\n", i, mseq1[i] );
-			fprintf( stderr, "%s\n", aseq1[i] );
-		}
-		for( i=0; i<jcyc; i++ )
-		{
-			fprintf( stderr, ">2-%d\n%s\n", i, mseq2[i] );
-			fprintf( stderr, "%s\n", aseq2[i] );
-		}
-
-//		exit( 1 );
-	}
-	else
-	{
-		fprintf( stderr, "value = %.0f, maxwm = %.0f -> ok\n", value, maxwm );
-	}
-#endif
-
-#if MEMSAVE
-#else
-	for( i=0; i<icyc; i++ ) strcat( mseq1[i], aseq1[i] );
-	for( i=0; i<jcyc; i++ ) strcat( mseq2[i], aseq2[i] );
-#endif
-
-//	fprintf( stderr, "====(s) aseq1[0] (%d) = %s (%d-%d)\n", depth, aseq1[0], ist, ien );
-//	fprintf( stderr, "====(s) aseq2[0] (%d) = %s (%d-%d)\n", depth, aseq2[0], jst, jen );
 
 	free( gaps );
 #if MEMSAVE
@@ -2329,8 +2207,8 @@ float Lalignmm_hmout( char **seq1, char **seq2, double *eff1, double *eff2, int
 	fgcp2 = AllocateFloatVec( ll2+2 );
 
 
-	cpmx1 = AllocateFloatMtx( ll1+2, 27 );
-	cpmx2 = AllocateFloatMtx( ll2+2, 27 );
+	cpmx1 = AllocateFloatMtx( ll1+2, nalphabets+1 );
+	cpmx2 = AllocateFloatMtx( ll2+2, nalphabets+1 );
 
 	for( i=0; i<icyc; i++ ) 
 	{
