@@ -16,7 +16,7 @@ static TLS int reccycle = 0;
 static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int lgth2, float **floatwork, int **intwork, int initialize )
 {
 	int j, k, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
 	int count = 0;
@@ -24,13 +24,14 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 	float **cpmxpdpt;
 	int **cpmxpdnpt;
 	int cpkd;
-
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 	if( initialize )
 	{
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[j][l] )
 				{
@@ -43,10 +44,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		}
 	}
 
-	for( l=0; l<26; l++ )
+	for( l=0; l<nalphabets; l++ )
 	{
 		scarr[l] = 0.0;
-		for( k=0; k<26; k++ )
+		for( k=0; k<nalphabets; k++ )
 		{
 			scarr[l] += n_dis[k][l] * cpmx1[i1][k];
 		}
@@ -87,6 +88,7 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		cpmxpdpt++;
 	}
 #endif
+	free( scarr );
 }
 
 static float Atracking( float *lasthorizontalw, float *lastverticalw,
@@ -238,7 +240,7 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 	return( 0.0 );
 }
 
-static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, float **gapinfo, int headgp, int tailgp )
+static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, float **gapinfo, int headgp, int tailgp, float headgapfreq1_g, float headgapfreq2_g )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
@@ -271,15 +273,26 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 	float *fgcp1;
 	float *ogcp2;
 	float *fgcp2;
+	float *gapfreq1f;
+	float *gapfreq2f;
 //	char **aseq1;
 //	char **aseq2;
 //	char **aseq1bk, **aseq2bk;
+	float headgapfreq1;
+	float headgapfreq2;
 
 
 	ogcp1 = gapinfo[0] + ist;
 	fgcp1 = gapinfo[1] + ist;
 	ogcp2 = gapinfo[2] + jst;
 	fgcp2 = gapinfo[3] + jst;
+	gapfreq1f = gapinfo[4] + ist;
+	gapfreq2f = gapinfo[5] + jst;
+
+	if( ist > 0 ) headgapfreq1 = gapfreq1f[-1];
+	else headgapfreq1 = headgapfreq1_g;
+	if( jst > 0 ) headgapfreq2 = gapfreq2f[-1];
+	else headgapfreq2 = headgapfreq2_g;
 
 #if STOREWM
 	char ttt1[10000], ttt2[10000];
@@ -326,8 +339,8 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 	m = AllocateFloatVec( ll2+2 );
 	mp = AllocateIntVec( ll2+2 );
 
-	floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 27 ); 
-	intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 27 ); 
+	floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets+1 ); 
+	intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets+1 ); 
 
 
 	intmtx = AllocateIntMtx( ll1+1, ll2+1 );
@@ -345,20 +358,20 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 	{
 		for( i=1; i<lgth1+1; i++ )
 		{
-			initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] ) ;
+			initverticalw[i] += ( ogcp1[0] * headgapfreq2 + fgcp1[i-1] * gapfreq2f[0] ) ;
 		}
 	}
 	if( headgp || jst != 0 )
 	{
 		for( j=1; j<lgth2+1; j++ )
 		{
-			currentw[j] += ( ogcp2[0] + fgcp2[j-1] ) ;
+			currentw[j] += ( ogcp2[0] * headgapfreq1 + fgcp2[j-1] * gapfreq1f[0] ) ;
 		}
 	}
 
 	for( j=1; j<lgth2+1; ++j ) 
 	{
-		m[j] = currentw[j-1] + ogcp1[1]; mp[j] = 0;;
+		m[j] = currentw[j-1] + ogcp1[1] * gapfreq2f[j-1]; mp[j] = 0;;
 	}
 
 	lastverticalw[0] = currentw[lgth2-1];
@@ -378,7 +391,7 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 		match_calc( currentw, cpmx1+ist, cpmx2+jst, i, lgth2, floatwork, intwork, 0 );
 		currentw[0] = initverticalw[i];
 
-		mi = previousw[0] + ogcp2[1]; 
+		mi = previousw[0] + ogcp2[1] * gapfreq1f[i-1];
 		mpi = 0;
 
 		ijppt = ijp[i] + 1;
@@ -396,7 +409,7 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 #if 0
 			fprintf( stderr, "%5.0f->", wm );
 #endif
-			g = mi + fgcp2[j-1];
+			g = mi + fgcp2[j-1]  * gapfreq1f[i];
 #if 0
 			fprintf( stderr, "%5.0f?", g );
 #endif
@@ -405,7 +418,7 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 				wm = g;
 				*ijppt = -( j - mpi );
 			}
-			g = *prept + ogcp2[j];
+			g = *prept + ogcp2[j] * gapfreq1f[i-1];
 			if( g >= mi )
 			{
 				mi = g;
@@ -415,7 +428,7 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 			mi += fpenalty_ex;
 #endif
 
-			g = *mjpt + fgcp1[i-1];
+			g = *mjpt + fgcp1[i-1] * gapfreq2f[j];
 #if 0 
 			fprintf( stderr, "%5.0f?", g );
 #endif
@@ -424,7 +437,7 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 				wm = g;
 				*ijppt = +( i - *mpjpt );
 			}
-			g = *prept + ogcp1[i];
+			g = *prept + ogcp1[i] * gapfreq2f[j-1];
 			if( g >= *mjpt )
 			{
 				*mjpt = g;
@@ -539,7 +552,7 @@ static void freearrays_rec2( char *gaps, char **aseq1, char **aseq2 )
 #endif
 }
 
-static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, int depth, float **gapinfo, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
+static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, int depth, float **gapinfo, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp, float headgapfreq1_g, float headgapfreq2_g )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
@@ -600,6 +613,10 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	static TLS char ttt1[50000];
 	static TLS char ttt2[50000];
 #endif
+	float *gapfreq1f;
+	float *gapfreq2f;
+	float headgapfreq1;
+	float headgapfreq2;
 
 #if 0
 	int nglen1, nglen2;
@@ -613,6 +630,13 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	fgcp1 = gapinfo[1] + ist;
 	ogcp2 = gapinfo[2] + jst;
 	fgcp2 = gapinfo[3] + jst;
+	gapfreq1f = gapinfo[4] + ist;
+	gapfreq2f = gapinfo[5] + jst;
+
+	if( ist > 0 ) headgapfreq1 = gapfreq1f[-1];
+	else headgapfreq1 = headgapfreq1_g;
+	if( jst > 0 ) headgapfreq2 = gapfreq2f[-1];
+	else headgapfreq2 = headgapfreq2_g;
 
 	depth++;
 	reccycle++;
@@ -649,7 +673,8 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 		{
 			mseq2[i][0] = 0;
 			for( j=0; j<lgth1; j++ )
-				strcat( mseq2[i], "-" );
+//				strcat( mseq2[i], "-" );
+				strcat( mseq2[i], newgapstr );
 		}
 
 //		fprintf( stderr, "==== mseq1[0] (%d) = %s\n", depth, mseq1[0] );
@@ -677,7 +702,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	{
 //		fprintf( stderr, "==== Going to _tanni\n" );
 
-		value = MSalignmm_tanni( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist, ien, jst, jen, alloclen, fulllen1, fulllen2, aseq1, aseq2, gapinfo, headgp, tailgp );	
+		value = MSalignmm_tanni( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist, ien, jst, jen, alloclen, fulllen1, fulllen2, aseq1, aseq2, gapinfo, headgp, tailgp, headgapfreq1_g, headgapfreq2_g );	
 
 
 #if MEMSAVE
@@ -722,8 +747,8 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	mp = AllocateIntVec( ll2+2 );
 	gaps = AllocateCharVec( MAX( ll1, ll2 ) + 2 );
 
-	floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 26 ); 
-	intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 26 ); 
+	floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
+	intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets ); 
 
 #if DEBUG
 	fprintf( stderr, "succeeded\n" );
@@ -752,11 +777,12 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 
 	for( i=1; i<lgth1+1; i++ )
 	{
-		initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] ) ;
+//		initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] ) ;
+		initverticalw[i] += ( ogcp1[0] * headgapfreq2 + fgcp1[i-1] * gapfreq2f[0] ) ;
 	}
 	for( j=1; j<lgth2+1; j++ )
 	{
-		currentw[j] += ( ogcp2[0] + fgcp2[j-1] ) ;
+		currentw[j] += ( ogcp2[0] * headgapfreq1 + fgcp2[j-1] * gapfreq1f[0]) ;
 	}
 
 #if STOREWM
@@ -774,7 +800,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 
 	for( j=1; j<lgth2+1; ++j ) 
 	{
-		m[j] = currentw[j-1] + ogcp1[1];
+		m[j] = currentw[j-1] + ogcp1[1] * gapfreq2f[j-1];
 //		m[j] = currentw[j-1];
 		mp[j] = 0;
 	}
@@ -826,7 +852,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 #endif
 		if( i == imid ) midm[0] = m[0];
 
-		mi = previousw[0] + ogcp2[1]; 
+		mi = previousw[0] + ogcp2[1] * gapfreq1f[i-1]; 
 //		mi = previousw[0];
 		mpi = 0;
 
@@ -847,7 +873,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 #if 0
 			fprintf( stderr, "%5.0f->", wm );
 #endif
-			g = mi + fgcp2[j-1];
+			g = mi + fgcp2[j-1] * gapfreq1f[i];
 //			g = mi + fpenalty;
 #if 0
 			fprintf( stderr, "%5.0f?", g );
@@ -857,7 +883,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 				wm = g;
 //				*ijppt = -( j - mpi );
 			}
-			g = *prept + ogcp2[j];
+			g = *prept + ogcp2[j] * gapfreq1f[i-1];
 //			g = *prept;
 			if( g >= mi )
 			{
@@ -868,7 +894,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 			mi += fpenalty_ex;
 #endif
 
-			g = *mjpt + fgcp1[i-1];
+			g = *mjpt + fgcp1[i-1] * gapfreq2f[j];
 //			g = *mjpt + fpenalty;
 #if 0 
 			fprintf( stderr, "%5.0f?", g );
@@ -880,7 +906,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 			}
 
 
-			g = *prept + ogcp1[i];
+			g = *prept + ogcp1[i] * gapfreq2f[j-1];
 //			g = *prept;
 			if( g >= *mjpt )
 			{
@@ -965,13 +991,13 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 
 	for( i=0; i<lgth1-1; i++ )
 	{
-		initverticalw[i] += ( fgcp1[lgth1-1] + ogcp1[i+1] );
-//		initverticalw[i] += fpenalty;
+//		initverticalw[i] += ( fgcp1[lgth1-1] + ogcp1[i+1] );
+		initverticalw[i] += ( fgcp1[lgth1-1] * gapfreq2f[lgth2] + ogcp1[i+1] * gapfreq2f[lgth2-1] );
 	}
 	for( j=0; j<lgth2-1; j++ )
 	{
-		currentw[j] += ( fgcp2[lgth2-1] + ogcp2[j+1] );
-//		currentw[j] += fpenalty;
+//		currentw[j] += ( fgcp2[lgth2-1] + ogcp2[j+1] );
+		currentw[j] += ( fgcp2[lgth2-1] * gapfreq1f[lgth1] + ogcp2[j+1] * gapfreq1f[lgth1-1] );
 	}
 
 #if STOREWM
@@ -992,12 +1018,21 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 
 
 
+#if 0
 	for( j=lgth2-1; j>0; --j )
 	{
 		m[j-1] = currentw[j] + fgcp2[lgth2-2];
 //		m[j-1] = currentw[j];
 		mp[j] = lgth1-1;
 	}
+#else
+	for( j=lgth2-1; j>-1; --j )
+	{
+		m[j] = currentw[j+1] + fgcp1[lgth1-2] * gapfreq2f[j+1];
+//		m[j-1] = currentw[j];
+		mp[j] = lgth1-1;
+	}
+#endif
 
 //	for( j=0; j<lgth2; j++ ) m[j] = 0.0;
 	// m[lgth2-1] ha irunoka?
@@ -1042,7 +1077,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 //		WMMTX2[i][lgth2] += m[lgth2];
 //		fprintf( stderr, "m[] = %f\n", m[lgth2] );
 
-		mi = previousw[lgth2-1] + fgcp2[lgth2-2];
+		mi = previousw[lgth2-1] + fgcp2[lgth2-2] * gapfreq1f[i+1];
 //		mi = previousw[lgth2-1];
 		mpi = lgth2 - 1;
 
@@ -1058,7 +1093,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 			ijpi = i+1;
 			ijpj = j+1;
 
-			g = mi + ogcp2[j+1];
+			g = mi + ogcp2[j+1] * gapfreq1f[i];
 //			g = mi + fpenalty;
 			if( g > wm )
 			{
@@ -1067,7 +1102,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 				ijpi = i+1;
 			}
 
-			g = *prept + fgcp2[j];
+			g = *prept + fgcp2[j] * gapfreq1f[i+1];
 //			g = *prept;
 			if( g >= mi )
 			{
@@ -1081,7 +1116,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 #endif
 
 //			fprintf( stderr, "i,j=%d,%d *mpjpt = %d\n", i, j, *mpjpt );
-			g = *mjpt + ogcp1[i+1];
+			g = *mjpt + ogcp1[i+1] * gapfreq2f[j];
 //			g = *mjpt + fpenalty;
 			if( g > wm )
 			{
@@ -1091,7 +1126,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 			}
 
 //			if( i == imid )fprintf( stderr, "i,j=%d,%d \n", i, j );
-			g = *prept + fgcp1[i];
+			g = *prept + fgcp1[i] * gapfreq2f[j+1];
 //			g = *prept;
 			if( g >= *mjpt )
 			{
@@ -1388,7 +1423,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 
 //	fprintf( stderr, "==== calling myself (first)\n" );
 
-	value = MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist, ist+jumpi, jst, jst+jumpj, alloclen, fulllen1, fulllen2, aseq1, aseq2, depth, gapinfo, NULL, 0, NULL, headgp, tailgp ); // chudan mada	
+	value = MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist, ist+jumpi, jst, jst+jumpj, alloclen, fulllen1, fulllen2, aseq1, aseq2, depth, gapinfo, NULL, 0, NULL, headgp, tailgp, headgapfreq1_g, headgapfreq2_g ); // chudan mada	
 #if 0
 		fprintf( stderr, "aseq1[0] = %s\n", aseq1[0] );
 		fprintf( stderr, "aseq2[0] = %s\n", aseq2[0] );
@@ -1408,7 +1443,8 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 //	fprintf( stderr, "l=%d\n", l );
 	if( l > 0 )
 	{
-		for( i=0; i<l; i++ ) gaps[i] = '-'; gaps[i] = 0;
+//		for( i=0; i<l; i++ ) gaps[i] = '-'; gaps[i] = 0;
+		for( i=0; i<l; i++ ) gaps[i] = *newgapstr; gaps[i] = 0;
 		for( i=0; i<icyc; i++ ) 
 		{
 			strcat( mseq1[i], gaps );
@@ -1428,7 +1464,8 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 //	fprintf( stderr, "l=%d\n", l );
 	if( l > 0 )
 	{
-		for( i=0; i<l; i++ ) gaps[i] = '-'; gaps[i] = 0;
+//		for( i=0; i<l; i++ ) gaps[i] = '-'; gaps[i] = 0;
+		for( i=0; i<l; i++ ) gaps[i] = *newgapstr; gaps[i] = 0;
 		for( i=0; i<icyc; i++ )
 		{
 			strncat( mseq1[i], seq1[i]+ist+jumpi+1, l );
@@ -1461,7 +1498,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	for( i=0; i<jcyc; i++ ) aseq2[i] += alnlen;
 #endif
 
-	value += MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist+imid, ien, jst+jmid, jen, alloclen, fulllen1, fulllen2, aseq1, aseq2, depth, gapinfo, NULL, 0, NULL, headgp, tailgp ); // chudan mada
+	value += MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist+imid, ien, jst+jmid, jen, alloclen, fulllen1, fulllen2, aseq1, aseq2, depth, gapinfo, NULL, 0, NULL, headgp, tailgp, headgapfreq1_g, headgapfreq2_g ); // chudan mada
 #if 0
 		fprintf( stderr, "aseq1[0] = %s\n", aseq1[0] );
 		fprintf( stderr, "aseq2[0] = %s\n", aseq2[0] );
@@ -1531,6 +1568,8 @@ static void freearrays(
 	float *fgcp2, 
 	float **cpmx1,
 	float **cpmx2, 
+	float *gapfreq1f,
+	float *gapfreq2f,
 	float **gapinfo, 
 	char **mseq1, 
 	char **mseq2 
@@ -1542,6 +1581,8 @@ static void freearrays(
 	FreeFloatVec( fgcp2 );
 	FreeFloatMtx( cpmx1 );
 	FreeFloatMtx( cpmx2 );
+	FreeFloatVec( gapfreq1f );
+	FreeFloatVec( gapfreq2f );
 	free( (void *)gapinfo );
 
 	FreeCharMtx( mseq1 );
@@ -1566,7 +1607,11 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	float **cpmx2;
 	float **gapinfo;
 	float fpenalty = (float)penalty;
+	float *gapfreq1f;
+	float *gapfreq2f;
 	int nglen1, nglen2;
+	float headgapfreq1;
+	float headgapfreq2;
 
 #if 0
 	fprintf( stderr, "eff in SA+++align\n" );
@@ -1592,15 +1637,18 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	mseq1 = AllocateCharMtx( icyc, ll1+ll2 );
 	mseq2 = AllocateCharMtx( jcyc, ll1+ll2 );
 
-	gapinfo = AllocateFloatMtx( 4, 0 );
+	gapinfo = AllocateFloatMtx( 6, 0 );
 	ogcp1 = AllocateFloatVec( ll1+2 );
 	ogcp2 = AllocateFloatVec( ll2+2 );
 	fgcp1 = AllocateFloatVec( ll1+2 );
 	fgcp2 = AllocateFloatVec( ll2+2 );
 
 
-	cpmx1 = AllocateFloatMtx( ll1+2, 27 );
-	cpmx2 = AllocateFloatMtx( ll2+2, 27 );
+	cpmx1 = AllocateFloatMtx( ll1+2, nalphabets+1 );
+	cpmx2 = AllocateFloatMtx( ll2+2, nalphabets+1 );
+
+	gapfreq1f = AllocateFloatVec( ll1+2 ); // must be filled with 0.0
+	gapfreq2f = AllocateFloatVec( ll2+2 ); // must be filled with 0.0
 
 	for( i=0; i<icyc; i++ ) 
 	{
@@ -1633,6 +1681,10 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		new_OpeningGapCount( ogcp2, jcyc, seq2, eff2, lgth2, sgap2 );
 		new_FinalGapCount( fgcp1, icyc, seq1, eff1, lgth1, egap2 );
 		new_FinalGapCount( fgcp2, jcyc, seq2, eff2, lgth2, egap2 );
+		outgapcount( &headgapfreq1, icyc, sgap1, eff1 );
+		outgapcount( &headgapfreq2, jcyc, sgap2, eff2 );
+		outgapcount( gapfreq1f+lgth1, icyc, egap1, eff1 );
+		outgapcount( gapfreq2f+lgth2, jcyc, egap2, eff2 );
 	}
 	else
 	{
@@ -1640,19 +1692,40 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		st_OpeningGapCount( ogcp2, jcyc, seq2, eff2, lgth2 );
 		st_FinalGapCount( fgcp1, icyc, seq1, eff1, lgth1 );
 		st_FinalGapCount( fgcp2, jcyc, seq2, eff2, lgth2 );
+		headgapfreq1 = 0.0;
+		headgapfreq2 = 0.0;
+		gapfreq1f[lgth1] = 0.0;
+		gapfreq2f[lgth2] = 0.0;
+	}
+
+	if( legacygapcost == 0 )
+	{
+		gapcountf( gapfreq1f, seq1, icyc, eff1, lgth1 );
+		gapcountf( gapfreq2f, seq2, jcyc, eff2, lgth2 );
+		for( i=0; i<lgth1+1; i++ ) gapfreq1f[i] = 1.0 - gapfreq1f[i];
+		for( i=0; i<lgth2+1; i++ ) gapfreq2f[i] = 1.0 - gapfreq2f[i];
+		headgapfreq1 = 1.0 - headgapfreq1;
+		headgapfreq2 = 1.0 - headgapfreq2;
+	}
+	else
+	{
+		for( i=0; i<lgth1+1; i++ ) gapfreq1f[i] = 1.0;
+		for( i=0; i<lgth2+1; i++ ) gapfreq2f[i] = 1.0;
+		headgapfreq1 = 1.0;
+		headgapfreq2 = 1.0;
 	}
 
 #if 1
 	for( i=0; i<lgth1; i++ ) 
 	{
-		ogcp1[i] = 0.5 * ( 1.0 - ogcp1[i] ) * fpenalty;
-		fgcp1[i] = 0.5 * ( 1.0 - fgcp1[i] ) * fpenalty;
+		ogcp1[i] = 0.5 * ( 1.0 - ogcp1[i] ) * fpenalty * ( gapfreq1f[i] );
+		fgcp1[i] = 0.5 * ( 1.0 - fgcp1[i] ) * fpenalty * ( gapfreq1f[i] );
 //		fprintf( stderr, "fgcp1[%d] = %f\n", i, fgcp1[i] );
 	}
 	for( i=0; i<lgth2; i++ ) 
 	{
-		ogcp2[i] = 0.5 * ( 1.0 - ogcp2[i] ) * fpenalty;
-		fgcp2[i] = 0.5 * ( 1.0 - fgcp2[i] ) * fpenalty;
+		ogcp2[i] = 0.5 * ( 1.0 - ogcp2[i] ) * fpenalty * ( gapfreq2f[i] );
+		fgcp2[i] = 0.5 * ( 1.0 - fgcp2[i] ) * fpenalty * ( gapfreq2f[i] );
 //		fprintf( stderr, "fgcp2[%d] = %f\n", i, fgcp2[i] );
 	}
 #else
@@ -1672,6 +1745,8 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	gapinfo[1] = fgcp1;
 	gapinfo[2] = ogcp2;
 	gapinfo[3] = fgcp2;
+	gapinfo[4] = gapfreq1f;
+	gapinfo[5] = gapfreq2f;
 #endif
 
 #if 0
@@ -1689,22 +1764,24 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	fflush( stdout );
 #endif
 
-	wm = MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, 0, lgth1-1, 0, lgth2-1, alloclen, lgth1, lgth2, mseq1, mseq2, 0, gapinfo, chudanpt, chudanref, chudanres, headgp, tailgp );
+	wm = MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, 0, lgth1-1, 0, lgth2-1, alloclen, lgth1, lgth2, mseq1, mseq2, 0, gapinfo, chudanpt, chudanref, chudanres, headgp, tailgp, headgapfreq1, headgapfreq2 );
 #ifdef enablemultithread
 	if( chudanres && *chudanres ) 
 	{
 //		fprintf( stderr, "\n\n## CHUUDAN!!! relay\n" );
 		*chudanres = 1;
-		freearrays( ogcp1, ogcp2, fgcp1, fgcp2, cpmx1, cpmx2, gapinfo, mseq1, mseq2 );
+		freearrays( ogcp1, ogcp2, fgcp1, fgcp2, cpmx1, cpmx2, gapfreq1f, gapfreq2f, gapinfo, mseq1, mseq2 );
 		return( -1.0 );
 	}
 #endif
 
-#if DEBUG
+#if 0
+		fprintf( stderr, "\n" );
 		fprintf( stderr, " seq1[0] = %s\n", seq1[0] );
 		fprintf( stderr, " seq2[0] = %s\n", seq2[0] );
 		fprintf( stderr, "mseq1[0] = %s\n", mseq1[0] );
 		fprintf( stderr, "mseq2[0] = %s\n", mseq2[0] );
+		fprintf( stderr, "\n" );
 #endif
 
 //	fprintf( stderr, "wm = %f\n", wm );
@@ -1726,7 +1803,7 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	}
 
 
-	freearrays( ogcp1, ogcp2, fgcp1, fgcp2, cpmx1, cpmx2, gapinfo, mseq1, mseq2 );
+	freearrays( ogcp1, ogcp2, fgcp1, fgcp2, cpmx1, cpmx2, gapfreq1f, gapfreq2f, gapinfo, mseq1, mseq2 );
 
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
@@ -1749,5 +1826,12 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		}
 	}
 
+#if 0
+	fprintf( stderr, "\n" );
+	for( i=0; i<icyc; i++ ) fprintf( stderr, " seq1[i] = %s\n", seq1[i] );
+	for( j=0; j<jcyc; j++ ) fprintf( stderr, " seq2[j] = %s\n", seq2[j] );
+	fprintf( stderr, "\n" );
+#endif
+
 	return( wm );
 }
