@@ -298,18 +298,20 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 {
 #if FASTMATCHCALC
 	int j, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
 	float *matchpt, *cpmxpdpt, **cpmxpdptpt;
 	int *cpmxpdnpt, **cpmxpdnptpt;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 	if( initialize )
 	{
 		int count = 0;
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[l][j] )
 				{
@@ -323,10 +325,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 	}
 
 	{
-		for( l=0; l<26; l++ )
+		for( l=0; l<nalphabets; l++ )
 		{
 			scarr[l] = 0.0;
-			for( j=0; j<26; j++ )
+			for( j=0; j<nalphabets; j++ )
 				scarr[l] += n_dis_consweight_multi[j][l] * cpmx1[j][i1];
 //			scarr[l] *= consweight_multi;
 		}
@@ -343,11 +345,14 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 			matchpt++;
 		} 
 	}
+	free( scarr );
 #else
 	int j, k, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 // simple
 	if( initialize )
 	{
@@ -355,7 +360,7 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[l][j] )
 				{
@@ -367,10 +372,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 			cpmxpdn[count][j] = -1;
 		}
 	}
-	for( l=0; l<26; l++ )
+	for( l=0; l<nalphabets; l++ )
 	{
 		scarr[l] = 0.0;
-		for( k=0; k<26; k++ )
+		for( k=0; k<nalphabets; k++ )
 			scarr[l] += n_dis_consweight_multi[k][l] * cpmx1[k][i1];
 //		scarr[l] *= consweight_multi;
 	}
@@ -380,6 +385,7 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		for( k=0; cpmxpdn[k][j]>-1; k++ )
 			match[j] += scarr[cpmxpdn[k][j]] * cpmxpd[k][j];
 	} 
+	free( scarr );
 #endif
 }
 
@@ -1036,15 +1042,15 @@ float Q__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		gapz_n1 = AllocateFloatVec( ll1+2 );
 		gapz_n2 = AllocateFloatVec( ll2+2 );
 
-		cpmx1 = AllocateFloatMtx( 26, ll1+2 );
-		cpmx2 = AllocateFloatMtx( 26, ll2+2 );
+		cpmx1 = AllocateFloatMtx( nalphabets, ll1+2 );
+		cpmx2 = AllocateFloatMtx( nalphabets, ll2+2 );
 
 #if FASTMATCHCALC
-		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 26 ); 
-		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 27 ); 
+		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
+		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets+1 ); 
 #else
-		floatwork = AllocateFloatMtx( 26, MAX( ll1, ll2 )+2 ); 
-		intwork = AllocateIntMtx( 26, MAX( ll1, ll2 )+2 ); 
+		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
+		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 #endif
 
 #if DEBUG
@@ -1703,15 +1709,15 @@ float Q__align_gapmap( char **seq1, char **seq2, double *eff1, double *eff2, int
 		gapz_n1 = AllocateFloatVec( ll1+2 );
 		gapz_n2 = AllocateFloatVec( ll2+2 );
 
-		cpmx1 = AllocateFloatMtx( 26, ll1+2 );
-		cpmx2 = AllocateFloatMtx( 26, ll2+2 );
+		cpmx1 = AllocateFloatMtx( nalphabets, ll1+2 );
+		cpmx2 = AllocateFloatMtx( nalphabets, ll2+2 );
 
 #if FASTMATCHCALC
-		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 26 ); 
-		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 27 ); 
+		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
+		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets+1 ); 
 #else
-		floatwork = AllocateFloatMtx( 26, MAX( ll1, ll2 )+2 ); 
-		intwork = AllocateIntMtx( 26, MAX( ll1, ll2 )+2 ); 
+		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
+		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 #endif
 
 #if DEBUG
@@ -1805,6 +1811,7 @@ float Q__align_gapmap( char **seq1, char **seq2, double *eff1, double *eff2, int
 		getgapfreq_zure( gapz2, jcyc, seq2, eff2, lgth2 );
 	}
 
+
 #if 1
 	lastj = lgth2+2;
 	for( i=0; i<lastj; i++ )
