@@ -9,7 +9,7 @@
 #define FASTMATCHCALC 1
 
 
-static float **impmtx = NULL;
+static TLS float **impmtx = NULL;
 #if 1 // tditeration to naiveRscore_imp de tsukawareru.
 float imp_match_out_scR( int i1, int j1 )
 {
@@ -43,11 +43,11 @@ static void imp_match_out_vead_tateR( float *imp, int j1, int lgth1 )
 void imp_match_init_strictR( float *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, LocalHom ***localhom, int forscore )
 {
 	int i, j, k1, k2, tmpint, start1, start2, end1, end2;
-	static int impalloclen = 0;
+	static TLS int impalloclen = 0;
 	float effij;
 	double effijx;
 	char *pt, *pt1, *pt2;
-	static char *nocount1 = NULL;
+	static TLS char *nocount1 = NULL;
 	static char *nocount2 = NULL;
 	LocalHom *tmpptr;
 
@@ -257,18 +257,20 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 {
 #if FASTMATCHCALC
 	int j, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
 	float *matchpt, *cpmxpdpt, **cpmxpdptpt;
 	int *cpmxpdnpt, **cpmxpdnptpt;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 	if( initialize )
 	{
 		int count = 0;
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[l][j] )
 				{
@@ -282,10 +284,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 	}
 
 	{
-		for( l=0; l<26; l++ )
+		for( l=0; l<nalphabets; l++ )
 		{
 			scarr[l] = 0.0;
-			for( j=0; j<26; j++ )
+			for( j=0; j<nalphabets; j++ )
 				scarr[l] += n_dis[j][l] * cpmx1[j][i1];
 		}
 		matchpt = match;
@@ -301,11 +303,14 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 			matchpt++;
 		} 
 	}
+	free( scarr );
 #else
 	int j, k, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 // simple
 	if( initialize )
 	{
@@ -313,7 +318,7 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[l][j] )
 				{
@@ -325,10 +330,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 			cpmxpdn[count][j] = -1;
 		}
 	}
-	for( l=0; l<26; l++ )
+	for( l=0; l<nalphabets; l++ )
 	{
 		scarr[l] = 0.0;
-		for( k=0; k<26; k++ )
+		for( k=0; k<nalphabets; k++ )
 			scarr[l] += n_dis[k][l] * cpmx1[k][i1];
 	}
 	for( j=0; j<lgth2; j++ )
@@ -337,6 +342,7 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		for( k=0; cpmxpdn[k][j]>-1; k++ )
 			match[j] += scarr[cpmxpdn[k][j]] * cpmxpd[k][j];
 	} 
+	free( scarr );
 #endif
 }
 
@@ -581,37 +587,37 @@ float R__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	float *mjpt, *prept, *curpt;
 	int *mpjpt;
 #endif
-	static float mi, *m;
-	static int **ijp;
-	static int mpi, *mp;
-	static float *w1, *w2;
-	static float *match;
-	static float *initverticalw;    /* kufuu sureba iranai */
-	static float *lastverticalw;    /* kufuu sureba iranai */
-	static char **mseq1;
-	static char **mseq2;
-	static char **mseq;
-	static float *digf1;
-	static float *digf2;
-	static float *diaf1;
-	static float *diaf2;
-	static float *gapz1;
-	static float *gapz2;
-	static float *gapf1;
-	static float *gapf2;
-	static float *ogcp1g;
-	static float *ogcp2g;
-	static float *fgcp1g;
-	static float *fgcp2g;
-	static float *ogcp1;
-	static float *ogcp2;
-	static float *fgcp1;
-	static float *fgcp2;
-	static float **cpmx1;
-	static float **cpmx2;
-	static int **intwork;
-	static float **floatwork;
-	static int orlgth1 = 0, orlgth2 = 0;
+	static TLS float mi, *m;
+	static TLS int **ijp;
+	static TLS int mpi, *mp;
+	static TLS float *w1, *w2;
+	static TLS float *match;
+	static TLS float *initverticalw;    /* kufuu sureba iranai */
+	static TLS float *lastverticalw;    /* kufuu sureba iranai */
+	static TLS char **mseq1;
+	static TLS char **mseq2;
+	static TLS char **mseq;
+	static TLS float *digf1;
+	static TLS float *digf2;
+	static TLS float *diaf1;
+	static TLS float *diaf2;
+	static TLS float *gapz1;
+	static TLS float *gapz2;
+	static TLS float *gapf1;
+	static TLS float *gapf2;
+	static TLS float *ogcp1g;
+	static TLS float *ogcp2g;
+	static TLS float *fgcp1g;
+	static TLS float *fgcp2g;
+	static TLS float *ogcp1;
+	static TLS float *ogcp2;
+	static TLS float *fgcp1;
+	static TLS float *fgcp2;
+	static TLS float **cpmx1;
+	static TLS float **cpmx2;
+	static TLS int **intwork;
+	static TLS float **floatwork;
+	static TLS int orlgth1 = 0, orlgth2 = 0;
 	float fpenalty = (float)penalty;
 	float tmppenal;
 	float cumpenal;
@@ -721,15 +727,15 @@ float R__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		fgcp1g = AllocateFloatVec( ll1+2 );
 		fgcp2g = AllocateFloatVec( ll2+2 );
 
-		cpmx1 = AllocateFloatMtx( 26, ll1+2 );
-		cpmx2 = AllocateFloatMtx( 26, ll2+2 );
+		cpmx1 = AllocateFloatMtx( nalphabets, ll1+2 );
+		cpmx2 = AllocateFloatMtx( nalphabets, ll2+2 );
 
 #if FASTMATCHCALC
-		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 26 ); 
-		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 27 ); 
+		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
+		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets+1 ); 
 #else
-		floatwork = AllocateFloatMtx( 26, MAX( ll1, ll2 )+2 ); 
-		intwork = AllocateIntMtx( 26, MAX( ll1, ll2 )+2 ); 
+		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
+		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 #endif
 
 #if DEBUG
