@@ -460,18 +460,20 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 {
 #if FASTMATCHCALC
 	int j, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
 	float *matchpt, *cpmxpdpt, **cpmxpdptpt;
 	int *cpmxpdnpt, **cpmxpdnptpt;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 	if( initialize )
 	{
 		int count = 0;
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[l][j] )
 				{
@@ -485,10 +487,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 	}
 
 	{
-		for( l=0; l<26; l++ )
+		for( l=0; l<nalphabets; l++ )
 		{
 			scarr[l] = 0.0;
-			for( j=0; j<26; j++ )
+			for( j=0; j<nalphabets; j++ )
 //				scarr[l] += n_dis[j][l] * cpmx1[j][i1];
 				scarr[l] += n_dis_consweight_multi[j][l] * cpmx1[j][i1];
 		}
@@ -505,11 +507,14 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 			matchpt++;
 		} 
 	}
+	free( scarr );
 #else
 	int j, k, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 // simple
 	if( initialize )
 	{
@@ -517,7 +522,7 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[l][j] )
 				{
@@ -529,10 +534,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 			cpmxpdn[count][j] = -1;
 		}
 	}
-	for( l=0; l<26; l++ )
+	for( l=0; l<nalphabets; l++ )
 	{
 		scarr[l] = 0.0;
-		for( k=0; k<26; k++ )
+		for( k=0; k<nalphabets; k++ )
 			scarr[l] += n_dis_consweight_multi[k][l] * cpmx1[k][i1];
 //			scarr[l] += n_dis[k][l] * cpmx1[k][i1];
 	}
@@ -542,6 +547,7 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		for( k=0; cpmxpdn[k][j]>-1; k++ )
 			match[j] += scarr[cpmxpdn[k][j]] * cpmxpd[k][j];
 	} 
+	free( scarr );
 #endif
 }
 
@@ -917,11 +923,19 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	static TLS int **intwork;
 	static TLS float **floatwork;
 	static TLS int orlgth1 = 0, orlgth2 = 0;
+	static TLS float *gapfreq1;
+	static TLS float *gapfreq2;
 	float fpenalty = (float)penalty;
 	float *fgcp2pt;
 	float *ogcp2pt;
 	float fgcp1va;
 	float ogcp1va;
+	float *gf2pt;
+	float *gf2ptpre;
+	float gf1va;
+	float gf1vapre;
+	float headgapfreq1;
+	float headgapfreq2;
 
 
 
@@ -957,6 +971,9 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 			FreeFloatMtx( cpmx1 );
 			FreeFloatMtx( cpmx2 );
 
+			FreeFloatVec( gapfreq1 );
+			FreeFloatVec( gapfreq2 );
+
 			FreeFloatMtx( floatwork );
 			FreeIntMtx( intwork );
 
@@ -970,11 +987,12 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
-#if 1
+#if 0
 	if( lgth1 == 0 || lgth2 == 0 )
 	{
 		fprintf( stderr, "WARNING (Aalignmm): lgth1=%d, lgth2=%d\n", lgth1, lgth2 );
 	}
+#endif
 	if( lgth1 == 0 && lgth2 == 0 )
 		return( 0.0 );
 
@@ -984,7 +1002,7 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		{
 			j = lgth2;
 			seq1[i][j] = 0;
-			while( j ) seq1[i][--j] = '-';
+			while( j ) seq1[i][--j] = *newgapstr;
 //			fprintf( stderr, "seq1[i] = %s\n", seq1[i] );
 		}
 		return( 0.0 );
@@ -996,12 +1014,11 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		{
 			j = lgth1;
 			seq2[i][j] = 0;
-			while( j ) seq2[i][--j] = '-';
+			while( j ) seq2[i][--j] = *newgapstr;
 //			fprintf( stderr, "seq2[i] = %s\n", seq2[i] );
 		}
 		return( 0.0 );
 	}
-#endif
 
 
 #if 0
@@ -1048,6 +1065,9 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 			FreeFloatMtx( cpmx1 );
 			FreeFloatMtx( cpmx2 );
 
+			FreeFloatVec( gapfreq1 );
+			FreeFloatVec( gapfreq2 );
+
 			FreeFloatMtx( floatwork );
 			FreeIntMtx( intwork );
 		}
@@ -1076,15 +1096,18 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		fgcp1 = AllocateFloatVec( ll1+2 );
 		fgcp2 = AllocateFloatVec( ll2+2 );
 
-		cpmx1 = AllocateFloatMtx( 26, ll1+2 );
-		cpmx2 = AllocateFloatMtx( 26, ll2+2 );
+		cpmx1 = AllocateFloatMtx( nalphabets, ll1+2 );
+		cpmx2 = AllocateFloatMtx( nalphabets, ll2+2 );
+
+		gapfreq1 = AllocateFloatVec( ll1+2 );
+		gapfreq2 = AllocateFloatVec( ll2+2 );
 
 #if FASTMATCHCALC
-		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 26 ); 
-		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 27 ); 
+		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
+		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets+1 ); 
 #else
-		floatwork = AllocateFloatMtx( 26, MAX( ll1, ll2 )+2 ); 
-		intwork = AllocateIntMtx( 26, MAX( ll1, ll2 )+2 ); 
+		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
+		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 #endif
 
 #if DEBUG
@@ -1153,6 +1176,10 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		new_OpeningGapCount( ogcp2, jcyc, seq2, eff2, lgth2, sgap2 );
 		new_FinalGapCount( fgcp1, icyc, seq1, eff1, lgth1, egap1 );
 		new_FinalGapCount( fgcp2, jcyc, seq2, eff2, lgth2, egap2 );
+		outgapcount( &headgapfreq1, icyc, sgap1, eff1 );
+		outgapcount( &headgapfreq2, jcyc, sgap2, eff2 );
+		outgapcount( gapfreq1+lgth1, icyc, egap1, eff1 );
+		outgapcount( gapfreq2+lgth2, jcyc, egap2, eff2 );
 	}
 	else
 	{
@@ -1160,17 +1187,50 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		st_OpeningGapCount( ogcp2, jcyc, seq2, eff2, lgth2 );
 		st_FinalGapCount( fgcp1, icyc, seq1, eff1, lgth1 );
 		st_FinalGapCount( fgcp2, jcyc, seq2, eff2, lgth2 );
+		headgapfreq1 = 0.0;
+		headgapfreq2 = 0.0;
+		gapfreq1[lgth1] = 0.0;
+		gapfreq2[lgth2] = 0.0;
 	}
 
+	if( legacygapcost == 0 )
+	{
+		gapcountf( gapfreq1, seq1, icyc, eff1, lgth1 );
+		gapcountf( gapfreq2, seq2, jcyc, eff2, lgth2 );
+		for( i=0; i<lgth1+1; i++ ) gapfreq1[i] = 1.0 - gapfreq1[i];
+		for( i=0; i<lgth2+1; i++ ) gapfreq2[i] = 1.0 - gapfreq2[i];
+		headgapfreq1 = 1.0 - headgapfreq1;
+		headgapfreq2 = 1.0 - headgapfreq2;
+	}
+	else
+	{
+		for( i=0; i<lgth1+1; i++ ) gapfreq1[i] = 1.0;
+		for( i=0; i<lgth2+1; i++ ) gapfreq2[i] = 1.0;
+		headgapfreq1 = 1.0;
+		headgapfreq2 = 1.0;
+	}
+
+#if 0
+	fprintf( stderr, "\ngapfreq1[] =" );
+	for( i=0; i<lgth1; i++ ) fprintf( stderr, "%5.2f ", gapfreq1[i] );
+	fprintf( stderr, "\n" );
+
+	fprintf( stderr, "\ngapfreq2[] =" );
+	for( i=0; i<lgth2; i++ ) fprintf( stderr, "%5.2f ", gapfreq2[i] );
+	fprintf( stderr, "\n" );
+#endif
+	
+
 	for( i=0; i<lgth1; i++ ) 
 	{
-		ogcp1[i] = 0.5 * ( 1.0 - ogcp1[i] ) * fpenalty;
-		fgcp1[i] = 0.5 * ( 1.0 - fgcp1[i] ) * fpenalty;
+		ogcp1[i] = 0.5 * ( 1.0 - ogcp1[i] ) * fpenalty * ( gapfreq1[i] );
+		fgcp1[i] = 0.5 * ( 1.0 - fgcp1[i] ) * fpenalty * ( gapfreq1[i] );
 	}
+
 	for( i=0; i<lgth2; i++ ) 
 	{
-		ogcp2[i] = 0.5 * ( 1.0 - ogcp2[i] ) * fpenalty;
-		fgcp2[i] = 0.5 * ( 1.0 - fgcp2[i] ) * fpenalty;
+		ogcp2[i] = 0.5 * ( 1.0 - ogcp2[i] ) * fpenalty * ( gapfreq2[i] );
+		fgcp2[i] = 0.5 * ( 1.0 - fgcp2[i] ) * fpenalty * ( gapfreq2[i] );
 	}
 #if 0
 	for( i=0; i<lgth1; i++ ) 
@@ -1197,11 +1257,13 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	{
 		for( i=1; i<lgth1+1; i++ )
 		{
-			initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] ) ;
+//			initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] ) ;
+			initverticalw[i] += ( ogcp1[0] * headgapfreq2 + fgcp1[i-1] * gapfreq2[0] ) ;
 		}
 		for( j=1; j<lgth2+1; j++ )
 		{
-			currentw[j] += ( ogcp2[0] + fgcp2[j-1] ) ;
+//			currentw[j] += ( ogcp2[0] + fgcp2[j-1] ) ;
+			currentw[j] += ( ogcp2[0] * headgapfreq1 + fgcp2[j-1] * gapfreq1[0] ) ;
 		}
 	}
 #if OUTGAP0TRY
@@ -1228,7 +1290,8 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 
 	for( j=1; j<lgth2+1; ++j ) 
 	{
-		m[j] = currentw[j-1] + ogcp1[1]; mp[j] = 0;
+//		m[j] = currentw[j-1] + ogcp1[1]; mp[j] = 0;
+		m[j] = currentw[j-1] + ogcp1[1] * gapfreq2[j-1]; mp[j] = 0;;
 	}
 	if( lgth2 == 0 )
 		lastverticalw[0] = 0.0; // Falign kara yobaretatoki kounarukanousei ari
@@ -1316,7 +1379,8 @@ fprintf( stderr, "\n" );
 		fprintf( stderr, "\n"  );
 #endif
 	
-		mi = previousw[0] + ogcp2[1]; mpi = 0;
+//		mi = previousw[0] + ogcp2[1]; mpi = 0;
+		mi = previousw[0] + ogcp2[1] * gapfreq1[i-1]; mpi=0;
 		ijppt = ijp[i] + 1;
 		mjpt = m + 1;
 		prept = previousw;
@@ -1326,7 +1390,13 @@ fprintf( stderr, "\n" );
 		ogcp2pt = ogcp2 + 1;
 		fgcp1va = fgcp1[i-1];
 		ogcp1va = ogcp1[i];
+		gf1va = gapfreq1[i];
+		gf1vapre = gapfreq1[i-1];
+		gf2pt = gapfreq2+1;
+		gf2ptpre = gapfreq2;
 		lastj = lgth2+1;
+
+
 		for( j=1; j<lastj; j++ )
 		{
 #ifdef xxxenablemultithread
@@ -1342,17 +1412,17 @@ fprintf( stderr, "\n" );
 			*ijppt = 0;
 
 #if 0
+			fprintf( stderr, "\n i=%d, j=%d %c, %c", i, j, seq1[0][i], seq2[0][j] );
 			fprintf( stderr, "%5.0f->", wm );
+			fprintf( stderr, "%5.0f? (penal=%5.2f)", g=mi+*fgcp2pt*(1.0-gapfreq1[i]), *fgcp2pt*(1.0-gapfreq1[i]) );
 #endif
-#if 0
-			fprintf( stderr, "%5.0f?", g );
-#endif
-			if( (g=mi+*fgcp2pt) > wm )
+			if( (g=mi+*fgcp2pt*gf1va) > wm )
 			{
 				wm = g;
 				*ijppt = -( j - mpi );
+//				fprintf( stderr, "Jump to %d (%c)!", mpi, seq2[0][mpi] );
 			}
-			if( (g=*prept+*ogcp2pt) >= mi )
+			if( (g=*prept+*ogcp2pt*gf1vapre) >= mi )
 			{
 				mi = g;
 				mpi = j-1;
@@ -1362,14 +1432,16 @@ fprintf( stderr, "\n" );
 #endif
 
 #if 0 
-			fprintf( stderr, "%5.0f?", g );
+			fprintf( stderr, "%5.0f->", wm );
+			fprintf( stderr, "%5.0f? (penal=%5.2f)", g=*mjpt+fgcp1va*(1.0-gapfreq2[j]), fgcp1va*(1.0-gapfreq2[j]) );
 #endif
-			if( (g=*mjpt+fgcp1va) > wm )
+			if( (g=*mjpt+ fgcp1va* *gf2pt) > wm )
 			{
 				wm = g;
 				*ijppt = +( i - *mpjpt );
+//				fprintf( stderr, "Jump to %d (%c)!", *mpjpt, seq1[0][*mpjpt] );
 			}
-			if( (g=*prept+ogcp1va) >= *mjpt )
+			if( (g=*prept+ ogcp1va* *gf2ptpre) >= *mjpt )
 			{
 				*mjpt = g;
 				*mpjpt = i-1;
@@ -1388,6 +1460,8 @@ fprintf( stderr, "\n" );
 			mpjpt++;
 			fgcp2pt++;
 			ogcp2pt++;
+			gf2ptpre++;
+			gf2pt++;
 		}
 		lastverticalw[i] = currentw[lgth2-1];
 	}
@@ -1447,436 +1521,7 @@ fprintf( stderr, "\n" );
 float A__align_gapmap( char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, LocalHom ***localhom, float *impmatch, int *gapmap1, int *gapmap2 )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
-//	int k;
-	register int i, j;
-	int lasti, lastj;     /* outgap == 0 -> lgth1, outgap == 1 -> lgth1+1 */
-	int lgth1, lgth2;
-	int resultlen;
-	float wm = 0.0;   /* int ?????? */
-	float g;
-	float *currentw, *previousw;
-//	float fpenalty = (float)penalty;
-#if USE_PENALTY_EX
-	float fpenalty_ex = (float)penalty_ex;
-#endif
-#if 1
-	float *wtmp;
-	int *ijppt;
-	float *mjpt, *prept, *curpt;
-	int *mpjpt;
-#endif
-	static TLS float mi, *m;
-	static TLS int **ijp;
-	static TLS int mpi, *mp;
-	static TLS float *w1, *w2;
-	static TLS float *match;
-	static TLS float *initverticalw;    /* kufuu sureba iranai */
-	static TLS float *lastverticalw;    /* kufuu sureba iranai */
-	static TLS char **mseq1;
-	static TLS char **mseq2;
-	static TLS char **mseq;
-	static TLS float *ogcp1;
-	static TLS float *ogcp2;
-	static TLS float *fgcp1;
-	static TLS float *fgcp2;
-	static TLS float **cpmx1;
-	static TLS float **cpmx2;
-	static TLS int **intwork;
-	static TLS float **floatwork;
-	static TLS int orlgth1 = 0, orlgth2 = 0;
-	float *fgcp2pt;
-	float *ogcp2pt;
-	float fgcp1va;
-	float ogcp1va;
-
-
-#if 0
-	fprintf( stderr, "eff in SA+++align\n" );
-	for( i=0; i<icyc; i++ ) fprintf( stderr, "eff1[%d] = %f\n", i, eff1[i] );
-#endif
-	if( orlgth1 == 0 )
-	{
-		mseq1 = AllocateCharMtx( njob, 0 );
-		mseq2 = AllocateCharMtx( njob, 0 );
-	}
-
-
-	lgth1 = strlen( seq1[0] );
-	lgth2 = strlen( seq2[0] );
-#if 0
-	if( lgth1 == 0 || lgth2 == 0 )
-	{
-		fprintf( stderr, "WARNING (Aalign_gapmap): lgth1=%d, lgth2=%d\n", lgth1, lgth2 );
-	}
-#endif
-
-	if( lgth1 > orlgth1 || lgth2 > orlgth2 )
-	{
-		int ll1, ll2;
-
-		if( orlgth1 > 0 && orlgth2 > 0 )
-		{
-			FreeFloatVec( w1 );
-			FreeFloatVec( w2 );
-			FreeFloatVec( match );
-			FreeFloatVec( initverticalw );
-			FreeFloatVec( lastverticalw );
-
-			FreeFloatVec( m );
-			FreeIntVec( mp );
-
-			FreeCharMtx( mseq );
-
-			FreeFloatVec( ogcp1 );
-			FreeFloatVec( ogcp2 );
-			FreeFloatVec( fgcp1 );
-			FreeFloatVec( fgcp2 );
-
-
-			FreeFloatMtx( cpmx1 );
-			FreeFloatMtx( cpmx2 );
-
-			FreeFloatMtx( floatwork );
-			FreeIntMtx( intwork );
-		}
-
-		ll1 = MAX( (int)(1.3*lgth1), orlgth1 ) + 100;
-		ll2 = MAX( (int)(1.3*lgth2), orlgth2 ) + 100;
-
-#if DEBUG
-		fprintf( stderr, "\ntrying to allocate (%d+%d)xn matrices ... ", ll1, ll2 );
-#endif
-
-		w1 = AllocateFloatVec( ll2+2 );
-		w2 = AllocateFloatVec( ll2+2 );
-		match = AllocateFloatVec( ll2+2 );
-
-		initverticalw = AllocateFloatVec( ll1+2 );
-		lastverticalw = AllocateFloatVec( ll1+2 );
-
-		m = AllocateFloatVec( ll2+2 );
-		mp = AllocateIntVec( ll2+2 );
-
-		mseq = AllocateCharMtx( njob, ll1+ll2 );
-
-		ogcp1 = AllocateFloatVec( ll1+2 );
-		ogcp2 = AllocateFloatVec( ll2+2 );
-		fgcp1 = AllocateFloatVec( ll1+2 );
-		fgcp2 = AllocateFloatVec( ll2+2 );
-
-		cpmx1 = AllocateFloatMtx( 26, ll1+2 );
-		cpmx2 = AllocateFloatMtx( 26, ll2+2 );
-
-#if FASTMATCHCALC
-		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 26 ); 
-		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 26 ); 
-#else
-		floatwork = AllocateFloatMtx( 26, MAX( ll1, ll2 )+2 ); 
-		intwork = AllocateIntMtx( 27, MAX( ll1, ll2 )+2 ); 
-#endif
-
-#if DEBUG
-		fprintf( stderr, "succeeded\n" );
-#endif
-
-		orlgth1 = ll1 - 100;
-		orlgth2 = ll2 - 100;
-	}
-
-
-	for( i=0; i<icyc; i++ )
-	{
-		mseq1[i] = mseq[i];
-		seq1[i][lgth1] = 0;
-	}
-	for( j=0; j<jcyc; j++ )
-	{
-		mseq2[j] = mseq[icyc+j];
-		seq2[j][lgth2] = 0;
-	}
-
-
-	if( orlgth1 > commonAlloc1 || orlgth2 > commonAlloc2 )
-	{
-		int ll1, ll2;
-
-		if( commonAlloc1 && commonAlloc2 )
-		{
-			FreeIntMtx( commonIP );
-		}
-
-		ll1 = MAX( orlgth1, commonAlloc1 );
-		ll2 = MAX( orlgth2, commonAlloc2 );
-
-#if DEBUG
-		fprintf( stderr, "\n\ntrying to allocate %dx%d matrices ... ", ll1+1, ll2+1 );
-#endif
-
-		commonIP = AllocateIntMtx( ll1+10, ll2+10 );
-
-#if DEBUG
-		fprintf( stderr, "succeeded\n\n" );
-#endif
-
-		commonAlloc1 = ll1;
-		commonAlloc2 = ll2;
-	}
-	ijp = commonIP;
-
-	cpmx_calc_new( seq1, cpmx1, eff1, strlen( seq1[0] ), icyc );
-	cpmx_calc_new( seq2, cpmx2, eff2, strlen( seq2[0] ), jcyc );
-
-	st_OpeningGapCount( ogcp1, icyc, seq1, eff1, lgth1  );
-	st_OpeningGapCount( ogcp2, jcyc, seq2, eff2, lgth2 );
-	st_FinalGapCount( fgcp1, icyc, seq1, eff1, lgth1 );
-	st_FinalGapCount( fgcp2, jcyc, seq2, eff2, lgth2 );
-
-	for( i=0; i<lgth1; i++ ) 
-	{
-		ogcp1[i] = 0.5 * ( 1.0 - ogcp1[i] );
-		fgcp1[i] = 0.5 * ( 1.0 - fgcp1[i] );
-	}
-	for( i=0; i<lgth2; i++ ) 
-	{
-		ogcp2[i] = 0.5 * ( 1.0 - ogcp2[i] );
-		fgcp2[i] = 0.5 * ( 1.0 - fgcp2[i] );
-	}
-#if 0
-	for( i=0; i<lgth1; i++ ) 
-		fprintf( stderr, "ogcp1[%d]=%f\n", i, ogcp1[i] );
-#endif
-
-	currentw = w1;
-	previousw = w2;
-
-
-	match_calc( initverticalw, cpmx2, cpmx1, 0, lgth1, floatwork, intwork, 1 );
-	if( localhom )
-		imp_match_out_vead_tate_gapmap( initverticalw, gapmap2[0], lgth1, gapmap1 ); // 060306
-
-
-	match_calc( currentw, cpmx1, cpmx2, 0, lgth2, floatwork, intwork, 1 );
-	if( localhom )
-		imp_match_out_vead_gapmap( currentw, gapmap1[0], lgth2, gapmap2 ); // 060306
-#if 0 // -> tbfast.c
-	if( localhom )
-		imp_match_calc( currentw, icyc, jcyc, lgth1, lgth2, seq1, seq2, eff1, eff2, localhom, 1, 0 );
-
-#endif
-
-	if( 1 ) // tsuneni outgap=1
-	{
-		for( i=1; i<lgth1+1; i++ )
-		{
-			initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] ) ;
-		}
-		for( j=1; j<lgth2+1; j++ )
-		{
-			currentw[j] += ( ogcp2[0] + fgcp2[j-1] ) ;
-		}
-	}
-#if OUTGAP0TRY
-	else
-	{
-		for( j=1; j<lgth2+1; j++ )
-			currentw[j] -= offset * j / 2.0;
-		for( i=1; i<lgth1+1; i++ )
-			initverticalw[i] -= offset * i / 2.0;
-	}
-#endif
-
-	for( j=1; j<lgth2+1; ++j ) 
-	{
-		m[j] = currentw[j-1] + ogcp1[1]; mp[j] = 0;
-	}
-
-	if( lgth2 == 0 )
-		lastverticalw[0] = 0.0;
-	else
-		lastverticalw[0] = currentw[lgth2-1];
-
-	if( 1 ) lasti = lgth1+1; else lasti = lgth1; // tsuneni outgap=1
-
-#if XXXXXXX
-fprintf( stderr, "currentw = \n" );
-for( i=0; i<lgth1+1; i++ )
-{
-	fprintf( stderr, "%5.2f ", currentw[i] );
-}
-fprintf( stderr, "\n" );
-fprintf( stderr, "initverticalw = \n" );
-for( i=0; i<lgth2+1; i++ )
-{
-	fprintf( stderr, "%5.2f ", initverticalw[i] );
-}
-fprintf( stderr, "\n" );
-fprintf( stderr, "fcgp\n" );
-for( i=0; i<lgth1; i++ ) 
-	fprintf( stderr, "fgcp1[%d]=%f\n", i, ogcp1[i] );
-for( i=0; i<lgth2; i++ ) 
-	fprintf( stderr, "fgcp2[%d]=%f\n", i, ogcp2[i] );
-#endif
-
-	for( i=1; i<lasti; i++ )
-	{
-		wtmp = previousw; 
-		previousw = currentw;
-		currentw = wtmp;
-
-		previousw[0] = initverticalw[i-1];
-
-		match_calc( currentw, cpmx1, cpmx2, i, lgth2, floatwork, intwork, 0 );
-#if XXXXXXX
-fprintf( stderr, "\n" );
-fprintf( stderr, "i=%d\n", i );
-fprintf( stderr, "currentw = \n" );
-for( j=0; j<lgth2; j++ )
-{
-	fprintf( stderr, "%5.2f ", currentw[j] );
-}
-fprintf( stderr, "\n" );
-#endif
-		if( localhom )
-		{
-//			fprintf( stderr, "Calling imp_match_calc (o) lgth = %d, i = %d\n", lgth1, i );
-#if  0
-			imp_match_out_vead( currentw, i, lgth2 );
-#else
-			imp_match_out_vead_gapmap( currentw, gapmap1[i], lgth2, gapmap2 );
-#endif
-		}
-#if XXXXXXX
-fprintf( stderr, "\n" );
-fprintf( stderr, "i=%d\n", i );
-fprintf( stderr, "currentw = \n" );
-for( j=0; j<lgth2; j++ )
-{
-	fprintf( stderr, "%5.2f ", currentw[j] );
-}
-fprintf( stderr, "\n" );
-#endif
-		currentw[0] = initverticalw[i];
-
-
-		mi = previousw[0] + ogcp2[1]; mpi = 0;
-
-		ijppt = ijp[i] + 1;
-		mjpt = m + 1;
-		prept = previousw;
-		curpt = currentw + 1;
-		mpjpt = mp + 1;
-		fgcp2pt = fgcp2;
-		ogcp2pt = ogcp2 + 1;
-		fgcp1va = fgcp1[i-1];
-		ogcp1va = ogcp1[i];
-		lastj = lgth2+1;
-		for( j=1; j<lastj; j++ )
-		{
-			wm = *prept;
-			*ijppt = 0;
-
-#if 0
-			fprintf( stderr, "%5.0f->", wm );
-#endif
-			g = mi + *fgcp2pt;
-#if 0
-			fprintf( stderr, "%5.0f?", g );
-#endif
-			if( g > wm )
-			{
-				wm = g;
-				*ijppt = -( j - mpi );
-			}
-			g = *prept + *ogcp2pt;
-			if( g >= mi )
-			{
-				mi = g;
-				mpi = j-1;
-			}
-#if USE_PENALTY_EX
-			mi += fpenalty_ex;
-#endif
-
-			g = *mjpt + fgcp1va;
-#if 0 
-			fprintf( stderr, "%5.0f?", g );
-#endif
-			if( g > wm )
-			{
-				wm = g;
-				*ijppt = +( i - *mpjpt );
-			}
-			g = *prept + ogcp1va;
-			if( g >= *mjpt )
-			{
-				*mjpt = g;
-				*mpjpt = i-1;
-			}
-#if USE_PENALTY_EX
-			m[j] += fpenalty_ex;
-#endif
-
-#if 0
-			fprintf( stderr, "%5.0f ", wm );
-#endif
-			*curpt++ += wm;
-			ijppt++;
-			mjpt++;
-			prept++;
-			mpjpt++;
-			fgcp2pt++;
-			ogcp2pt++;
-		}
-		lastverticalw[i] = currentw[lgth2-1];
-	}
-
-#if OUTGAP0TRY
-	if( !outgap )
-	{
-		for( j=1; j<lgth2+1; j++ )
-			currentw[j] -= offset * ( lgth2 - j ) / 2.0;
-		for( i=1; i<lgth1+1; i++ )
-			lastverticalw[i] -= offset * ( lgth1 - i  / 2.0);
-	}
-#endif
-		
-	/*
-	fprintf( stderr, "\n" );
-	for( i=0; i<icyc; i++ ) fprintf( stderr,"%s\n", seq1[i] );
-	fprintf( stderr, "#####\n" );
-	for( j=0; j<jcyc; j++ ) fprintf( stderr,"%s\n", seq2[j] );
-	fprintf( stderr, "====>" );
-	for( i=0; i<icyc; i++ ) strcpy( mseq1[i], seq1[i] );
-	for( j=0; j<jcyc; j++ ) strcpy( mseq2[j], seq2[j] );
-	*/
-	if( localhom )
-	{
-		Atracking_localhom_gapmap( impmatch, currentw, lastverticalw, seq1, seq2, mseq1, mseq2, cpmx1, cpmx2, ijp, icyc, jcyc, gapmap1, gapmap2 );
-	}
-	else
-		Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, cpmx1, cpmx2, ijp, icyc, jcyc, 1 );
-
-//	fprintf( stderr, "### impmatch = %f\n", *impmatch );
-
-	resultlen = strlen( mseq1[0] );
-	if( alloclen < resultlen || resultlen > N )
-	{
-		fprintf( stderr, "alloclen=%d, resultlen=%d, N=%d\n", alloclen, resultlen, N );
-		ErrorExit( "LENGTH OVER!\n" );
-	}
-
-
-	for( i=0; i<icyc; i++ ) strcpy( seq1[i], mseq1[i] );
-	for( j=0; j<jcyc; j++ ) strcpy( seq2[j], mseq2[j] );
-	/*
-	fprintf( stderr, "\n" );
-	for( i=0; i<icyc; i++ ) fprintf( stderr, "%s\n", mseq1[i] );
-	fprintf( stderr, "#####\n" );
-	for( j=0; j<jcyc; j++ ) fprintf( stderr, "%s\n", mseq2[j] );
-	*/
-
-//	fprintf( stderr, "wm = %f\n", wm );
-
-	return( wm );
+	fprintf( stderr, "Unexpected error.  Please contact kazutaka.katoh@aist.go.jp\n" );
+	exit( 1 );
 }
 
