@@ -1,5 +1,7 @@
 #include "mltaln.h"
 
+#define SMALLMEMORY 1
+
 #define DEBUG 0
 #define IODEBUG 0
 #define SCOREOUT 0
@@ -48,6 +50,7 @@ typedef struct _thread_arg
 	Treedep *dep;
 	int  ***topol;
 	float  **len;
+	Addtree *addtree;
 #ifdef enablemultithread
 	int *iaddshare;
 	int thread_no;
@@ -138,7 +141,338 @@ static void countnewres( int len, Blocktorealign *realign, int *posmap, int *gap
 		regstart = posmap[i]+1;
 	}
 }
+static void fillgap( char *s, int len )
+{
+	int orilen = strlen( s );
+	s += orilen;
+	len -= orilen;
+	while( len-- )
+		*s++ = '-';
+	*s = 0;
+}
 
+static int lencomp( const void *a, const void *b ) // osoikamo
+{
+	char **ast = (char **)a;
+	char **bst = (char **)b;
+	int lena = strlen( *ast );
+	int lenb = strlen( *bst );
+//	fprintf( stderr, "a=%s, b=%s\n", *ast, *bst );
+//	fprintf( stderr, "lena=%d, lenb=%d\n", lena, lenb );
+	if( lena > lenb ) return -1;
+	else if( lena < lenb ) return 1;
+	else return( 0 );
+}
+
+static int dorealignment_tree( Blocktorealign *block, char **fullseq, int *fullseqlenpt, int norg, int ***topol, int *follows )
+{
+	int i, j, k, posinold, newlen, *nmem;
+	int n0, n1, localloclen, nhit, hit1, hit2;
+	int *pickhistory;
+	int nprof1, nprof2, pos, zure;
+	char **prof1, **prof2;
+	int *iinf0, *iinf1;
+	int *group, *nearest, *g2n, ngroup;
+	char ***mem;
+	static char **tmpaln0 = NULL;
+	static char **tmpaln1 = NULL;
+	static char **tmpseq;
+	int ***topolpick;
+	int *tmpint;
+	int *intptr, *intptrx;
+	char *tmpseq0, *cptr, **cptrptr;
+
+
+	localloclen = 4 * ( block->end - block->start + 1 );	 // ookisugi?
+	tmpaln0 = AllocateCharMtx( njob, localloclen );
+	tmpaln1 = AllocateCharMtx( njob, localloclen );
+	tmpseq = AllocateCharMtx( 1, *fullseqlenpt * 4 );
+	iinf0 = AllocateIntVec( njob );
+	iinf1 = AllocateIntVec( njob );
+	nearest = AllocateIntVec( njob ); // oosugi
+
+	posinold = block->start;
+
+	n0 = 0;
+	n1 = 0;
+	for( i=0; i<njob; i++ )
+	{
+		strncpy( tmpseq[0], fullseq[i] + block->start, block->end - block->start + 1 );
+		tmpseq[0][block->end - block->start + 1] = 0;
+		commongappick( 1, tmpseq );
+		if( tmpseq[0][0] != 0 )
+		{
+			if( i < norg )
+			{
+				fprintf( stderr, "BUG!!!!\n" );
+				exit( 1 );
+			}
+			strcpy( tmpaln0[n0], tmpseq[0] );
+			iinf0[n0] = i;
+			nearest[n0] = follows[i-norg];
+			n0++;
+		}
+		else
+		{
+			strcpy( tmpaln1[n0], "" );
+			iinf1[n1] = i;
+			n1++;
+		}
+	}
+	mem = AllocateCharCub( n0, n0+1, 0 ); // oosugi
+	nmem = AllocateIntVec( n0 ); // oosugi
+	g2n = AllocateIntVec( n0 ); // oosugi
+	group = AllocateIntVec( n0 ); // oosugi
+	for( i=0; i<n0; i++ ) mem[i][0] = NULL;
+	for( i=0; i<n0; i++ ) nmem[i] = 0;
+	ngroup = 0;
+	for( i=0; i<n0; i++ )
+	{
+		for( j=0; j<i; j++ ) if( nearest[j] == nearest[i] ) break;
+		if( j == i ) group[i] = ngroup++;
+		else group[i] = group[j];
+
+		for( j=0; mem[group[i]][j]; j++ )
+			;
+		mem[group[i]][j] = tmpaln0[i];
+		mem[group[i]][j+1] = NULL;
+		nmem[group[i]]++;
+		g2n[group[i]] = nearest[i];
+//		fprintf( stderr, "%d -> %d -> group%d\n", i, nearest[i], group[i] );
+//		fprintf( stderr, "mem[%d][%d] = %s\n", group[i], j, mem[group[i]][j] );
+	}
+
+	for( i=0; i<ngroup; i++ )
+	{
+//		fprintf( stderr, "before sort:\n" );
+//		for( j=0; j<nmem[i]; j++ ) fprintf( stderr, "%s\n", mem[i][j] );
+//		fprintf( stderr, "\n" );
+		qsort( mem[i], nmem[i], sizeof( char * ), lencomp );
+//		fprintf( stderr, "after sort:\n" );
+//		for( j=0; j<nmem[i]; j++ ) fprintf( stderr, "%s\n", mem[i][j] );
+//		fprintf( stderr, "\n" );
+	}
+
+#if 0
+	for( i=1; i<n0; i++ )
+	{
+		profilealignment( 1, n1, i, tmpaln0+i, tmpaln1, tmpaln0, localloclen, alg );
+	}
+	newlen = strlen( tmpaln0[0] );
+	for( i=0; i<n1; i++ ) eq2dash( tmpaln1[i] );
+#else
+//	newlen = 0;
+	for( i=0; i<ngroup; i++ )
+	{
+//		for( k=0; mem[i][k]; k++ ) fprintf( stderr, "mem[%d][%d] = %s\n", i, j, mem[i][k] );
+
+		for( j=1; j<nmem[i]; j++ )
+		{
+			profilealignment2( 1, j, mem[i]+j, mem[i], localloclen, alg );
+		}
+//		for( j=0; j<nmem[i]; j++ ) fprintf( stderr, "j=%d, %s\n", j, mem[i][j] );
+
+#if 0 // iru
+		if( ( j = strlen( mem[i][0] ) ) > newlen ) newlen = j;
+		for( j=0; j<=i; j++ )
+		{
+			for( k=0; mem[j][k]; k++ )
+				fillgap( mem[j][k], newlen );
+		}
+#endif
+
+	}
+#if 0
+	fprintf( stderr, "After ingroupalignment (original order):\n" );
+	for( i=0; i<n0; i++ ) fprintf( stderr, "%s\n", tmpaln0[i] );
+#endif
+#endif
+
+	topolpick = AllocateIntCub( ngroup, 2, ngroup );
+	pickhistory = AllocateIntVec( ngroup );
+	tmpint = AllocateIntVec( 2 );
+	prof1 = AllocateCharMtx( n0, 0 );
+	prof2 = AllocateCharMtx( n0, 0 );
+	for( i=0; i<ngroup; i++ )
+	{
+		topolpick[i][0][0] = -1;
+		topolpick[i][1][0] = -1;
+		pickhistory[i] = -1;
+	}
+
+	nhit = 0;
+	for( i=0; i<norg-1; i++ )
+	{
+		for( intptr=topol[i][0]; *intptr>-1; intptr++ )
+		{
+			for( intptrx=g2n,k=0; k<ngroup; intptrx++,k++ ) 
+			{
+				if( *intptr == *intptrx )
+				{
+					hit1 = k;
+					goto exitloop1;
+				}
+			}
+		}
+		continue; 
+		exitloop1:
+//		fprintf( stderr, "hit1! group%d -> %d\n", k, topol[i][0][j] );
+
+		for( intptr=topol[i][1]; *intptr>-1; intptr++ )
+		{
+			for( intptrx=g2n,k=0; k<ngroup; intptrx++,k++ ) 
+			{
+				if( *intptr == *intptrx )
+				{
+					hit2 = k;
+					goto exitloop2;
+				}
+			}
+		}
+		continue; 
+		exitloop2:
+
+		if( pickhistory[hit1] == -1 )
+		{
+			topolpick[nhit][0][0] = hit1;
+			topolpick[nhit][0][1] = -1;
+		}
+		else
+		{
+			intcpy( topolpick[nhit][0], topolpick[pickhistory[hit1]][0] );
+			intcat( topolpick[nhit][0], topolpick[pickhistory[hit1]][1] );
+		}
+		if( pickhistory[hit2] == -1 )
+		{
+			topolpick[nhit][1][0] = hit2;
+			topolpick[nhit][1][1] = -1;
+		}
+		else
+		{
+			intcpy( topolpick[nhit][1], topolpick[pickhistory[hit2]][0] );
+			intcat( topolpick[nhit][1], topolpick[pickhistory[hit2]][1] );
+		}
+
+		pickhistory[hit1] = nhit;
+		pickhistory[hit2] = nhit;
+		nhit++;
+//		g2n[hit1] = -1;
+//		g2n[hit2] = -1;
+
+//		fprintf( stderr, "hit2! group%d -> %d\n", k, topol[i][1][j] );
+
+#if 0
+		fprintf( stderr, "\nHIT!!! \n" );
+		fprintf( stderr, "\nSTEP %d\n", i );
+		for( j=0; topol[i][0][j]>-1; j++ ) fprintf( stderr, "%3d ", topol[i][0][j] );
+		fprintf( stderr, "\n" );
+		for( j=0; topol[i][1][j]>-1; j++ ) fprintf( stderr, "%3d ", topol[i][1][j] );
+		fprintf( stderr, "\n" );
+#endif
+	}
+
+	for( i=0; i<ngroup-1; i++ )
+	{
+#if 0
+		fprintf( stderr, "\npickSTEP %d\n", i );
+		for( j=0; topolpick[i][0][j]>-1; j++ ) fprintf( stderr, "%3d ", topolpick[i][0][j] );
+		fprintf( stderr, "\n" );
+		for( j=0; topolpick[i][1][j]>-1; j++ ) fprintf( stderr, "%3d ", topolpick[i][1][j] );
+		fprintf( stderr, "\n" );
+#endif
+
+		pos = 0;
+//		for( j=0; topolpick[i][0][j]>-1; j++ ) for( k=0; (cptr=mem[topolpick[i][0][j]][k]); k++ ) prof1[pos++] = cptr;
+		for( intptr=topolpick[i][0]; *intptr>-1; intptr++ ) 
+			for( cptrptr=mem[*intptr]; (cptr=*cptrptr); cptrptr++ ) 
+				prof1[pos++] = cptr;
+		nprof1 = pos;
+		pos = 0;
+//		for( j=0; topolpick[i][1][j]>-1; j++ ) for( k=0; (cptr=mem[topolpick[i][1][j]][k]); k++ ) prof2[pos++] = cptr;
+		for( intptr=topolpick[i][1]; *intptr>-1; intptr++ ) 
+			for( cptrptr=mem[*intptr]; (cptr=*cptrptr); cptrptr++ ) 
+				prof2[pos++] = cptr;
+		nprof2 = pos;
+
+
+		profilealignment2( nprof1, nprof2, prof1, prof2, localloclen, alg );
+#if 0
+		for( j=0; j<nprof1; j++ ) fprintf( stderr, "prof1[%d] = %s\n", j, prof1[j] );
+		for( j=0; j<nprof2; j++ ) fprintf( stderr, "prof2[%d] = %s\n", j, prof2[j] );
+		fprintf( stderr, "done.\n" );
+#endif
+	}
+	newlen = strlen( tmpaln0[0] );
+	for( j=0; j<n1; j++ ) fillgap( tmpaln1[j], newlen );
+
+#if 0
+	fprintf( stderr, "After rerealignment (original order):\n" );
+	for( i=0; i<n0; i++ ) fprintf( stderr, "%s\n", tmpaln0[i] );
+#endif
+
+//	newlen = strlen( tmpaln0[0] );
+	zure = ( block->end - block->start + 1 - newlen );
+//	fprintf( stderr, "zure = %d, localloclen=%d, newlen=%d\n", zure, localloclen, newlen );
+
+
+	if( *fullseqlenpt < strlen( fullseq[0] ) - (block->end-block->start+1)  + newlen + 1 )
+	{
+		*fullseqlenpt = strlen( fullseq[0] ) * 2;
+		fprintf( stderr, "reallocating..." );
+		for( i=0; i<njob; i++ )
+		{
+			fullseq[i] = realloc( fullseq[i], *fullseqlenpt * sizeof( char ) );
+			if( !fullseq[i] )
+			{
+				fprintf( stderr, "Cannot reallocate seq[][]\n" );
+				exit( 1 );
+			}
+		}
+		fprintf( stderr, "done.\n" );
+	}
+
+
+	tmpseq0 = tmpseq[0];
+	posinold = block->end+1;
+	for( i=0; i<n0; i++ ) 
+	{
+		strncpy( tmpseq0, tmpaln0[i], newlen );
+		strcpy( tmpseq0+newlen, fullseq[iinf0[i]] + posinold );
+		strcpy( fullseq[iinf0[i]]+block->start, tmpseq0 );
+	}
+	for( i=0; i<n1; i++ ) 
+	{
+//		eq2dash( tmpaln1[i] );
+		strncpy( tmpseq0, tmpaln1[i], newlen );
+		strcpy( tmpseq0+newlen, fullseq[iinf1[i]] + posinold );
+		strcpy( fullseq[iinf1[i]]+block->start, tmpseq0 );
+	}
+	FreeCharMtx( tmpaln0 );
+	FreeCharMtx( tmpaln1 );
+	FreeCharMtx( tmpseq );
+	for( i=0; i<n0; i++ ) 
+	{
+//		for( j=0; j<njob; j++ ) free( mem[i][j] );
+		free( mem[i] );
+	}
+	free( mem );
+	free( nmem );
+	free( iinf0 );
+	free( iinf1 );
+	free( group );
+	free( g2n );
+	free( prof1 );
+	free( prof2 );
+	free( nearest );
+	FreeIntCub( topolpick );
+	free( pickhistory );
+	free( tmpint );
+
+	return( zure );
+}
+
+
+#if 0
 static int dorealignment( Blocktorealign *block, char **fullseq, int alloclen, int fullseqlen, int norg )
 {
 	int i, posinnew, posinold, newlen;
@@ -160,7 +494,6 @@ static int dorealignment( Blocktorealign *block, char **fullseq, int alloclen, i
 	}
 	posinold = block->start;
 
-	fprintf( stderr, "\rRealigning %d - %d          \r", block->start, block->end );
 
 	n0 = 0;
 	n1 = 0;
@@ -195,10 +528,10 @@ static int dorealignment( Blocktorealign *block, char **fullseq, int alloclen, i
 		profilealignment( 1, n1, i, tmpaln0+i, tmpaln1, tmpaln0, alloclen, alg ); // n1 ha allgap
 	}
 
-#if 0
+#if 1
 	fprintf( stderr, "After realignment:\n" );
 	for( i=0; i<n0; i++ ) fprintf( stderr, "%s\n", tmpaln0[i] );
-	for( i=0; i<n1; i++ ) fprintf( stderr, "%s\n", tmpaln1[i] );
+//	for( i=0; i<n1; i++ ) fprintf( stderr, "%s\n", tmpaln1[i] );
 #endif
 
 	newlen = strlen( tmpaln0[0] );
@@ -217,15 +550,19 @@ static int dorealignment( Blocktorealign *block, char **fullseq, int alloclen, i
 
 	for( i=0; i<njob; i++ ) 
 	{
-//		strcpy( fullseq[i]+posinnew, fullseq[i]+posinold ); // ??
+#if 0
+		strcpy( fullseq[i]+posinnew, fullseq[i]+posinold ); // ??
+#else
 		opt = fullseq[i] + posinold;
 		npt = fullseq[i] + posinnew;
 		while( ( *npt++ = *opt++ ) );
 		*npt = 0;
+#endif
 	}
 
 	return( zure );
 }
+#endif
 
 static void adjustposmap( int len, int *posmap, int *gaplist )
 {
@@ -237,6 +574,17 @@ static void adjustposmap( int len, int *posmap, int *gaplist )
 	zure = 0;
 	mpt1 = newposmap;
 	mpt2 = posmap;
+
+#if 0
+	int i;
+	fprintf( stderr, "posmapa = " );
+	for( i=0; i<len+2; i++ )
+	{
+		fprintf( stderr, "%3d ", posmap[i] );
+	}
+	fprintf( stderr, "\n" );
+#endif
+
 	while( len-- ) 
 	{
 		zure += *gaplist++;
@@ -251,9 +599,17 @@ static void adjustposmap( int len, int *posmap, int *gaplist )
 	while( len-- ) *mpt2++ = *mpt1++;
 	*mpt2 = *mpt1;
 	free( newposmap );
+#if 0
+	fprintf( stderr, "posmapm = " );
+	for( i=0; i<lenbk+2; i++ )
+	{
+		fprintf( stderr, "%3d ", posmap[i] );
+	}
+	fprintf( stderr, "\n" );
+#endif
 }
 
-static int insertgapsbyotherfragments_simple( int len, char *a, char *s, int *l, int *p )
+static int insertgapsbyotherfragments_compact( int len, char *a, char *s, int *l, int *p )
 {
 	int gaplen;
 	int i, pos, pi;
@@ -264,8 +620,14 @@ static int insertgapsbyotherfragments_simple( int len, char *a, char *s, int *l,
 	{
 		gaplen = l[i];
 		pi = p[i];
-		while( gaplen-- ) *a++ = *s++;
 		pos = prevp + 1;
+//		fprintf( stderr, "gaplen = %d\n", gaplen );
+		while( gaplen-- )
+		{
+			pos++;
+			*a++ = *s++;
+		}
+//		fprintf( stderr, "pos = %d, pi = %d\n", pos, pi );
 		while( pos++ < pi )
 		{
 			*a++ = '=';
@@ -276,8 +638,12 @@ static int insertgapsbyotherfragments_simple( int len, char *a, char *s, int *l,
 	}
 	gaplen = l[i];
 	pi = p[i];
-	while( gaplen-- ) *a++ = *s++;
 	pos = prevp + 1;
+	while( gaplen-- )
+	{
+		pos++;
+		*a++ = *s++;
+	}
 	while( pos++ < pi )
 	{
 		*a++ = '=';
@@ -287,6 +653,33 @@ static int insertgapsbyotherfragments_simple( int len, char *a, char *s, int *l,
 	return( realignment );
 }
 
+void makegaplistcompact( int len, int *p, int *c, int *l )
+{
+	int i;
+	int pg;
+	int prep = -1;
+	for( i=0; i<len+2; i++ )
+	{
+		if( ( pg = p[i]-prep-1) > 0 && l[i] > 0 )
+		{
+			if( pg < l[i] ) 
+			{
+				c[i] = l[i] - pg;
+			}
+			else
+			{
+				c[i] = 0;
+			}
+		}
+		else
+		{
+			c[i] = l[i];
+		}
+		prep = p[i];
+	}
+}
+
+
 void gaplist2alnx( int len, char *a, char *s, int *l, int *p, int lenlimit )
 {
 	int gaplen;
@@ -296,7 +689,23 @@ void gaplist2alnx( int len, char *a, char *s, int *l, int *p, int lenlimit )
 	char *sp;
 //	char *abk = a;
 
-//	for( i=0; i<len; i++ )
+#if 0
+	int i;
+	char *abk = a;
+	fprintf( stderr, "s = %s\n", s );
+	fprintf( stderr, "posmap  = " );
+	for( i=0; i<len+2; i++ )
+	{
+		fprintf( stderr, "%3d ", p[i] );
+	}
+	fprintf( stderr, "\n" );
+	fprintf( stderr, "gaplist = " );
+	for( i=0; i<len+2; i++ )
+	{
+		fprintf( stderr, "%3d ", l[i] );
+	}
+	fprintf( stderr, "\n" );
+#endif
 	while( len-- )
 	{
 		gaplen = *l++;
@@ -352,6 +761,7 @@ void gaplist2alnx( int len, char *a, char *s, int *l, int *p, int lenlimit )
 	}
 	*a = 0;
 //	fprintf( stderr, "reslen = %d, strlen(a) = %d\n", reslen, strlen( abk ) );
+//	fprintf( stderr, "a = %s\n", abk );
 }
 
 static void makenewgaplist( int *l, char *a )
@@ -435,6 +845,7 @@ void arguments( int argc, char *argv[] )
 	nadd = 0;
 	multidist = 0;
 	tuplesize = -1;
+	legacygapcost = 0;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -448,7 +859,7 @@ void arguments( int argc, char *argv[] )
 					--argc;
                     goto nextoption;
 				case 'I':
-					nadd = atoi( *++argv );
+					nadd = myatoi( *++argv );
 					fprintf( stderr, "nadd = %d\n", nadd );
 					--argc;
 					goto nextoption;
@@ -476,25 +887,25 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'k':
-					kimuraR = atoi( *++argv );
+					kimuraR = myatoi( *++argv );
 					fprintf( stderr, "kappa = %d\n", kimuraR );
 					--argc;
 					goto nextoption;
 				case 'b':
-					nblosum = atoi( *++argv );
+					nblosum = myatoi( *++argv );
 					scoremtx = 1;
 					fprintf( stderr, "blosum %d / kimura 200\n", nblosum );
 					--argc;
 					goto nextoption;
 				case 'j':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
 					fprintf( stderr, "jtt/kimura %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'm':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
 					fprintf( stderr, "tm %d\n", pamN );
@@ -515,7 +926,7 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'C':
-					nthread = atoi( *++argv );
+					nthread = myatoi( *++argv );
 					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
 					goto nextoption;
@@ -632,7 +1043,7 @@ void arguments( int argc, char *argv[] )
 					multidist = 1;
 					break;
 				case 'W':
-					tuplesize = atoi( *++argv );
+					tuplesize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 #if 0
@@ -646,16 +1057,19 @@ void arguments( int argc, char *argv[] )
 					break;
 /* modification end. */
 				case 'z':
-					fftThreshold = atoi( *++argv );
+					fftThreshold = myatoi( *++argv );
 					--argc; 
 					goto nextoption;
 				case 'w':
-					fftWinSize = atoi( *++argv );
+					fftWinSize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'Z':
 					checkC = 1;
 					break;
+				case 'L':
+					legacygapcost = 1;
+					break;
                 default:
                     fprintf( stderr, "illegal option %c\n", c );
                     argc = 0;
@@ -726,15 +1140,20 @@ static float treebase( int nseq, int *nlen, char **aseq, int nadd, char *mergeor
 	indication1 = AllocateCharVec( 150 );
 	indication2 = AllocateCharVec( 150 );
 	alreadyaligned = AllocateIntVec( nseq );
-#if 0
-#else
 	if( constraint )
 	{
 		localhomshrink = (LocalHom ***)calloc( nseq, sizeof( LocalHom ** ) );
-		for( i=0; i<nseq; i++)
-			localhomshrink[i] = (LocalHom **)calloc( nseq, sizeof( LocalHom *) );
-	}
+#if SMALLMEMORY
+		if( multidist )
+		{
+			for( i=0; i<nseq; i++) localhomshrink[i] = (LocalHom **)calloc( 1, sizeof( LocalHom *) );
+		}
+		else
 #endif
+		{
+			for( i=0; i<nseq; i++) localhomshrink[i] = (LocalHom **)calloc( nseq, sizeof( LocalHom *) );
+		}
+	}
 	effarr1_kozo = AllocateDoubleVec( nseq ); //tsuneni allocate sareru.
 	effarr2_kozo = AllocateDoubleVec( nseq ); //tsuneni allocate sareru.
 	for( i=0; i<nseq; i++ ) effarr1_kozo[i] = 0.0;
@@ -748,7 +1167,17 @@ static float treebase( int nseq, int *nlen, char **aseq, int nadd, char *mergeor
 	for( l=0; l<nseq; l++ ) fftlog[l] = 1;
 
 
-	if( constraint ) calcimportance( nseq, effarr, aseq, localhomtable );
+	if( constraint ) 
+	{
+#if SMALLMEMORY
+		if( multidist )
+			dontcalcimportance_firstone( nseq, effarr, aseq, localhomtable );
+		else
+			calcimportance( nseq, effarr, aseq, localhomtable );
+#else
+		calcimportance( nseq, effarr, aseq, localhomtable );
+#endif
+	}
 
 	tscore = 0.0;
 	for( l=0; l<nseq-1; l++ )
@@ -852,7 +1281,17 @@ static float treebase( int nseq, int *nlen, char **aseq, int nadd, char *mergeor
 
 		if( constraint )
 		{
-			fastshrinklocalhom( topol[l][0], topol[l][1], localhomtable, localhomshrink );
+#if SMALLMEMORY
+			if( multidist )
+			{
+				fastshrinklocalhom_one( topol[l][0], topol[l][1], nseq-1, localhomtable, localhomshrink );
+			}
+			else
+#endif
+			{
+				fastshrinklocalhom( topol[l][0], topol[l][1], localhomtable, localhomshrink );
+			}
+
 //			msfastshrinklocalhom( topol[l][0], topol[l][1], localhomtable, localhomshrink );
 //			fprintf( stdout, "localhomshrink =\n" );
 //			outlocalhompt( localhomshrink, clus1, clus2 );
@@ -885,10 +1324,11 @@ static float treebase( int nseq, int *nlen, char **aseq, int nadd, char *mergeor
 			fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode.\n", len1, len2 );
 			alg = 'M';
 			if( commonIP ) FreeIntMtx( commonIP );
+			commonIP = NULL; // 2013/Jul17
 			commonAlloc1 = 0;
 			commonAlloc2 = 0;
 		}
-		
+
 
 //		if( fftlog[m1] && fftlog[m2] ) ffttry = ( nlen[m1] > clus1 && nlen[m2] > clus2 );
 		if( fftlog[m1] && fftlog[m2] ) ffttry = ( nlen[m1] > clus1 && nlen[m2] > clus2 && clus1 < 1000 && clus2 < 1000 );
@@ -968,6 +1408,7 @@ static float treebase( int nseq, int *nlen, char **aseq, int nadd, char *mergeor
 			}
 		}
 
+
 		nlen[m1] = 0.5 * ( nlen[m1] + nlen[m2] );
 
 //		fprintf( stderr, "aseq[last] = %s\n", aseq[nseq-1] );
@@ -1077,7 +1518,6 @@ static void	*addsinglethread( void *arg )
 		int ***topolc;
 		float **lenc;
 		LocalHom **localhomtablec = NULL;
-		int *ordertablec;
 		int *memlist0;
 		int *memlist1;
 		int *addmem;
@@ -1107,6 +1547,7 @@ static void	*addsinglethread( void *arg )
 		Treedep *dep = targ->dep;
 		int ***topol = targ->topol;
 		float **len = targ->len;
+		Addtree *addtree = targ->addtree;
 		float pscore;
 
 
@@ -1122,11 +1563,20 @@ static void	*addsinglethread( void *arg )
 		namec = AllocateCharMtx( njob, 0 );
 		nlenc = AllocateIntVec( njob );
 		mergeoralign = AllocateCharVec( njob );
-		ordertablec = AllocateIntVec( njobc+1 );
 		if( constraint )
 		{
-			localhomtablec = (LocalHom **)calloc( njobc, sizeof( LocalHom *) );
-			for( i=0; i<njobc; i++) localhomtablec[i] = (LocalHom *)calloc( njobc, sizeof( LocalHom ) );
+			localhomtablec = (LocalHom **)calloc( njobc, sizeof( LocalHom *) ); // motto chiisaku dekiru.
+#if SMALLMEMORY
+			if( multidist )
+			{
+				for( i=0; i<njobc; i++) localhomtablec[i] = (LocalHom *)calloc( 1, sizeof( LocalHom ) ); // motto chiisaku dekiru.
+			}
+			else
+#endif
+			{
+				for( i=0; i<njobc; i++) localhomtablec[i] = (LocalHom *)calloc( njobc, sizeof( LocalHom ) ); // motto chiisaku dekiru.
+				for( i=0; i<norg; i++ ) for( j=0; j<norg; j++ ) localhomtablec[i][j] = localhomtable[i][j]; // iru!
+			}
 		}
 
 
@@ -1136,7 +1586,6 @@ static void	*addsinglethread( void *arg )
 //		for( i=0; i<norg; i++ ) nlenc[i] = strlen( seq[i] );
 		for( i=0; i<norg; i++ ) nlenc[i] = nlen[i];
 		for( i=0; i<norg; i++ ) namec[i] = name[i];
-		if( constraint ) for( i=0; i<norg; i++ ) for( j=0; j<norg; j++ ) localhomtablec[i][j] = localhomtable[i][j];
 		memlist0 = AllocateIntVec( norg+1 );
 		memlist1 = AllocateIntVec( 2 );
 		for( i=0; i<norg; i++ ) memlist0[i] = i;
@@ -1177,14 +1626,14 @@ static void	*addsinglethread( void *arg )
 
 			mtxcpy( norg, njobc, &iscorec, iscore );
 	
-//			if( tuplesize > 0 ) // Tsuneni
+			if( multidist || tuplesize > 0 )
 			{
 				for( i=0; i<norg; i++ ) iscorec[i][norg-i] = nscore[i][iadd];
 			}
-//			else
-//			{
-//				for( i=0; i<norg; i++ ) iscorec[i][norg-i] = iscore[i][norg+iadd-i];
-//			}
+			else
+			{
+				for( i=0; i<norg; i++ ) iscorec[i][norg-i] = iscore[i][norg+iadd-i];
+			}
 
 
 #if 0
@@ -1203,10 +1652,20 @@ static void	*addsinglethread( void *arg )
 			{
 				for( i=0; i<norg; i++ )
 				{
-					localhomtablec[i][norg] = localhomtable[i][norg+iadd];
-					localhomtablec[norg][i] = localhomtable[norg+iadd][i];
+#if SMALLMEMORY
+					if( multidist )
+					{
+						localhomtablec[i][0] = localhomtable[i][iadd];
+//						localhomtablec[norg][i] = localhomtable[norg+iadd][i];
+					}
+					else
+#endif
+					{
+						localhomtablec[i][norg] = localhomtable[i][norg+iadd];
+						localhomtablec[norg][i] = localhomtable[norg+iadd][i];
+					}
 				}
-				localhomtablec[norg][norg] = localhomtable[norg+iadd][norg+iadd];
+//				localhomtablec[norg][norg] = localhomtable[norg+iadd][norg+iadd]; // iranai!!
 			}
 	
 //			fprintf( stderr, "Constructing a UPGMA tree %d ... ", iadd );
@@ -1217,9 +1676,13 @@ static void	*addsinglethread( void *arg )
 //			{
 //			}
 //			fixed_musclesupg_float_realloc_nobk_halfmtx( njobc, iscorec, topolc, lenc, depc, 0 );
-			addonetip( njobc, topolc, lenc, iscorec, topol, len, dep, treeout, iadd, name );
+			neighbor = addonetip( njobc, topolc, lenc, iscorec, topol, len, dep, treeout, addtree, iadd, name );
 
-			if( noalign ) continue;
+			if( noalign ) 
+			{
+				FreeFloatHalfMtx( iscorec, njobc );
+				continue;
+			}
 	
 			if( tbrweight )
 			{
@@ -1235,19 +1698,17 @@ static void	*addsinglethread( void *arg )
 //			FreeFloatMtx( lenc );
 		
 #if 0
-			fprintf( stderr, "topol = \n" );
-			for( j=0; topolc[njobc-2][0][j]!=-1; j++ ) fprintf( stderr, "%d ", topolc[njobc-2][0][j] );
-			fprintf( stderr, "\n" );
-			for( j=0; topolc[njobc-2][1][j]!=-1; j++ ) fprintf( stderr, "%d ", topolc[njobc-2][1][j] );
-			fprintf( stderr, "\n" );
-#endif
+			for( i=0; i<njobc-1; i++ )
+			{
+				fprintf( stderr, "topol[%d] = \n", i );
+				for( j=0; topolc[i][0][j]!=-1; j++ ) fprintf( stderr, "%d ", topolc[i][0][j] );
+				fprintf( stderr, "\n" );
+				for( j=0; topolc[i][1][j]!=-1; j++ ) fprintf( stderr, "%d ", topolc[i][1][j] );
+				fprintf( stderr, "\n" );
+			}
 
-			cnctintvec( ordertablec, topolc[njobc-2][0], topolc[njobc-2][1] );
-//			fprintf( stderr, "searching neighbor in topolc[][0]\n" );
-			for( j=0; ordertablec[j]!=norg; j++ )
-				;
-			neighbor = ordertablec[j-1];
-//			fprintf( stderr, "neighbor = %d, iadd = %d\n", neighbor, iadd );
+			fprintf( stderr, "\nneighbor = %d, iadd = %d\n", neighbor, iadd );
+#endif
 			follows[iadd] = neighbor;
 
 			for( i=0; i<njobc-1; i++ ) mergeoralign[i] = 'n';
@@ -1361,6 +1822,8 @@ static void	*addsinglethread( void *arg )
 			eq2dash( seq[norg+iadd] );
 
 		}
+
+
 		if( constraint )
 		{
 			for( i=0; i<njobc; i++ ) free( localhomtablec[i] );
@@ -1379,7 +1842,6 @@ static void	*addsinglethread( void *arg )
 		free( addmem );
 		free( mseq1 );
 		free( mseq2 );
-		free( ordertablec );
 		Falign( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
 		A__align( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
 		if( commonIP ) FreeIntMtx( commonIP );
@@ -1392,6 +1854,7 @@ static void	*addsinglethread( void *arg )
 
 static int maxl;
 static int tsize;
+static int nunknown = 0;
 
 void seq_grp_nuc( int *grp, char *seq )
 {
@@ -1403,7 +1866,7 @@ void seq_grp_nuc( int *grp, char *seq )
 		if( tmp < 4 )
 			*grp++ = tmp;
 		else
-			fprintf( stderr, "WARNING : Unknown character %c\r", *(seq-1) );
+			nunknown++;
 	}
 	*grp = END_OF_VEC;
 	if( grp - grpbk < tuplesize )
@@ -1424,7 +1887,7 @@ void seq_grp( int *grp, char *seq )
 		if( tmp < 6 )
 			*grp++ = tmp;
 		else
-			fprintf( stderr, "WARNING : Unknown character %c\r", *(seq-1) );
+			nunknown++;
 	}
 	*grp = END_OF_VEC;
 	if( grp - grpbk < 6 )
@@ -1880,7 +2343,9 @@ void ktupledistancematrix( int nseq, int norg, int nlenmax, char **seq, char **n
 			seq_grp( grpseq, tmpseq );
 			makepointtable( pointt[i], grpseq );
 		}
+
 	}
+	if( nunknown ) fprintf( stderr, "\nWARNING : %d unknown characters\n", nunknown );
 
 	for( i=0; i<nseq; i++ ) // serial de jubun
 	{
@@ -2110,6 +2575,7 @@ int main( int argc, char *argv[] )
 	static char **name, **seq;
 	static char  **tmpseq;
 	static char  *tmpseq1;
+//	static char  *check1, *check2;
 	static float **iscore, **iscore_kozo;
 	static double *eff_kozo, *eff_kozo_mapped = NULL;
 	int i, j, f, ien;
@@ -2128,6 +2594,7 @@ int main( int argc, char *argv[] )
 	int nkozo;
 	int njobc, norg, lenfull;
 	int **newgaplist_o;
+	int *newgaplist_compact;
 	int **follower;
 	int *follows;
 	int *istherenewgap;
@@ -2141,6 +2608,8 @@ int main( int argc, char *argv[] )
 	int ***topol;
 	float **len;
 	float **iscoreo, **nscore;
+	FILE *fp;
+	Addtree *addtree;
 
 
 	arguments( argc, argv );
@@ -2148,6 +2617,8 @@ int main( int argc, char *argv[] )
 	nthread = 0;
 #endif
 
+	if( fastathreshold < 0.0001 ) constraint = 0;
+
 	if( inputfile )
 	{
 		infp = fopen( inputfile, "r" );
@@ -2177,6 +2648,7 @@ int main( int argc, char *argv[] )
 	njobc = norg+1;
 	fprintf( stderr, "norg = %d\n", norg );
 	fprintf( stderr, "njobc = %d\n", njobc );
+	if( norg > 1000 || nadd > 1000 ) use_fft = 0;
 
 	fullseqlen = alloclen = nlenmax*4+1; //chuui!
 	seq = AllocateCharMtx( njob, alloclen );
@@ -2185,16 +2657,16 @@ int main( int argc, char *argv[] )
 	nlen = AllocateIntVec( njob );
 
 
-//	if( tuplesize > 0 ) // Tsuneni
+	if( multidist || tuplesize > 0 )
 	{
 		iscore = AllocateFloatHalfMtx( norg );
 		nscore = AllocateFloatMtx( norg, nadd );
 	}
-//	else
-//	{
-//		iscore = AllocateFloatHalfMtx( njob );
-//		nscore = NULL;
-//	}
+	else
+	{
+		iscore = AllocateFloatHalfMtx( njob );
+		nscore = NULL;
+	}
 
 	kozoarivec = AllocateCharVec( njob );
 
@@ -2204,28 +2676,81 @@ int main( int argc, char *argv[] )
 
 	if( constraint )
 	{
-		localhomtable = (LocalHom **)calloc( njob, sizeof( LocalHom *) );
-		for( i=0; i<njob; i++)
+#if SMALLMEMORY
+		if( multidist )
 		{
-			localhomtable[i] = (LocalHom *)calloc( njob, sizeof( LocalHom ) );
-			for( j=0; j<njob; j++)
+			localhomtable = (LocalHom **)calloc( norg, sizeof( LocalHom *) );
+			for( i=0; i<norg; i++)
 			{
-				localhomtable[i][j].start1 = -1;
-				localhomtable[i][j].end1 = -1;
-				localhomtable[i][j].start2 = -1;
-				localhomtable[i][j].end2 = -1;
-				localhomtable[i][j].overlapaa = -1.0;
-				localhomtable[i][j].opt = -1.0;
-				localhomtable[i][j].importance = -1.0;
-				localhomtable[i][j].next = NULL;
-				localhomtable[i][j].korh = 'h';
+				localhomtable[i] = (LocalHom *)calloc( nadd, sizeof( LocalHom ) );
+				for( j=0; j<nadd; j++)
+				{
+					localhomtable[i][j].start1 = -1;
+					localhomtable[i][j].end1 = -1;
+					localhomtable[i][j].start2 = -1;
+					localhomtable[i][j].end2 = -1;
+					localhomtable[i][j].overlapaa = -1.0;
+					localhomtable[i][j].opt = -1.0;
+					localhomtable[i][j].importance = -1.0;
+					localhomtable[i][j].next = NULL;
+					localhomtable[i][j].korh = 'h';
+				}
+			}
+//			localhomtable = (LocalHom **)calloc( norg+nadd, sizeof( LocalHom *) );
+//			for( i=norg; i<norg+nadd; i++) // hontou ha iranai
+//			{
+//				localhomtable[i] = (LocalHom *)calloc( norg, sizeof( LocalHom ) );
+//				for( j=0; j<norg; j++)
+//				{
+//					localhomtable[i][j].start1 = -1;
+//					localhomtable[i][j].end1 = -1;
+//					localhomtable[i][j].start2 = -1;
+//					localhomtable[i][j].end2 = -1;
+//					localhomtable[i][j].overlapaa = -1.0;
+//					localhomtable[i][j].opt = -1.0;
+//					localhomtable[i][j].importance = -1.0;
+//					localhomtable[i][j].next = NULL;
+//					localhomtable[i][j].korh = 'h';
+//				}
+//			}
+		}
+		else
+#endif
+		{
+			localhomtable = (LocalHom **)calloc( njob, sizeof( LocalHom *) );
+			for( i=0; i<njob; i++)
+			{
+				localhomtable[i] = (LocalHom *)calloc( njob, sizeof( LocalHom ) );
+				for( j=0; j<njob; j++)
+				{
+					localhomtable[i][j].start1 = -1;
+					localhomtable[i][j].end1 = -1;
+					localhomtable[i][j].start2 = -1;
+					localhomtable[i][j].end2 = -1;
+					localhomtable[i][j].overlapaa = -1.0;
+					localhomtable[i][j].opt = -1.0;
+					localhomtable[i][j].importance = -1.0;
+					localhomtable[i][j].next = NULL;
+					localhomtable[i][j].korh = 'h';
+				}
 			}
 		}
 
 		fprintf( stderr, "Loading 'hat3' ... " );
 		prep = fopen( "hat3", "r" );
 		if( prep == NULL ) ErrorExit( "Make hat3." );
-		readlocalhomtable( prep, njob, localhomtable, kozoarivec );
+#if SMALLMEMORY
+		if( multidist )
+		{
+//			readlocalhomtable_two( prep, norg, nadd, localhomtable, localhomtable+norg, kozoarivec );
+			readlocalhomtable_one( prep, norg, nadd, localhomtable, kozoarivec );
+		}
+		else
+#endif
+		{
+			readlocalhomtable( prep, njob, localhomtable, kozoarivec );
+		}
+
 		fclose( prep );
 		fprintf( stderr, "\ndone.\n" );
 
@@ -2246,7 +2771,11 @@ int main( int argc, char *argv[] )
 		}
 
 
+#if SMALLMEMORY
+//		outlocalhom_part( localhomtable, norg, nadd );
+#else
 //		outlocalhom( localhomtable, njob );
+#endif
 
 #if 0
 		fprintf( stderr, "Extending localhom ... " );
@@ -2376,6 +2905,7 @@ int main( int argc, char *argv[] )
 
 //	newgaplist_o = AllocateIntMtx( nadd, alloclen ); //ookisugi
 	newgaplist_o = AllocateIntMtx( nadd, lenfull*2 );
+	newgaplist_compact = AllocateIntVec( lenfull*2 );
 	istherenewgap = AllocateIntVec( nadd );
 	follower = AllocateIntMtx( norg, 1 );
 	for( i=0; i<norg; i++ ) follower[i][0] = -1;
@@ -2387,6 +2917,16 @@ int main( int argc, char *argv[] )
 //	iscoreo = AllocateFloatHalfMtx( norg );
 	mtxcpy( norg, norg, &iscoreo, iscore );
 
+	if( treeout )
+	{
+		addtree = (Addtree *)calloc( nadd, sizeof( Addtree ) );
+		if( !addtree )
+		{
+			fprintf( stderr, "Cannot allocate addtree\n" );
+			exit( 1 );
+		}
+	}
+
 
 //	nlim = norg-1;
 //	for( i=0; i<nlim; i++ )
@@ -2450,6 +2990,7 @@ int main( int argc, char *argv[] )
 			targ[i].dep = dep;
 			targ[i].topol = topol;
 			targ[i].len = len;
+			targ[i].addtree = addtree;
 			targ[i].mutex_counter = &mutex_counter;
 			pthread_create( handle+i, NULL, addsinglethread, (void *)(targ+i) );
 		}
@@ -2484,15 +3025,38 @@ int main( int argc, char *argv[] )
 		targ[0].dep = dep;
 		targ[0].topol = topol;
 		targ[0].len = len;
+		targ[0].addtree = addtree;
 		addsinglethread( targ );
 		free( targ );
 	}
 	free( dep );
-	FreeIntCub( topol );
 	FreeFloatMtx( len );
+	if( multidist || tuplesize > 0 ) FreeFloatMtx( nscore );
 
 //	for( i=0; i<nadd; i++ ) fprintf( stdout, ">%s (%d) \n%s\n", name[norg+i], norg+i, seq[norg+i] );
 
+	if( treeout )
+	{
+		fp = fopen( "infile.tree", "a" );
+		if( fp == 0 )
+		{
+			fprintf( stderr, "File error!\n" );
+			exit( 1 );
+		}
+		for( i=0; i<nadd; i++ )
+		{
+			fprintf( fp, "\n" );
+			fprintf( fp, "%8d: %s\n", norg+i+1, name[norg+i]+1 );
+			fprintf( fp, "          nearest sequence: %d\n", addtree[i].nearest + 1 );
+			fprintf( fp, "          approximate distance: %f\n", addtree[i].dist1 );
+			fprintf( fp, "          sister group: %s\n", addtree[i].neighbors );
+			fprintf( fp, "          approximate distance: %f\n", addtree[i].dist2 );
+			free( addtree[i].neighbors );
+		}
+		fclose( fp );
+		free( addtree );
+	}
+
 	for( iadd=0; iadd<nadd; iadd++ )
 	{
 		f = follows[iadd];
@@ -2506,7 +3070,7 @@ int main( int argc, char *argv[] )
 		follower[f][i] = iadd;
 		follower[f][i+1] = -1;
 #if 0
-		fprintf( stderr, "followers of %d = ", f );
+		fprintf( stderr, "\nfollowers of %d = ", f );
 		for( i=0; follower[f][i]!=-1; i++ )
 			fprintf( stderr, "%d ", follower[f][i]  );
 		fprintf( stderr, "\n" );
@@ -2522,8 +3086,10 @@ int main( int argc, char *argv[] )
 	for( i=0; ordertable[i]!=-1; i++ )
 	{
 		fprintf( orderfp, "%d\n", ordertable[i] );
-		for( j=0; follower[i][j]!=-1; j++ )
-			fprintf( orderfp, "%d\n", follower[i][j]+norg );
+//		for( j=0; follower[i][j]!=-1; j++ )
+//			fprintf( orderfp, "%d\n", follower[i][j]+norg );
+		for( j=0; follower[ordertable[i]][j]!=-1; j++ )
+			fprintf( orderfp, "%d\n", follower[ordertable[i]][j]+norg );
 //			fprintf( orderfp, "%d -> %d\n", follower[i][j]+norg, i );
 	}
 	fclose( orderfp );
@@ -2540,19 +3106,23 @@ int main( int argc, char *argv[] )
 
 	fprintf( stderr, "\n\nCombining ..\n" );
 	fflush( stderr );
-	tmpseqlen = alloclen * 20;
+	tmpseqlen = alloclen * 100;
 	tmpseq = AllocateCharMtx( 1, tmpseqlen );
-	
-//	lengthall = lenfull;
+//	check1 = AllocateCharVec( tmpseqlen );
+//	check2 = AllocateCharVec( tmpseqlen );
+//	gappick0( check2, seq[0] );
 	for( iadd=0; iadd<nadd; iadd++ )
 	{
 //		fprintf( stderr, "%d / %d\r", iadd, nadd );
 		fflush( stderr );
 
+//		fprintf( stderr, "\niadd == %d\n", iadd );
+		makegaplistcompact( lenfull, posmap, newgaplist_compact, newgaplist_o[iadd] );
 		if( iadd == 0 || istherenewgap[iadd] )
 		{
 			tmpseq1 = tmpseq[0];
- 			gaplist2alnx( lenfull, tmpseq1, seq[0], newgaplist_o[iadd], posmap, tmpseqlen );
+// 			gaplist2alnx( lenfull, tmpseq1, seq[0], newgaplist_o[iadd], posmap, tmpseqlen );
+ 			gaplist2alnx( lenfull, tmpseq1, seq[0], newgaplist_compact, posmap, tmpseqlen );
 //			fprintf( stderr, "len = %d ? %d\n", strlen( tmpseq1 ), alloclen );
 			if( ( tmplen = strlen( tmpseq1 ) ) >= fullseqlen )
 			{
@@ -2598,7 +3168,8 @@ int main( int argc, char *argv[] )
 					targ[i].tmpseqlen = tmpseqlen;
 					targ[i].lenfull = lenfull;
 					targ[i].seq = seq;
-					targ[i].newgaplist = newgaplist_o[iadd];
+//					targ[i].newgaplist = newgaplist_o[iadd];
+					targ[i].newgaplist = newgaplist_compact;
 					targ[i].posmap = posmap;
 					targ[i].mutex = &mutex;
 
@@ -2620,20 +3191,24 @@ int main( int argc, char *argv[] )
 				{
 					tmpseq1 = tmpseq[0];
 					if( i == 1 ) fprintf( stderr, " %d / %d\r", iadd, nadd );
- 					gaplist2alnx( lenfull, tmpseq1, seq[i], newgaplist_o[iadd], posmap, tmpseqlen  );
+// 					gaplist2alnx( lenfull, tmpseq1, seq[i], newgaplist_o[iadd], posmap, tmpseqlen  );
+ 					gaplist2alnx( lenfull, tmpseq1, seq[i], newgaplist_compact, posmap, tmpseqlen  );
 //					fprintf( stderr, ">%s (iadd=%d)\n%s\n", name[i], iadd, tmpseq1 );
 					strcpy( seq[i], tmpseq1 );
 				}
 			}
 		}
 		tmpseq1 = tmpseq[0];
-		insertgapsbyotherfragments_simple( lenfull, tmpseq1, seq[norg+iadd], newgaplist_o[iadd], posmap );
-//		fprintf( stderr, ">%s (iadd=%d, new)\n%s\n", name[norg+iadd], iadd, tmpseq1 );
+//		insertgapsbyotherfragments_simple( lenfull, tmpseq1, seq[norg+iadd], newgaplist_o[iadd], posmap );
+		insertgapsbyotherfragments_compact( lenfull, tmpseq1, seq[norg+iadd], newgaplist_o[iadd], posmap );
+//		fprintf( stderr, "%d = %s\n", iadd, tmpseq1 );
 		eq2dash( tmpseq1 );
 		strcpy( seq[norg+iadd], tmpseq1 );
 
-		adjustposmap( lenfull, posmap, newgaplist_o[iadd] );
+//		adjustposmap( lenfull, posmap, newgaplist_o[iadd] );
+		adjustposmap( lenfull, posmap, newgaplist_compact );
 		countnewres( lenfull, realign, posmap, newgaplist_o[iadd] ); // muda?
+//		countnewres( lenfull, realign, posmap, newgaplist_compact ); // muda?
 
 	}
 	fprintf( stderr, "\r   done.                      \n\n" );
@@ -2646,12 +3221,37 @@ int main( int argc, char *argv[] )
 	}
 #endif
 
+#if 0
+	fprintf( stderr, "realign[].nnewres = " );
+	for( i=0; i<lenfull+1; i++ )
+	{
+		fprintf( stderr, "%d ", realign[i].nnewres );
+	}
+	fprintf( stderr, "\n" );
+#endif
+
+	for( i=0; i<lenfull+1; i++ )
+	{
+	}
 
 	for( i=0; i<lenfull+1; i++ )
 	{
 		if( realign[i].nnewres > 1 ) 
 		{
-			zure = dorealignment( realign+i, seq, alloclen, fullseqlen, norg );
+//			fprintf( stderr, "i=%d: %d-%d\n", i, realign[i].start, realign[i].end );
+			fprintf( stderr, "\rRealigning %d/%d           \r", i, lenfull );
+//			zure = dorealignment_compact( realign+i, seq, &fullseqlen, norg );
+//			zure = dorealignment_order( realign+i, seq, &fullseqlen, norg, ordertable, follows );
+			zure = dorealignment_tree( realign+i, seq, &fullseqlen, norg, topol, follows );
+#if 0
+			gappick0( check1, seq[0] );
+			fprintf( stderr, "check1 = %s\n", check1 );
+			if( strcmp( check1, check2 ) )
+			{
+				fprintf( stderr, "CHANGED!!!!!\n" );
+				exit( 1 );
+			}
+#endif
 			for( j=i+1; j<lenfull+1; j++ )
 			{
 				if( realign[j].nnewres )
@@ -2662,12 +3262,14 @@ int main( int argc, char *argv[] )
 			}
 		}
 	}
+	FreeIntCub( topol );
 	fprintf( stderr, "\r   done.                      \n\n" );
 
 	fflush( stderr );
 
 
 	FreeIntMtx( newgaplist_o );
+	FreeIntVec( newgaplist_compact );
 	FreeIntVec( posmap );
 	free( realign );
 	free( istherenewgap );
@@ -2687,7 +3289,17 @@ int main( int argc, char *argv[] )
 	fprintf( stderr, "OSHIMAI\n" );
 #endif
 
-	if( constraint ) FreeLocalHomTable( localhomtable, njob );
+#if SMALLMEMORY
+	if( multidist )
+	{
+//		if( constraint ) FreeLocalHomTable_two( localhomtable, norg, nadd );
+		if( constraint ) FreeLocalHomTable_one( localhomtable, norg, nadd );
+	}
+	else
+#endif
+	{
+		if( constraint ) FreeLocalHomTable( localhomtable, njob );
+	}
 
 	SHOWVERSION;
 	return( 0 );
