@@ -11,6 +11,7 @@
 
 #define NORMALIZE1 1
 
+
 static int shishagonyuu( double in )
 {
 	int out;
@@ -21,6 +22,47 @@ static int shishagonyuu( double in )
 	return( out );
 }
 
+static void ambiguousscore( int *amino_n, int n_dis[0x80][0x80] )
+{
+	int i;
+	for( i=0; i<26; i++ )
+	{
+		n_dis[i][amino_n['r']] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['a']][i] + n_dis[amino_n['g']][i] ) );
+		n_dis[i][amino_n['y']] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['c']][i] + n_dis[amino_n['t']][i] ) );
+		n_dis[i][amino_n['k']] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['g']][i] + n_dis[amino_n['t']][i] ) );
+		n_dis[i][amino_n['m']] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['a']][i] + n_dis[amino_n['c']][i] ) );
+		n_dis[i][amino_n['s']] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['g']][i] + n_dis[amino_n['c']][i] ) );
+		n_dis[i][amino_n['w']] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['a']][i] + n_dis[amino_n['t']][i] ) );
+		n_dis[i][amino_n['b']] = shishagonyuu( (double)1/3 * ( n_dis[amino_n['c']][i] + n_dis[amino_n['g']][i] + n_dis[amino_n['t']][i] ) );
+		n_dis[i][amino_n['d']] = shishagonyuu( (double)1/3 * ( n_dis[amino_n['a']][i] + n_dis[amino_n['g']][i] + n_dis[amino_n['t']][i] ) );
+		n_dis[i][amino_n['h']] = shishagonyuu( (double)1/3 * ( n_dis[amino_n['a']][i] + n_dis[amino_n['c']][i] + n_dis[amino_n['t']][i] ) );
+		n_dis[i][amino_n['v']] = shishagonyuu( (double)1/3 * ( n_dis[amino_n['a']][i] + n_dis[amino_n['c']][i] + n_dis[amino_n['g']][i] ) );
+
+		n_dis[amino_n['r']][i] = n_dis[i][amino_n['r']];
+		n_dis[amino_n['y']][i] = n_dis[i][amino_n['y']];
+		n_dis[amino_n['k']][i] = n_dis[i][amino_n['k']];
+		n_dis[amino_n['m']][i] = n_dis[i][amino_n['m']];
+		n_dis[amino_n['s']][i] = n_dis[i][amino_n['s']];
+		n_dis[amino_n['w']][i] = n_dis[i][amino_n['w']];
+		n_dis[amino_n['b']][i] = n_dis[i][amino_n['b']];
+		n_dis[amino_n['d']][i] = n_dis[i][amino_n['d']];
+		n_dis[amino_n['h']][i] = n_dis[i][amino_n['h']];
+		n_dis[amino_n['v']][i] = n_dis[i][amino_n['v']];
+	}
+
+	i = amino_n['r']; n_dis[i][i] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['a']][amino_n['a']] + n_dis[amino_n['g']][amino_n['g']] ) );
+	i = amino_n['y']; n_dis[i][i] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['c']][amino_n['c']] + n_dis[amino_n['t']][amino_n['t']] ) );
+	i = amino_n['k']; n_dis[i][i] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['g']][amino_n['g']] + n_dis[amino_n['t']][amino_n['t']] ) );
+	i = amino_n['m']; n_dis[i][i] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['a']][amino_n['a']] + n_dis[amino_n['c']][amino_n['c']] ) );
+	i = amino_n['s']; n_dis[i][i] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['g']][amino_n['g']] + n_dis[amino_n['c']][amino_n['c']] ) );
+	i = amino_n['w']; n_dis[i][i] = shishagonyuu( (double)1/2 * ( n_dis[amino_n['a']][amino_n['a']] + n_dis[amino_n['t']][amino_n['t']] ) );
+	i = amino_n['b']; n_dis[i][i] = shishagonyuu( (double)1/3 * ( n_dis[amino_n['c']][amino_n['c']] + n_dis[amino_n['g']][amino_n['g']] + n_dis[amino_n['t']][amino_n['t']] ) );
+	i = amino_n['d']; n_dis[i][i] = shishagonyuu( (double)1/3 * ( n_dis[amino_n['a']][amino_n['a']] + n_dis[amino_n['g']][amino_n['g']] + n_dis[amino_n['t']][amino_n['t']] ) );
+	i = amino_n['h']; n_dis[i][i] = shishagonyuu( (double)1/3 * ( n_dis[amino_n['a']][amino_n['a']] + n_dis[amino_n['c']][amino_n['c']] + n_dis[amino_n['t']][amino_n['t']] ) );
+	i = amino_n['v']; n_dis[i][i] = shishagonyuu( (double)1/3 * ( n_dis[amino_n['a']][amino_n['a']] + n_dis[amino_n['c']][amino_n['c']] + n_dis[amino_n['g']][amino_n['g']] ) );
+}
+
+
 static void calcfreq_nuc( int nseq, char **seq, double *datafreq )
 {
 	int i, j, l;
@@ -43,8 +85,9 @@ static void calcfreq_nuc( int nseq, char **seq, double *datafreq )
 			}
 		}
 	}
-	for( i=0; i<4; i++ )
-		if( datafreq[i] < 0.0001 ) datafreq[i] = 0.0001;
+	total = 0.0; for( i=0; i<4; i++ ) total += datafreq[i];
+	for( i=0; i<4; i++ ) datafreq[i] /= (double)total;
+	for( i=0; i<4; i++ ) if( datafreq[i] < 0.0001 ) datafreq[i] = 0.0001;
 
 
 	total = 0.0; for( i=0; i<4; i++ ) total += datafreq[i];
@@ -65,7 +108,7 @@ static void calcfreq( int nseq, char **seq, double *datafreq )
 	int i, j, l;
 	int aan;
 	double total;
-	for( i=0; i<20; i++ )
+	for( i=0; i<nscoredalphabets; i++ )
 		datafreq[i] = 0.0;
 	total = 0.0;
 	for( i=0; i<nseq; i++ )
@@ -74,30 +117,60 @@ static void calcfreq( int nseq, char **seq, double *datafreq )
 		for( j=0; j<l; j++ )
 		{
 			aan = amino_n[(int)seq[i][j]];
-			if( aan >= 0 && aan < 20 )
+			if( aan >= 0 && aan < nscoredalphabets && seq[i][j] != '-' )
 			{
 				datafreq[aan] += 1.0;
 				total += 1.0;
 			}
 		}
 	}
-	for( i=0; i<20; i++ )
-		if( datafreq[i] < 0.0001 ) datafreq[i] = 0.0001;
+	total = 0.0; for( i=0; i<nscoredalphabets; i++ ) total += datafreq[i];
+	for( i=0; i<nscoredalphabets; i++ ) datafreq[i] /= (double)total;
+	for( i=0; i<nscoredalphabets; i++ ) if( datafreq[i] < 0.0001 ) datafreq[i] = 0.0001;
 
-	fprintf( stderr, "datafreq = \n" );
-	for( i=0; i<20; i++ )
-		fprintf( stderr, "%f\n", datafreq[i] );
+//	fprintf( stderr, "datafreq = \n" );
+//	for( i=0; i<nscoredalphabets; i++ )
+//		fprintf( stderr, "%f\n", datafreq[i] );
 
-	total = 0.0; for( i=0; i<20; i++ ) total += datafreq[i];
-	fprintf( stderr, "total = %f\n", total );
-	for( i=0; i<20; i++ ) datafreq[i] /= (double)total;
+	total = 0.0; for( i=0; i<nscoredalphabets; i++ ) total += datafreq[i];
+//	fprintf( stderr, "total = %f\n", total );
+	for( i=0; i<nscoredalphabets; i++ ) datafreq[i] /= (double)total;
 }
 
+static void generatenuc1pam( double **pam1, int kimuraR, double *freq )
+{
+	int i, j;
+	double R[4][4], mut[4], total, tmp;
+
+	R[0][0] = 0.0;     R[0][1] = kimuraR; R[0][2] = 1.0;     R[0][3] = 1.0;
+	R[1][0] = kimuraR; R[1][1] = 0.0;     R[1][2] = 1.0;     R[1][3] = 1.0;
+	R[2][0] = 1.0;     R[2][1] = 1.0;     R[2][2] = 0.0;     R[2][3] = kimuraR;
+	R[3][0] = 1.0;     R[3][1] = 1.0;     R[3][2] = kimuraR; R[3][3] = 0.0;
+
+
+	total = 0.0;
+	for( i=0; i<4; i++ )
+	{
+		tmp = 0.0;
+		for( j=0; j<4; j++ ) tmp += R[i][j] * freq[j];
+		mut[i] = tmp;
+		total += tmp * freq[i];
+	}
+	for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
+	{
+		if( i != j ) pam1[i][j] = 0.01 / total * R[i][j] * freq[j];
+		else pam1[i][j] = 1.0 - 0.01 / total * mut[i];
+	}
+}
+
+
 void constants( int nseq, char **seq )
 {
 	int i, j, x;
 //	double tmp;
 
+	if( nblosum < 0 ) dorp = 'p';
+
 	if( dorp == 'd' )  /* DNA */
 	{
 		int k, m;
@@ -106,7 +179,6 @@ void constants( int nseq, char **seq )
 		double **pam1 = AllocateDoubleMtx( 4, 4 );
 		double *freq = AllocateDoubleVec( 4 );
 
-
 		scoremtx = -1;
 		if( RNAppenalty == NOTSPECIFIED ) RNAppenalty = DEFAULTRNAGOP_N;
 		if( RNAppenalty_ex == NOTSPECIFIED ) RNAppenalty_ex = DEFAULTRNAGEP_N;
@@ -131,14 +203,34 @@ void constants( int nseq, char **seq )
 		penalty_OP = (int)( 3 * 600.0 / 1000.0 * ppenalty_OP + 0.5);
 		penalty_ex = (int)( 3 * 600.0 / 1000.0 * ppenalty_ex + 0.5);
 		penalty_EX = (int)( 3 * 600.0 / 1000.0 * ppenalty_EX + 0.5);
-		offset = (int)( 3 * 600.0 / 1000.0 * poffset + 0.5);
-		offsetFFT = (int)( 3 * 600.0 / 1000.0 * (-0) + 0.5);
-		offsetLN = (int)( 3 * 600.0 / 1000.0 * 100 + 0.5);
+		offset = (int)( 1 * 600.0 / 1000.0 * poffset + 0.5);
+		offsetFFT = (int)( 1 * 600.0 / 1000.0 * (-0) + 0.5);
+		offsetLN = (int)( 1 * 600.0 / 1000.0 * 100 + 0.5);
 		penaltyLN = (int)( 3 * 600.0 / 1000.0 * -2000 + 0.5);
 		penalty_exLN = (int)( 3 * 600.0 / 1000.0 * -100 + 0.5);
 		sprintf( modelname, "%s%d (%d), %6.3f (%6.3f), %6.3f (%6.3f)", rnakozo?"RNA":"DNA", pamN, kimuraR,
         -(double)ppenalty*0.001, -(double)ppenalty*0.003, -(double)poffset*0.001, -(double)poffset*0.003 );
 
+		for( i=0; i<26; i++ ) amino[i] = locaminon[i];
+		for( i=0; i<0x80; i++ ) amino_n[i] = -1;
+		for( i=0; i<26; i++ ) amino_n[(int)amino[i]] = i;
+		if( fmodel == 1 )
+		{
+			calcfreq_nuc( nseq, seq, freq );
+			fprintf( stderr, "a, freq[0] = %f\n", freq[0] );
+			fprintf( stderr, "g, freq[1] = %f\n", freq[1] );
+			fprintf( stderr, "c, freq[2] = %f\n", freq[2] );
+			fprintf( stderr, "t, freq[3] = %f\n", freq[3] );
+		}
+		else
+		{
+			freq[0] = 0.25;
+			freq[1] = 0.25;
+			freq[2] = 0.25;
+			freq[3] = 0.25;
+		}
+
+
 		if( kimuraR == 9999 ) 
 		{
 			for( i=0; i<4; i++ ) for( j=0; j<4; j++ ) 
@@ -164,6 +256,7 @@ void constants( int nseq, char **seq )
 		}
 		else
 		{
+#if 0
 				double f = 0.99;
 				double s = (double)kimuraR / ( 2 + kimuraR ) * 0.01;
 				double v = (double)1       / ( 2 + kimuraR ) * 0.01;
@@ -171,6 +264,9 @@ void constants( int nseq, char **seq )
 				pam1[1][0] = s; pam1[1][1] = f; pam1[1][2] = v; pam1[1][3] = v;
 				pam1[2][0] = v; pam1[2][1] = v; pam1[2][2] = f; pam1[2][3] = s;
 				pam1[3][0] = v; pam1[3][1] = v; pam1[3][2] = s; pam1[3][3] = f;
+#else
+				generatenuc1pam( pam1, kimuraR, freq );
+#endif
 	
 				fprintf( stderr, "generating %dPAM scoring matrix for nucleotides ... ", pamN );
 	
@@ -189,8 +285,22 @@ void constants( int nseq, char **seq )
 	
 				MtxuntDouble( pamx, 4 );
 				for( x=0; x < pamN; x++ ) MtxmltDouble( pamx, pam1, 4 );
+
+		       	if( disp )
+   		    	{
+   		     		fprintf( stderr, " TPM \n" );
+   		        	for( i=0; i<4; i++ )
+   			       	{
+   		            	for( j=0; j<4; j++ )
+   		                	fprintf( stderr, "%+#6.10f", pamx[i][j] );
+   		            	fprintf( stderr, "\n" );
+   		        	}
+   		        	fprintf( stderr, "\n" );
+   		     	}
+
 				for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
-					pamx[i][j] /= 1.0 / 4.0;
+					pamx[i][j] /= freq[j];
+//					pamx[i][j] /= 0.25;
 	
 				for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
 				{
@@ -208,7 +318,7 @@ void constants( int nseq, char **seq )
    	        		for( i=0; i<4; i++ )
    		       		{
    	        	    	for( j=0; j<4; j++ )
-   	        	        	fprintf( stderr, "%+#6.10f", pamx[i][j] );
+   	        	        	fprintf( stderr, "%+10.6f ", pamx[i][j] );
    	        	    	fprintf( stderr, "\n" );
    	        		}
    	        		fprintf( stderr, "\n" );
@@ -216,23 +326,6 @@ void constants( int nseq, char **seq )
 
 
 // ?????
-			for( i=0; i<26; i++ ) amino[i] = locaminon[i];
-			for( i=0; i<0x80; i++ ) amino_n[i] = -1;
-			for( i=0; i<26; i++ ) amino_n[(int)amino[i]] = i;
-			if( fmodel == 1 )
-				calcfreq_nuc( nseq, seq, freq );
-			else
-			{
-				freq[0] = 0.25;
-				freq[1] = 0.25;
-				freq[2] = 0.25;
-				freq[3] = 0.25;
-			}
-//			fprintf( stderr, "a, freq[0] = %f\n", freq[0] );
-//			fprintf( stderr, "g, freq[1] = %f\n", freq[1] );
-//			fprintf( stderr, "c, freq[2] = %f\n", freq[2] );
-//			fprintf( stderr, "t, freq[3] = %f\n", freq[3] );
-
 			
 			average = 0.0;
 			for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
@@ -249,7 +342,7 @@ void constants( int nseq, char **seq )
 
 
 			for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
-				pamx[i][j] -= offset;        /* extending gap cost */
+				pamx[i][j] -= offset;
 
 			for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
 				pamx[i][j] = shishagonyuu( pamx[i][j] );
@@ -301,12 +394,16 @@ void constants( int nseq, char **seq )
                	fprintf( stderr, "\n" );
            	}
            	fprintf( stderr, "\n" );
+					exit( 1 );
         }
 
 		for( i=0; i<26; i++ ) amino[i] = locaminon[i];
 		for( i=0; i<26; i++ ) amino_grp[(int)amino[i]] = locgrpn[i];
 		for( i=0; i<26; i++ ) for( j=0; j<26; j++ ) n_dis[i][j] = 0;
 		for( i=0; i<10; i++ ) for( j=0; j<10; j++ ) n_dis[i][j] = shishagonyuu( pamx[i][j] );
+
+		ambiguousscore( amino_n, n_dis );
+
         if( disp )
         {
             fprintf( stderr, " score matrix  \n" );
@@ -421,7 +518,188 @@ void constants( int nseq, char **seq )
 		free( freq );
 
 	}
-	else if( dorp == 'p' && scoremtx == 1 )  /* Blosum */
+	else if( dorp == 'p' && scoremtx == 1 && nblosum == -2 )  /* extended */
+	{
+		double *freq;
+		double *freq1;
+		double *datafreq;
+		double average;
+//		double tmp;
+		double **n_distmp;
+
+		nalphabets = 0x80;
+		nscoredalphabets = 0x80;
+
+		fprintf( stderr, "nalphabets = %d\n", nalphabets );
+
+		n_distmp = AllocateDoubleMtx( nalphabets, nalphabets );
+		datafreq = AllocateDoubleVec( nalphabets );
+		freq = AllocateDoubleVec( nalphabets );
+
+		if( ppenalty == NOTSPECIFIED ) ppenalty = DEFAULTGOP_B;
+		if( ppenalty_OP == NOTSPECIFIED ) ppenalty_OP = DEFAULTGOP_B;
+		if( ppenalty_ex == NOTSPECIFIED ) ppenalty_ex = DEFAULTGEP_B;
+		if( ppenalty_EX == NOTSPECIFIED ) ppenalty_EX = DEFAULTGEP_B;
+		if( poffset == NOTSPECIFIED ) poffset = DEFAULTOFS_B;
+		if( pamN == NOTSPECIFIED ) pamN = 0;
+		if( kimuraR == NOTSPECIFIED ) kimuraR = 1;
+		penalty = (int)( 600.0 / 1000.0 * ppenalty + 0.5 );
+		penalty_OP = (int)( 600.0 / 1000.0 * ppenalty_OP + 0.5 );
+		penalty_ex = (int)( 600.0 / 1000.0 * ppenalty_ex + 0.5 );
+		penalty_EX = (int)( 600.0 / 1000.0 * ppenalty_EX + 0.5 );
+		offset = (int)( 600.0 / 1000.0 * poffset + 0.5 );
+		offsetFFT = (int)( 600.0 / 1000.0 * (-0) + 0.5);
+		offsetLN = (int)( 600.0 / 1000.0 * 100 + 0.5);
+		penaltyLN = (int)( 600.0 / 1000.0 * -2000 + 0.5);
+		penalty_exLN = (int)( 600.0 / 1000.0 * -100 + 0.5);
+
+		extendedmtx( n_distmp, freq, amino, amino_grp );
+		sprintf( modelname, "Extended, %6.3f, %+6.3f, %+6.3f", -(double)ppenalty/1000, -(double)poffset/1000, -(double)ppenalty_ex/1000 );
+#if 0
+		for( i=0; i<26; i++ ) amino[i] = locaminod[i];
+		for( i=0; i<26; i++ ) amino_grp[(int)amino[i]] = locgrpd[i];
+		for( i=0; i<0x80; i++ ) amino_n[i] = 0;
+		for( i=0; i<26; i++ ) amino_n[(int)amino[i]] = i;
+#endif
+		for( i=0; i<0x80; i++ )amino_n[i] = -1;
+		for( i=0; i<nalphabets; i++) 
+		{
+			amino_n[(int)amino[i]] = i;
+//			fprintf( stderr, "i=%d, amino = %c, amino_n = %d\n", i, amino[i], amino_n[amino[i]] );
+		}
+		if( fmodel == 1 )
+		{
+			calcfreq( nseq, seq, datafreq );
+			freq1 = datafreq;
+		}
+		else
+			freq1 = freq;
+#if TEST
+		fprintf( stderr, "raw scoreing matrix : \n" );
+		for( i=0; i<nalphabets; i++ )
+		{
+			for( j=0; j<nalphabets; j++ ) 
+			{
+				fprintf( stdout, "%6.2f", n_distmp[i][j] );
+			}
+			fprintf( stdout, "\n" );
+		}
+#endif
+		if( fmodel == -1 )
+			average = 0.0;
+		else
+		{
+			for( i=0; i<nalphabets; i++ )
+#if TEST 
+				fprintf( stdout, "freq[%c] = %f, datafreq[%c] = %f, freq1[] = %f\n", amino[i], freq[i], amino[i], datafreq[i], freq1[i] );
+#endif
+			average = 0.0;
+			for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ )
+				average += n_distmp[i][j] * freq1[i] * freq1[j];
+		}
+#if TEST
+		fprintf( stdout, "####### average2  = %f\n", average );
+#endif
+
+		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) 
+			n_distmp[i][j] -= average;
+#if TEST
+		fprintf( stdout, "average2 = %f\n", average );
+		fprintf( stdout, "after average substruction : \n" );
+		for( i=0; i<nalphabets; i++ )
+		{
+			for( j=0; j<nalphabets; j++ ) 
+			{
+				fprintf( stdout, "%6.2f", n_distmp[i][j] );
+			}
+			fprintf( stdout, "\n" );
+		}
+#endif
+		
+		average = 0.0;
+		for( i=0; i<nalphabets; i++ ) 
+			average += n_distmp[i][i] * freq1[i];
+#if TEST
+		fprintf( stdout, "####### average1  = %f\n", average );
+#endif
+
+		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) 
+			n_distmp[i][j] *= 600.0 / average;
+#if TEST
+        fprintf( stdout, "after average division : \n" );
+        for( i=0; i<nalphabets; i++ )
+        {
+            for( j=0; j<=i; j++ )
+            {
+                fprintf( stdout, "%7.1f", n_distmp[i][j] );
+            }
+            fprintf( stdout, "\n" );
+        }
+#endif
+
+		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) 
+			n_distmp[i][j] -= offset;  
+#if TEST
+		fprintf( stdout, "after offset substruction (offset = %d): \n", offset );
+		for( i=0; i<nalphabets; i++ )
+		{
+			for( j=0; j<=i; j++ ) 
+			{
+				fprintf( stdout, "%7.1f", n_distmp[i][j] );
+			}
+			fprintf( stdout, "\n" );
+		}
+#endif
+#if 0
+/*   */
+			penalty -= offset;
+#endif
+
+
+		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) 
+			n_distmp[i][j] = shishagonyuu( n_distmp[i][j] );
+
+        if( disp )
+        {
+            fprintf( stdout, " scoring matrix  \n" );
+            for( i=0; i<nalphabets; i++ )
+            {
+				fprintf( stdout, "%c    ", amino[i] );
+                for( j=0; j<nalphabets; j++ )
+                    fprintf( stdout, "%5.0f", n_distmp[i][j] );
+                fprintf( stdout, "\n" );
+            }
+			fprintf( stdout, "     " );
+            for( i=0; i<nalphabets; i++ )
+				fprintf( stdout, "    %c", amino[i] );
+
+			average = 0.0;
+        	for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ )
+				average += n_distmp[i][j] * freq1[i] * freq1[j];
+			fprintf( stdout, "average = %f\n", average );
+
+			average = 0.0;
+        	for( i=0; i<nalphabets; i++ )
+				average += n_distmp[i][i] * freq1[i];
+			fprintf( stdout, "itch average = %f\n", average );
+			fprintf( stderr, "parameters: %d, %d, %d\n", penalty, penalty_ex, offset );
+
+			
+  			exit( 1 );
+        }
+
+		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) n_dis[i][j] = 0;
+		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) n_dis[i][j] = (int)n_distmp[i][j];
+		for( i=0; i<nalphabets; i++ ) for( j=0; j<nalphabets; j++ ) n_dis[i][amino_n['-']] = n_dis[amino_n['-']][i] = 0.0;
+
+		FreeDoubleMtx( n_distmp );
+		FreeDoubleVec( datafreq );
+		FreeDoubleVec( freq );
+
+		fprintf( stderr, "done.\n" );
+
+	}
+	else if( dorp == 'p' && scoremtx == 1 )  /* Blosum, user-defined */
 	{
 		double *freq;
 		double *freq1;
@@ -667,6 +945,17 @@ void constants( int nseq, char **seq )
 
 		JTTmtx( rsr, freq, amino, amino_grp, (int)(TMorJTT==TM) );
 
+		for( i=0; i<0x80; i++ ) amino_n[i] = -1;
+		for( i=0; i<26; i++ ) amino_n[(int)amino[i]] = i;
+		if( fmodel == 1 )
+		{
+			calcfreq( nseq, seq, datafreq );
+			freq1 = datafreq;
+		}
+		else
+			freq1 = freq;
+
+
 #if TEST
 		fprintf( stdout, "rsr = \n" );
 		for( i=0; i<20; i++ )
@@ -679,16 +968,6 @@ void constants( int nseq, char **seq )
 		}
 #endif
 
-		for( i=0; i<0x80; i++ ) amino_n[i] = -1;
-		for( i=0; i<26; i++ ) amino_n[(int)amino[i]] = i;
-
-		if( fmodel == 1 )
-		{
-			calcfreq( nseq, seq, datafreq );
-			freq1 = datafreq;
-		}
-		else
-			freq1 = freq;
 
 		fprintf( stderr, "generating %dPAM %s scoring matrix for amino acids ... ", pamN, (TMorJTT==TM)?"Transmembrane":"JTT" );
 
@@ -697,8 +976,8 @@ void constants( int nseq, char **seq )
 		{
 			mutab[i] = 0.0;
 			for( j=0; j<20; j++ )
-				mutab[i] += rsr[i][j] * freq[j];
-			tmp += mutab[i] * freq[i];
+				mutab[i] += rsr[i][j] * freq1[j];
+			tmp += mutab[i] * freq1[i];
 		}
 #if TEST
 		fprintf( stdout, "mutability = \n" );
@@ -713,7 +992,7 @@ void constants( int nseq, char **seq )
 			for( j=0; j<20; j++ )
 			{
 				if( i != j )
-					pam1[i][j] = delta * rsr[i][j] * freq[i];
+					pam1[i][j] = delta * rsr[i][j] * freq1[j];
 				else
 					pam1[i][j] = 1.0 - delta * mutab[i];
 			}
@@ -736,7 +1015,7 @@ void constants( int nseq, char **seq )
 		for( x=0; x < pamN; x++ ) MtxmltDouble( pamx, pam1, 20 );
 
 		for( i=0; i<20; i++ ) for( j=0; j<20; j++ ) 
-			pamx[i][j] /= freq[j];
+			pamx[i][j] /= freq1[j];
 
         for( i=0; i<20; i++ ) for( j=0; j<20; j++ )
 		{
@@ -920,11 +1199,11 @@ void constants( int nseq, char **seq )
 #endif
 
 	for( i=0; i<0x80; i++ )amino_n[i] = -1;
-	for( i=0; i<26; i++) amino_n[(int)amino[i]] = i;
+	for( i=0; i<nalphabets; i++) amino_n[(int)amino[i]] = i;
     for( i=0; i<0x80; i++ ) for( j=0; j<0x80; j++ ) amino_dis[i][j] = 0;
     for( i=0; i<0x80; i++ ) for( j=0; j<0x80; j++ ) amino_disLN[i][j] = 0;
     for( i=0; i<0x80; i++ ) for( j=0; j<0x80; j++ ) amino_dis_consweight_multi[i][j] = 0.0;
-    for( i=0; i<26; i++) for( j=0; j<26; j++ )
+    for( i=0; i<nalphabets; i++) for( j=0; j<nalphabets; j++ )
 	{
         amino_dis[(int)amino[i]][(int)amino[j]] = n_dis[i][j];
 		n_dis_consweight_multi[i][j] = (float)n_dis[i][j] * consweight_multi;
