@@ -18,12 +18,12 @@ char amino_grp[0x80];
 int amino_dis[0x80][0x80];
 int amino_disLN[0x80][0x80];
 double amino_dis_consweight_multi[0x80][0x80];
-int n_dis[26][26];
-int n_disFFT[26][26];
-float n_dis_consweight_multi[26][26];
-char amino[26];
-double polarity[20];
-double volume[20];
+int n_dis[0x80][0x80];
+int n_disFFT[0x80][0x80];
+float n_dis_consweight_multi[0x80][0x80];
+char amino[0x80];
+double polarity[0x80];
+double volume[0x80];
 int ribosumdis[37][37];
 
 int ppid;
@@ -94,6 +94,7 @@ int rnakozo;
 char rnaprediction;
 int scoreout = 0;
 int outnumber = 0;
+int legacygapcost = 0;
 
 char *signalSM;
 FILE *prep_g;
@@ -105,4 +106,7 @@ float consweight_multi = 1.0;
 float consweight_rna = 0.0;
 char RNAscoremtx = 'n';
 
-char *newgapstr = "-";
+char TLS *newgapstr = "-";
+
+int nalphabets = 26;
+int nscoredalphabets = 20;
