@@ -14,6 +14,8 @@ static int noalign;
 static int distout;
 static float lenfaca, lenfacb, lenfacc, lenfacd;
 static int tuplesize;
+static int subalignment;
+static int subalignmentoffset;
 #if 0
 #define PLENFACA 0.0123
 #define PLENFACB 10252
@@ -52,6 +54,7 @@ typedef struct _treebasethread_arg
 	double *effarr;
 	int *alloclenpt;
 	int *fftlog;
+	char *mergeoralign;
 	pthread_mutex_t *mutex;
 	pthread_cond_t *treecond;
 } treebasethread_arg_t;
@@ -125,6 +128,9 @@ void arguments( int argc, char *argv[] )
 	TMorJTT = JTT;
 	scoreout = 0;
 	tuplesize = 6;
+	subalignment = 0;
+	subalignmentoffset = 0;
+	legacygapcost = 0;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -138,7 +144,7 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'I':
-					nadd = atoi( *++argv );
+					nadd = myatoi( *++argv );
 					fprintf( stderr, "nadd = %d\n", nadd );
 					--argc;
 					goto nextoption;
@@ -158,32 +164,32 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'k':
-					kimuraR = atoi( *++argv );
+					kimuraR = myatoi( *++argv );
 					fprintf( stderr, "kappa = %d\n", kimuraR );
 					--argc;
 					goto nextoption;
 				case 'b':
-					nblosum = atoi( *++argv );
+					nblosum = myatoi( *++argv );
 					scoremtx = 1;
 //					fprintf( stderr, "blosum %d / kimura 200 \n", nblosum );
 					--argc;
 					goto nextoption;
 				case 'j':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
 					fprintf( stderr, "jtt/kimura %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'm':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
 					fprintf( stderr, "tm %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'C':
-					nthread = atoi( *++argv );
+					nthread = myatoi( *++argv );
 					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
 					goto nextoption;
@@ -215,9 +221,17 @@ void arguments( int argc, char *argv[] )
 				case 'P':
 					dorp = 'p';
 					break;
+				case 'L':
+					legacygapcost = 1;
+					break;
 				case 'e':
 					fftscore = 0;
 					break;
+				case 'H':
+					subalignment = 1;
+					subalignmentoffset = myatoi( *++argv );
+					--argc;
+					goto nextoption;
 #if 0
 				case 'R':
 					fftRepeatStop = 1;
@@ -242,6 +256,9 @@ void arguments( int argc, char *argv[] )
 				case 'a':
 					alg = 'a';
 					break;
+				case 'H':
+					alg = 'H';
+					break;
 #endif
 				case 'R':
 					alg = 'R';
@@ -249,9 +266,6 @@ void arguments( int argc, char *argv[] )
 				case 'Q':
 					alg = 'Q';
 					break;
-				case 'H':
-					alg = 'H';
-					break;
 				case 'A':
 					alg = 'A';
 					break;
@@ -302,15 +316,15 @@ void arguments( int argc, char *argv[] )
 					tbutree = 0;
 					break;
 				case 'z':
-					fftThreshold = atoi( *++argv );
+					fftThreshold = myatoi( *++argv );
 					--argc; 
 					goto nextoption;
 				case 'w':
-					fftWinSize = atoi( *++argv );
+					fftWinSize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'W':
-					tuplesize = atoi( *++argv );
+					tuplesize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'Z':
@@ -347,6 +361,7 @@ void arguments( int argc, char *argv[] )
 
 static int maxl;
 static int tsize;
+static int nunknown = 0;
 
 void seq_grp_nuc( int *grp, char *seq )
 {
@@ -358,7 +373,7 @@ void seq_grp_nuc( int *grp, char *seq )
 		if( tmp < 4 )
 			*grp++ = tmp;
 		else
-			fprintf( stderr, "WARNING : Unknown character %c\r", *(seq-1) );
+			nunknown++;
 	}
 	*grp = END_OF_VEC;
 	if( grp - grpbk < tuplesize )
@@ -379,7 +394,7 @@ void seq_grp( int *grp, char *seq )
 		if( tmp < 6 )
 			*grp++ = tmp;
 		else
-			fprintf( stderr, "WARNING : Unknown character %c\r", *(seq-1) );
+			nunknown++;
 	}
 	*grp = END_OF_VEC;
 	if( grp - grpbk < 6 )
@@ -626,6 +641,7 @@ static void *treebasethread( void *arg )
 	double *effarr = targ->effarr;
 	int *alloclen = targ->alloclenpt;
 	int *fftlog = targ->fftlog;
+	char *mergeoralign = targ->mergeoralign;
 	
 	char **mseq1, **mseq2;
 	char **localcopy;
@@ -646,6 +662,7 @@ static void *treebasethread( void *arg )
 	mseq1 = AllocateCharMtx( njob, 0 );
 	mseq2 = AllocateCharMtx( njob, 0 );
 	localcopy = calloc( njob, sizeof( char * ) );
+	for( i=0; i<njob; i++ ) localcopy[i] = NULL;
 	effarr1 = AllocateDoubleVec( njob );
 	effarr2 = AllocateDoubleVec( njob );
 	indication1 = AllocateCharVec( 150 );
@@ -676,6 +693,9 @@ static void *treebasethread( void *arg )
 			if( commonIP ) FreeIntMtx( commonIP );
 			commonIP = NULL;
 			Falign( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
+			Falign_udpari_long( NULL, NULL, NULL, NULL, 0, 0, 0, NULL );
+			A__align( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
+			G__align11( NULL, NULL, 0, 0, 0 ); // iru?
 			free( mseq1 );
 			free( mseq2 );
 			free( localcopy );
@@ -701,6 +721,22 @@ static void *treebasethread( void *arg )
 			pthread_cond_wait( targ->treecond, targ->mutex );
 		(*nrunpt)++;
 
+
+		if( mergeoralign[l] == 'n' )
+		{
+//			fprintf( stderr, "SKIP!\n" );
+			dep[l].done = 1;
+			(*nrunpt)--;
+			pthread_cond_broadcast( targ->treecond );
+			free( topol[l][0] );
+			free( topol[l][1] );
+			free( topol[l] );
+			pthread_mutex_unlock( targ->mutex );
+			continue;
+		}
+
+
+
 		m1 = topol[l][0][0];
 		m2 = topol[l][1][0];
 
@@ -724,6 +760,22 @@ static void *treebasethread( void *arg )
 			localcopy[j] = calloc( *alloclen, sizeof( char ) );
 			strcpy( localcopy[j], aseq[j] );
 		}
+
+		if( !nevermemsave && ( alg != 'M' && ( len1 > 30000 || len2 > 30000  ) ) )
+		{
+			fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode\n", len1, len2 );
+			alg = 'M';
+		}
+
+		if( alg == 'M' ) // hoka no thread ga M ni shitakamo shirenainode
+		{
+//			fprintf( stderr, "Freeing commonIP (thread %d)\n", thread_no );
+			if( commonIP ) FreeIntMtx( commonIP );
+			commonIP = NULL;
+			commonAlloc1 = 0;
+			commonAlloc2 = 0;
+		}
+
 		pthread_mutex_unlock( targ->mutex );
 
 #if 1 // CHUUI@@@@
@@ -808,14 +860,6 @@ static void *treebasethread( void *arg )
 */
 
 
-		if( !nevermemsave && ( alg != 'M' && ( len1 > 30000 || len2 > 30000  ) ) )
-		{
-			fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode\n", len1, len2 );
-			alg = 'M';
-			if( commonIP ) FreeIntMtx( commonIP );
-			commonAlloc1 = 0;
-			commonAlloc2 = 0;
-		}
 
 //		if( fftlog[m1] && fftlog[m2] ) ffttry = ( nlen[m1] > clus1 && nlen[m2] > clus2 );
 		if( fftlog[m1] && fftlog[m2] ) ffttry = ( nlen[m1] > clus1 && nlen[m2] > clus2 && clus1 < 1000 && clus2 < 1000);
@@ -904,12 +948,23 @@ static void *treebasethread( void *arg )
 		pthread_mutex_unlock( targ->mutex );
 
 		for( i=0; (j=topol[l][0][i])!=-1; i++ )
-			free( localcopy[j] );
+		{
+			if(localcopy[j] ) free( localcopy[j] );
+			localcopy[j] = NULL;
+		}
 		for( i=0; (j=topol[l][1][i])!=-1; i++ )
-			free( localcopy[j] );
-		free( topol[l][0] );
-		free( topol[l][1] );
-		free( topol[l] );
+		{
+			if( localcopy[j] ) free( localcopy[j] );
+			localcopy[j] = NULL;
+		}
+
+
+		if( topol[l][0] ) free( topol[l][0] );
+		topol[l][0] = NULL;
+		if( topol[l][1] ) free( topol[l][1] );
+		topol[l][1] = NULL;
+		if( topol[l] ) free( topol[l] );
+		topol[l] = NULL;
 
 
 //		fprintf( stderr, "\n" );
@@ -1115,6 +1170,7 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 			fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode\n", len1, len2 );
 			alg = 'M';
 			if( commonIP ) FreeIntMtx( commonIP );
+			commonIP = NULL;
 			commonAlloc1 = 0;
 			commonAlloc2 = 0;
 		}
@@ -1136,6 +1192,7 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 			else
 			{
 				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1, NULL, 0, NULL );
+//				fprintf( stderr, "######### mseq1[0] = %s\n", mseq1[0] );
 			}
 		}
 		else
@@ -1207,16 +1264,18 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 		}
 		if( mergeoralign[l] == '2' )
 		{
+//			for( i=0; i<clus1; i++ ) fprintf( stderr, ">STEP0 mseq1[%d] = \n%s\n", i, mseq1[i] );
+//			for( i=0; i<clus2; i++ ) fprintf( stderr, ">STEP0 mseq2[%d] = \n%s\n", i, mseq2[i] );
 			adjustgapmap( strlen( mseq1[0] )-len1nocommongap+len1, gapmap, mseq1[0] );
-//			fprintf( stderr, ">STEP1 mseq1[0] = \n%s\n", mseq1[0] );
-//			fprintf( stderr, ">STEP1 mseq2[0] = \n%s\n", mseq2[0] );
+//			for( i=0; i<clus1; i++ ) fprintf( stderr, ">STEP1 mseq1[%d] = \n%s\n", i, mseq1[i] );
+//			for( i=0; i<clus2; i++ ) fprintf( stderr, ">STEP1 mseq2[%d] = \n%s\n", i, mseq2[i] );
 			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen, '-' );
-//			fprintf( stderr, "STEP2 mseq1[0] = %s\n", mseq1[0] );
-//			fprintf( stderr, "STEP2 mseq2[0] = %s\n", mseq2[0] );
+//			for( i=0; i<clus1; i++ ) fprintf( stderr, ">STEP2 mseq1[%d] = \n%s\n", i, mseq1[i] );
+//			for( i=0; i<clus2; i++ ) fprintf( stderr, ">STEP2 mseq2[%d] = \n%s\n", i, mseq2[i] );
 			findnewgaps( clus1, 0, mseq1, gaplen );
 			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, gapmap, *alloclen, alg, '-' );
-//			fprintf( stderr, "STEP3 mseq1[0] = %s\n", mseq1[0] );
-//			fprintf( stderr, "STEP3 mseq2[0] = %s\n", mseq2[0] );
+//			for( i=0; i<clus1; i++ ) fprintf( stderr, ">STEP3 mseq1[%d] = \n%s\n", i, mseq1[i] );
+//			for( i=0; i<clus2; i++ ) fprintf( stderr, ">STEP3 mseq2[%d] = \n%s\n", i, mseq2[i] );
 			for( i=0; i<njob; i++ ) eq2dash( aseq[i] );
 			for( i=0; (m=topol[l][1][i])>-1; i++ ) alreadyaligned[m] = 1;
 		}
@@ -1224,6 +1283,8 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 		free( topol[l][0] );
 		free( topol[l][1] );
 		free( topol[l] );
+
+//		fprintf( stderr, ">514\n%s\n", aseq[514] );
 	}
 #if SCOREOUT
 	fprintf( stderr, "totalscore = %10.2f\n\n", tscore );
@@ -1327,6 +1388,11 @@ int main( int argc, char *argv[] )
 	int alignmentlength;
 	char *mergeoralign;
 	int foundthebranch;
+	int nsubalignments, maxmem;
+	int **subtable;
+	int *insubtable;
+	int *preservegaps;
+	char ***subalnpt;
 
 	arguments( argc, argv );
 #ifndef enablemultithread
@@ -1362,6 +1428,21 @@ int main( int argc, char *argv[] )
 		exit( 1 );
 	}
 
+	if( subalignment )
+	{
+		readsubalignmentstable( njob, NULL, NULL, &nsubalignments, &maxmem );
+		fprintf( stderr, "nsubalignments = %d\n", nsubalignments );
+		fprintf( stderr, "maxmem = %d\n", maxmem );
+		subtable = AllocateIntMtx( nsubalignments, maxmem+1 );
+		insubtable = AllocateIntVec( njob );
+		preservegaps = AllocateIntVec( njob );
+		for( i=0; i<njob; i++ ) insubtable[i] = 0;
+		for( i=0; i<njob; i++ ) preservegaps[i] = 0;
+		subalnpt = AllocateCharCub( nsubalignments, maxmem, 0 );
+		readsubalignmentstable( njob, subtable, preservegaps, NULL, NULL );
+	}
+
+
 	seq = AllocateCharMtx( njob, nlenmax*1+1 );
 	mseq1 = AllocateCharMtx( njob, 0 );
 	mseq2 = AllocateCharMtx( njob, 0 );
@@ -1389,6 +1470,7 @@ int main( int argc, char *argv[] )
 	constants( njob, seq );
 
 
+
 #if 0
 	fprintf( stderr, "params = %d, %d, %d\n", penalty, penalty_ex, offset );
 #endif
@@ -1487,6 +1569,7 @@ int main( int argc, char *argv[] )
 				makepointtable( pointt[i], grpseq );
 			}
 		}
+		if( nunknown ) fprintf( stderr, "\nWARNING : %d unknown characters\n", nunknown );
 #ifdef enablemultithread
 		if( nthread > 0 )
 		{
@@ -1613,17 +1696,22 @@ int main( int argc, char *argv[] )
 	else if( topin )
 	{
 		fprintf( stderr, "Loading a topology ... " );
-		loadtop( njob, mtx, topol, len );
-		FreeFloatHalfMtx( mtx, njob );
+		fprintf( stderr, "--topin has been disabled\n" );
+		exit( 1 );
+//		loadtop( njob, mtx, topol, len );
+//		FreeFloatHalfMtx( mtx, njob );
 	}
-	else if( treeout )
+	else if( subalignment ) // merge error no tame
+	{
+		fprintf( stderr, "Constructing a UPGMA tree ... " );
+		fixed_supg_float_realloc_nobk_halfmtx_treeout_constrained( njob, mtx, topol, len, name, nlen, dep, nsubalignments, subtable );
+		FreeFloatHalfMtx( mtx, njob );
+//		fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( njob, iscore, topol, len, name, nlen, dep );
+	}
+	else if( treeout ) // merge error no tame
 	{
 		fprintf( stderr, "Constructing a UPGMA tree ... " );
-
 		fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( njob, mtx, topol, len, name, nogaplen, dep );
-//		veryfastsupg_float_realloc_nobk_halfmtx_treeout( njob, mtx, topol, len, name, nogaplen );
-
-
 		FreeFloatHalfMtx( mtx, njob );
 	}
 	else
@@ -1668,7 +1756,7 @@ int main( int argc, char *argv[] )
 #if 0
 		utree = 0; counteff( njob, topol, len, eff ); utree = 1;
 #else
-		counteff_simple_float( njob, topol, len, eff );
+		counteff_simple_float_nostatic( njob, topol, len, eff );
 #endif
 	}
 	else
@@ -1697,8 +1785,9 @@ int main( int argc, char *argv[] )
 			if( alignmentlength != strlen( seq[i] ) )
 			{
 				fprintf( stderr, "#################################################################################\n" );
-				fprintf( stderr, "# ERROR!                                                                        #\n" );
-				fprintf( stderr, "# The original%4d sequences must be aligned                                    #\n", njob-nadd );
+				fprintf( stderr, "# ERROR!\n" );
+				fprintf( stderr, "# The original %d sequences must be aligned\n", njob-nadd );
+				fprintf( stderr, "# alignmentlength = %d, but strlen(seq[%d])=%d\n", alignmentlength, i, (int)strlen( seq[i] ) );
 				fprintf( stderr, "#################################################################################\n" );
 				exit( 1 );
 			}
@@ -1711,9 +1800,9 @@ int main( int argc, char *argv[] )
 				if( alignmentlength != strlen( seq[i] ) )
 				{
 					fprintf( stderr, "###############################################################################\n" );
-					fprintf( stderr, "# ERROR!                                                                      #\n" );
-					fprintf( stderr, "# The%4d additional sequences must be aligned                                #\n", nadd );
-					fprintf( stderr, "# Otherwise, try the '--add' option, instead of '--addprofile' option.        #\n" );
+					fprintf( stderr, "# ERROR!\n" );
+					fprintf( stderr, "# The %d additional sequences must be aligned\n", nadd );
+					fprintf( stderr, "# Otherwise, try the '--add' option, instead of '--addprofile' option.\n" );
 					fprintf( stderr, "###############################################################################\n" );
 					exit( 1 );
 				}
@@ -1741,10 +1830,10 @@ int main( int argc, char *argv[] )
 			if( !foundthebranch )
 			{
 				fprintf( stderr, "###############################################################################\n" );
-				fprintf( stderr, "# ERROR!                                                                      #\n" );
-				fprintf( stderr, "# There is no appropriate position to add the%4d sequences in the guide tree.#\n", nadd );
-				fprintf( stderr, "# Check whether the%4d sequences form a monophyletic cluster.                #\n", nadd );
-				fprintf( stderr, "# If not, try the '--add' option, instead of the '--addprofile' option.       #\n" );
+				fprintf( stderr, "# ERROR!\n" );
+				fprintf( stderr, "# There is no appropriate position to add the %d sequences in the guide tree.\n", nadd );
+				fprintf( stderr, "# Check whether the %d sequences form a monophyletic cluster.\n", nadd );
+				fprintf( stderr, "# If not, try the '--add' option, instead of the '--addprofile' option.\n" );
 				fprintf( stderr, "############################################################################### \n" );
 				exit( 1 );
 			}
@@ -1810,6 +1899,126 @@ int main( int argc, char *argv[] )
 		commongappick( njob-nadd, seq );
 		for( i=0; i<njob-nadd; i++ ) strcpy( bseq[i], seq[i] );
 	}
+//--------------- kokokara ----
+	else if( subalignment )
+	{
+		for( i=0; i<njob-1; i++ ) mergeoralign[i] = 'a';
+		for( i=0; i<nsubalignments; i++ )
+		{
+			fprintf( stderr, "Checking subalignment %d:\n", i+1 );
+			alignmentlength = strlen( seq[subtable[i][0]] );
+//			for( j=0; subtable[i][j]!=-1; j++ )
+//				fprintf( stderr, " %d. %-30.30s\n", subtable[i][j]+1, name[subtable[i][j]]+1 );
+			for( j=0; subtable[i][j]!=-1; j++ )
+			{
+				if( subtable[i][j] >= njob ) // check sumi
+				{
+					fprintf( stderr, "No such sequence, %d.\n", subtable[i][j]+1 );
+					exit( 1 );
+				}
+				if( alignmentlength != strlen( seq[subtable[i][j]] ) )
+				{
+					fprintf( stderr, "\n" );
+					fprintf( stderr, "###############################################################################\n" );
+					fprintf( stderr, "# ERROR!\n" );
+					fprintf( stderr, "# Subalignment %d must be aligned.\n", i+1 );
+					fprintf( stderr, "# Please check the alignment lengths of following sequences.\n" );
+					fprintf( stderr, "#\n" );
+					fprintf( stderr, "# %d. %-10.10s -> %d letters (including gaps)\n", subtable[i][0]+1, name[subtable[i][0]]+1, alignmentlength );
+					fprintf( stderr, "# %d. %-10.10s -> %d letters (including gaps)\n", subtable[i][j]+1, name[subtable[i][j]]+1, (int)strlen( seq[subtable[i][j]] ) );
+					fprintf( stderr, "#\n" );
+					fprintf( stderr, "# See http://mafft.cbrc.jp/alignment/software/merge.html for details.\n" );
+					if( subalignmentoffset )
+					{
+						fprintf( stderr, "#\n" );
+						fprintf( stderr, "# You specified seed alignment(s) consisting of %d sequences.\n", subalignmentoffset );
+						fprintf( stderr, "# In this case, the rule of numbering is:\n" );
+						fprintf( stderr, "#   The aligned seed sequences are numbered as 1 .. %d\n", subalignmentoffset );
+						fprintf( stderr, "#   The input sequences to be aligned are numbered as %d .. %d\n", subalignmentoffset+1, subalignmentoffset+njob );
+					}
+					fprintf( stderr, "###############################################################################\n" );
+					fprintf( stderr, "\n" );
+					exit( 1 );
+				}
+				insubtable[subtable[i][j]] = 1;
+			}
+			for( j=0; j<njob-1; j++ )
+			{
+				if( includemember( topol[j][0], subtable[i] ) && includemember( topol[j][1], subtable[i] ) )
+				{
+					mergeoralign[j] = 'n';
+				}
+			}
+			foundthebranch = 0;
+			for( j=0; j<njob-1; j++ )
+			{
+				if( samemember( topol[j][0], subtable[i] ) || samemember( topol[j][1], subtable[i] ) )
+				{
+					foundthebranch = 1;
+					fprintf( stderr, " -> OK\n" );
+					break;
+				}
+			}
+			if( !foundthebranch )
+			{
+				system( "cp infile.tree GuideTree" ); // tekitou
+				fprintf( stderr, "\n" );
+				fprintf( stderr, "###############################################################################\n" );
+				fprintf( stderr, "# ERROR!\n" );
+				fprintf( stderr, "# Subalignment %d does not seem to form a monophyletic cluster\n", i+1 );
+				fprintf( stderr, "# in the guide tree ('GuideTree' in this directory) internally computed.\n" );
+				fprintf( stderr, "# If you really want to use this subalignment, pelase give a tree with --treein \n" );
+				fprintf( stderr, "# http://mafft.cbrc.jp/alignment/software/treein.html\n" );
+				fprintf( stderr, "# http://mafft.cbrc.jp/alignment/software/merge.html\n" );
+				if( subalignmentoffset )
+				{
+					fprintf( stderr, "#\n" );
+					fprintf( stderr, "# You specified seed alignment(s) consisting of %d sequences.\n", subalignmentoffset );
+					fprintf( stderr, "# In this case, the rule of numbering is:\n" );
+					fprintf( stderr, "#   The aligned seed sequences are numbered as 1 .. %d\n", subalignmentoffset );
+					fprintf( stderr, "#   The input sequences to be aligned are numbered as %d .. %d\n", subalignmentoffset+1, subalignmentoffset+njob );
+				}
+				fprintf( stderr, "############################################################################### \n" );
+				fprintf( stderr, "\n" );
+				exit( 1 );
+			}
+//			commongappick( seq[subtable[i]], subalignment[i] ); // irukamo
+		}
+#if 0
+		for( i=0; i<njob-1; i++ )
+		{
+			fprintf( stderr, "STEP %d\n", i+1 );
+			fprintf( stderr, "group1 = " );
+			for( j=0; topol[i][0][j] != -1; j++ )
+				fprintf( stderr, "%d ", topol[i][0][j]+1 );
+			fprintf( stderr, "\n" );
+			fprintf( stderr, "group2 = " );
+			for( j=0; topol[i][1][j] != -1; j++ )
+				fprintf( stderr, "%d ", topol[i][1][j]+1 );
+			fprintf( stderr, "\n" );
+			fprintf( stderr, "%d -> %c\n\n", i, mergeoralign[i] );
+		}
+#endif
+
+		for( i=0; i<njob; i++ ) 
+		{
+			if( insubtable[i] ) strcpy( bseq[i], seq[i] );
+			else gappick0( bseq[i], seq[i] );
+		}
+
+		for( i=0; i<nsubalignments; i++ ) 
+		{
+			for( j=0; subtable[i][j]!=-1; j++ ) subalnpt[i][j] = bseq[subtable[i][j]];
+			if( !preservegaps[i] ) commongappick( j, subalnpt[i] );
+		}
+
+		FreeIntMtx( subtable );
+		free( insubtable );
+		for( i=0; i<nsubalignments; i++ ) free( subalnpt[i] );
+		free( subalnpt );
+		free( preservegaps );
+	}
+//--------------- kokomade ----
 	else
 	{
 		for( i=0; i<njob; i++ ) gappick0( bseq[i], seq[i] );
@@ -1855,6 +2064,7 @@ int main( int argc, char *argv[] )
 			targ[i].effarr = eff;
 			targ[i].alloclenpt = &alloclen;
 			targ[i].fftlog = fftlog;
+			targ[i].mergeoralign = mergeoralign;
 			targ[i].mutex = &mutex;
 			targ[i].treecond = &treecond;
 
