@@ -3,6 +3,8 @@
 #define TEST 0
 
 static int treeout = 0;
+static int maxdist = 1;
+static int nadd = 0;
 
 #ifdef enablemultithread
 typedef struct _jobtable
@@ -34,6 +36,9 @@ void *athread( void *arg )
 
 	int i, j;
 	float ssi, ssj, bunbo;
+	double mtxv;
+
+	if( njob == 1 ) return( NULL );
 	
 	while( 1 )
 	{
@@ -41,17 +46,21 @@ void *athread( void *arg )
 		j = jobpospt->j;
 		i = jobpospt->i;
 		j++;
+//		fprintf( stderr, "\n i=%d, j=%d before check\n", i, j );
 		if( j == njob )
 		{
+//			fprintf( stderr, "\n j = %d, i = %d, njob = %d\n", j, i, njob );
 			fprintf( stderr, "%4d/%4d (thread %4d), dndpre\r", i+1, njob, thread_no );
 			i++;
 			j = i + 1;
 			if( i == njob-1 )
 			{
+//				fprintf( stderr, "\n i=%d, njob-1=%d\n", i, njob-1 );
 				pthread_mutex_unlock( targ->mutex );
 				return( NULL );
 			}
 		}
+//		fprintf( stderr, "\n i=%d, j=%d after check\n", i, j );
 		jobpospt->j = j;
 		jobpospt->i = i;
 		pthread_mutex_unlock( targ->mutex );
@@ -61,9 +70,27 @@ void *athread( void *arg )
 
 		bunbo = MIN( ssi, ssj );
 		if( bunbo == 0.0 )
-			mtx[i][j] = 1.0;
+			mtxv = maxdist;
 		else
-			mtx[i][j] = 1.0 - (double)naivepairscore11( seq[i], seq[j], penalty ) / bunbo;
+			mtxv = maxdist * ( 1.0 - (double)naivepairscore11( seq[i], seq[j], penalty ) / bunbo );
+#if 1
+		if( mtxv > 9.0 || mtxv < 0.0 )
+		{
+			fprintf( stderr, "Distance %d-%d is strange, %f.\n", i, j, mtxv );
+			exit( 1 );
+		}
+#else // CHUUI!!!  2012/05/16
+		if( mtxv > 2.0 )
+		{
+			mtxv = 2.0;
+		}
+		if( mtxv < 0.0 )
+		{
+			fprintf( stderr, "Distance %d-%d is strange, %f.\n", i, j, mtxv );
+			exit( 1 );
+		}
+#endif
+		mtx[i][j] = mtxv;
 	}
 }
 
@@ -73,6 +100,7 @@ void arguments( int argc, char *argv[] )
 {
     int c;
 
+	nadd = 0;
 	nthread = 1;
 	alg = 'X';
 	fmodel = 0;
@@ -105,13 +133,62 @@ void arguments( int argc, char *argv[] )
 				case 'P':
 					dorp = 'p';
 					break;
+				case 'K': // Hontou ha iranai. disttbfast.c, tbfast.c to awaserutame.
+					break;
+				case 'I':
+					nadd = myatoi( *++argv );
+					fprintf( stderr, "nadd = %d\n", nadd );
+					--argc;
+					goto nextoption;
+				case 'f':
+					ppenalty = (int)( atof( *++argv ) * 1000 - 0.5 );
+					--argc;
+					goto nextoption;
+				case 'g':
+					ppenalty_ex = (int)( atof( *++argv ) * 1000 - 0.5 );
+					--argc;
+					goto nextoption;
+				case 'h':
+					poffset = (int)( atof( *++argv ) * 1000 - 0.5 );
+					--argc;
+					goto nextoption;
+				case 'k':
+					kimuraR = myatoi( *++argv );
+//					fprintf( stderr, "kimuraR = %d\n", kimuraR );
+					--argc;
+					goto nextoption;
+				case 'b':
+					nblosum = myatoi( *++argv );
+					scoremtx = 1;
+//					fprintf( stderr, "blosum %d\n", nblosum );
+					--argc;
+					goto nextoption;
+				case 'j':
+					pamN = myatoi( *++argv );
+					scoremtx = 0;
+					TMorJTT = JTT;
+					fprintf( stderr, "jtt %d\n", pamN );
+					--argc;
+					goto nextoption;
+				case 'm':
+					pamN = myatoi( *++argv );
+					scoremtx = 0;
+					TMorJTT = TM;
+					fprintf( stderr, "TM %d\n", pamN );
+					--argc;
+					goto nextoption;
 				case 'i':
 					inputfile = *++argv;
 					fprintf( stderr, "inputfile = %s\n", inputfile );
 					--argc;
 					goto nextoption;
+				case 'M':
+					maxdist = myatoi( *++argv );
+					fprintf( stderr, "maxdist = %d\n", maxdist );
+					--argc; 
+					goto nextoption;
 				case 'C':
-					nthread = atoi( *++argv );
+					nthread = myatoi( *++argv );
 					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
 					goto nextoption;
@@ -129,12 +206,13 @@ void arguments( int argc, char *argv[] )
 
 int main( int argc, char **argv )
 {
-	int i, j;
+	int i, j, ilim;
 	char **seq;
 	static char **name;
 	static int nlen[M];
 	float *selfscore;
 	double **mtx;
+	double mtxv;
 	FILE *fp;
 	FILE *infp;
 	float ssi, ssj, bunbo;
@@ -164,6 +242,8 @@ int main( int argc, char **argv )
 #endif
 	rewind( infp );
 
+	njob -= nadd; // atarashii hairetsu ha mushi
+
 	seq = AllocateCharMtx( njob, nlenmax+1 );
 	name = AllocateCharMtx( njob, B+1 );
 	mtx = AllocateDoubleMtx( njob, njob );
@@ -190,7 +270,6 @@ int main( int argc, char **argv )
 	for( i=0; i<njob; i++ )
 	{
 		selfscore[i] = (float)naivepairscore11( seq[i], seq[i], penalty );
-
 	}
 #ifdef enablemultithread
 	if( nthread > 0 )
@@ -229,7 +308,8 @@ int main( int argc, char **argv )
 	else
 #endif
 	{
-		for( i=0; i<njob-1; i++ )
+		ilim = njob-1;
+		for( i=0; i<ilim; i++ )
 		{
 			ssi = selfscore[i];
 			fprintf( stderr, "%4d/%4d\r", i+1, njob );
@@ -238,11 +318,30 @@ int main( int argc, char **argv )
 				ssj = selfscore[j];
 				bunbo = MIN( ssi, ssj );
 				if( bunbo == 0.0 )
-					mtx[i][j] = 1.0;
+					mtxv = maxdist;
 				else
-					mtx[i][j] = 1.0 - (double)naivepairscore11( seq[i], seq[j], penalty ) / bunbo;
-//					mtx[i][j] = 1.0 - (double)naivepairscore11( seq[i], seq[j], penalty ) / MIN( selfscore[i], selfscore[j] );
-//				fprintf( stderr, "i=%d,j=%d, l=%d### %f, score = %d\n", i, j, nlen[0], mtx[i][j], naivepairscore11( seq[i], seq[j], penalty )  );
+					mtxv = maxdist * ( 1.0 - (double)naivepairscore11( seq[i], seq[j], penalty ) / bunbo );
+//					mtxv = 1.0 - (double)naivepairscore11( seq[i], seq[j], penalty ) / MIN( selfscore[i], selfscore[j] );
+//				fprintf( stderr, "i=%d,j=%d, l=%d### %f, score = %f, %f, %f\n", i, j, nlen[0], mtxv, naivepairscore11( seq[i], seq[j], penalty ), ssi, ssj  );
+
+#if 1
+				if( mtxv > 9.0 || mtxv < 0.0 )
+				{
+					fprintf( stderr, "Distance %d-%d is strange, %f.\n", i, j, mtxv );
+					exit( 1 );
+				}
+#else // CHUUI!!!  2012/05/16
+				if( mtxv > 2.0 )
+				{
+					mtxv = 2.0;
+				}
+				if( mtxv < 0.0 )
+				{
+					fprintf( stderr, "Distance %d-%d is strange, %f.\n", i, j, mtxv );
+					exit( 1 );
+				}
+#endif
+				mtx[i][j] = mtxv;
 			}
 		}
 	}
