@@ -5,6 +5,8 @@
 
 extern char **seq_g;
 extern char **res_g;
+static int subalignment;
+static int subalignmentoffset;
 
 static int intop;
 static int intree;
@@ -72,6 +74,9 @@ void arguments( int argc, char *argv[] )
 	TMorJTT = JTT;
 	consweight_multi = 1.0;
 	consweight_rna = 0.0;
+	subalignment = 0;
+	subalignmentoffset = 0;
+	legacygapcost = 0;
 
 	while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -85,7 +90,7 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'I':
-					niter = atoi( *++argv );
+					niter = myatoi( *++argv );
 					fprintf( stderr, "niter = %d\n", niter );
 					--argc;
 					goto nextoption;
@@ -113,25 +118,25 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'k':
-					kimuraR = atoi( *++argv );
+					kimuraR = myatoi( *++argv );
 					fprintf( stderr, "kappa = %d\n", kimuraR );
 					--argc; 
 					goto nextoption;
 				case 'b':
-					nblosum = atoi( *++argv );
+					nblosum = myatoi( *++argv );
 					scoremtx = 1;
 					fprintf( stderr, "blosum %d / kimura 200\n", nblosum );
 					--argc; 
 					goto nextoption;
 				case 'j':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
 					fprintf( stderr, "jtt/kimura %d\n", pamN );
 					--argc; 
 					goto nextoption;
 				case 'm':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
 					fprintf( stderr, "tm %d\n", pamN );
@@ -152,12 +157,17 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'C':
-					nthread = atoi( *++argv );
+					nthread = myatoi( *++argv );
 					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
 					goto nextoption;
+				case 'H':
+					subalignment = 1;
+					subalignmentoffset = myatoi( *++argv );
+					--argc;
+					goto nextoption;
 				case 't':
-					randomseed = atoi( *++argv );
+					randomseed = myatoi( *++argv );
 					fprintf( stderr, "randomseed = %d\n", randomseed );
 					--argc; 
 					goto nextoption;
@@ -231,10 +241,10 @@ void arguments( int argc, char *argv[] )
 				case 's' :
 					treemethod = 's';
 					break;
-#endif
 				case 'H':
 					alg = 'H';
 					break;
+#endif
 				case 'A':
 					alg = 'A';
 					break;
@@ -270,6 +280,9 @@ void arguments( int argc, char *argv[] )
 				case 'Y':
 					score_check = 2;
 					break;
+				case 'L':
+					legacygapcost = 1;
+					break;
 #if 0
 				case 'n' :
 					treemethod = 'n';
@@ -288,11 +301,11 @@ void arguments( int argc, char *argv[] )
 					treemethod = 'q';
 					break;
 				case 'z':
-					fftThreshold = atoi( *++argv );
+					fftThreshold = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'w':
-					fftWinSize = atoi( *++argv );
+					fftWinSize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				default:
@@ -335,6 +348,7 @@ int main( int argc, char *argv[] )
 	double **eff;
 	FILE *prep;
 	FILE *infp;
+	FILE *orderfp;
 	int alloclen;
 	int returnvalue;
 	char c;
@@ -346,6 +360,14 @@ int main( int argc, char *argv[] )
 	static char **nogap1seq;
 	static char *kozoarivec;
 	int nkozo;
+	int alignmentlength;
+	int **skipthisbranch;
+	int foundthebranch;
+	int nsubalignments, maxmem;
+	int **subtable;
+	int *insubtable;
+	int *preservegaps;
+	char ***subalnpt;
 
 	arguments( argc, argv );
 #ifndef enablemultithread
@@ -380,6 +402,29 @@ int main( int argc, char *argv[] )
 		exit( 1 );
 	}
 
+
+	if( subalignment )
+	{
+		readsubalignmentstable( njob, NULL, NULL, &nsubalignments, &maxmem );
+		fprintf( stderr, "nsubalignments = %d\n", nsubalignments );
+		fprintf( stderr, "maxmem = %d\n", maxmem );
+		subtable = AllocateIntMtx( nsubalignments, maxmem+1 );
+		insubtable = AllocateIntVec( njob );
+		preservegaps = AllocateIntVec( njob );
+		for( i=0; i<njob; i++ ) insubtable[i] = 0;
+		for( i=0; i<njob; i++ ) preservegaps[i] = 0;
+		subalnpt = AllocateCharCub( nsubalignments, maxmem, 0 );
+		readsubalignmentstable( njob, subtable, preservegaps, NULL, NULL );
+		for( i=0; i<nsubalignments; i++ ) for( j=0; j<insubtable[i]; j++ )
+		{
+			if( subtable[i][j] < 0 )
+			{
+				fprintf( stderr, "Not supported in the iterative refinmenment mode.\n" );
+				fprintf( stderr, "Please use a positive number to represent a sequence.\n" );
+			}
+		}
+	}
+
 	ocut = cut;
 
 	segment = (Segment *)calloc( MAXSEG, sizeof( Segment ) );
@@ -398,6 +443,8 @@ int main( int argc, char *argv[] )
 	seq_g_bk = AllocateCharMtx( njob, 0 );
 	for( i=0; i<njob; i++ ) seq_g_bk[i] = seq_g[i];
 	kozoarivec = AllocateCharVec( njob );
+	skipthisbranch = AllocateIntMtx( njob, 2 );
+	for( i=0; i<njob; i++ ) skipthisbranch[i][0] = skipthisbranch[i][1] = 0;
 
 	if( constraint )
 	{
@@ -532,13 +579,14 @@ int main( int argc, char *argv[] )
 		{
 			gappick0( nogap1seq[0], seq_g[i] );
 			nogaplen = strlen( nogap1seq[0] );
-			singlerna[i] = (RNApair **)calloc( nogaplen, sizeof( RNApair * ) );
+			singlerna[i] = (RNApair **)calloc( nogaplen+1, sizeof( RNApair * ) );
 			for( j=0; j<nogaplen; j++ )
 			{
 				singlerna[i][j] = (RNApair *)calloc( 1, sizeof( RNApair ) );
 				singlerna[i][j][0].bestpos = -1;
 				singlerna[i][j][0].bestscore = -1.0;
 			}
+			singlerna[i][nogaplen] = NULL;
 			readmccaskill( prep, singlerna[i], nogaplen );
 		}
 		fclose( prep );
@@ -567,9 +615,15 @@ int main( int argc, char *argv[] )
 		}
 #endif
 		if( intree )
-			veryfastsupg_double_loadtree( njob, eff, topol, len );
+			veryfastsupg_double_loadtree( njob, eff, topol, len, name );
 		else if( intop ) // v6.528 deha if( intop ) dattanode intree ga mukou datta.
-			veryfastsupg_double_loadtop( njob, eff, topol, len );
+		{
+			fprintf( stderr, "--topin has been disabled\n" );
+			exit( 1 );
+//			veryfastsupg_double_loadtop( njob, eff, topol, len );
+		}
+		else if( subalignment )
+			fixed_supg_double_treeout_constrained( njob, eff, topol, len, name, nsubalignments, subtable );
 		else if( treemethod == 'X' || treemethod == 'E' || treemethod == 'q' ) 
 //			veryfastsupg_double_outtree( njob, eff, topol, len, name );
 			fixed_musclesupg_double_treeout( njob, eff, topol, len, name );
@@ -585,6 +639,22 @@ int main( int argc, char *argv[] )
 	printf( "utree = %d\n", utree );
 #endif
 
+	orderfp = fopen( "order", "w" );
+	if( !orderfp )
+	{
+		fprintf( stderr, "Cannot open 'order'\n" );
+		exit( 1 );
+	}
+	for( i=0; (j=topol[njob-2][0][i])!=-1; i++ )
+	{
+		fprintf( orderfp, "%d\n", j );
+	}
+	for( i=0; (j=topol[njob-2][1][i])!=-1; i++ )
+	{
+		fprintf( orderfp, "%d\n", j );
+	}
+	fclose( orderfp );
+
 
 	fprintf( stderr, "\n" );
 	if( ( !utree && kobetsubunkatsu ) || constraint || !bunkatsu )
@@ -616,6 +686,131 @@ int main( int argc, char *argv[] )
 #endif
 	}
 
+
+//--------------- kokokara ----
+	if( subalignment )
+	{
+		for( i=0; i<nsubalignments; i++ )
+		{
+			fprintf( stderr, "Checking subalignment %d:\n", i+1 );
+			alignmentlength = strlen( seq[subtable[i][0]] );
+//			for( j=0; subtable[i][j]!=-1; j++ )
+//				fprintf( stderr, " %d. %-30.30s\n", subtable[i][j]+1, name[subtable[i][j]]+1 );
+			for( j=0; subtable[i][j]!=-1; j++ )
+			{
+				if( subtable[i][j] >= njob )
+				{
+					fprintf( stderr, "No such sequence, %d.\n", subtable[i][j]+1 );
+					exit( 1 );
+				}
+				if( alignmentlength != strlen( seq[subtable[i][j]] ) )
+				{
+					fprintf( stderr, "\n" );
+					fprintf( stderr, "###############################################################################\n" );
+					fprintf( stderr, "# ERROR!\n" );
+					fprintf( stderr, "# Subalignment %d must be aligned.\n", i+1 );
+					fprintf( stderr, "# Please check the alignment lengths of following sequences.\n" );
+					fprintf( stderr, "#\n" );
+					fprintf( stderr, "# %d. %-10.10s -> %d letters (including gaps)\n", subtable[i][0]+1, name[subtable[i][0]]+1, alignmentlength );
+					fprintf( stderr, "# %d. %-10.10s -> %d letters (including gaps)\n", subtable[i][j]+1, name[subtable[i][j]]+1, (int)strlen( seq[subtable[i][j]] ) );
+					fprintf( stderr, "#\n" );
+					fprintf( stderr, "# See http://mafft.cbrc.jp/alignment/software/merge.html for details.\n" );
+					if( subalignmentoffset )
+					{
+						fprintf( stderr, "#\n" );
+						fprintf( stderr, "# You specified seed alignment(s) consisting of %d sequences.\n", subalignmentoffset );
+						fprintf( stderr, "# In this case, the rule of numbering is:\n" );
+						fprintf( stderr, "#   The aligned seed sequences are numbered as 1 .. %d\n", subalignmentoffset );
+						fprintf( stderr, "#   The input sequences to be aligned are numbered as %d .. %d\n", subalignmentoffset+1, subalignmentoffset+njob );
+					}
+					fprintf( stderr, "###############################################################################\n" );
+					fprintf( stderr, "\n" );
+					exit( 1 );
+				}
+				insubtable[subtable[i][j]] = 1;
+			}
+			for( j=0; j<njob-1; j++ )
+			{
+				if( includemember( topol[j][0], subtable[i] ) && !samemember( topol[j][0], subtable[i] ) )
+					skipthisbranch[j][0] = 1;
+				if( includemember( topol[j][1], subtable[i] ) && !samemember( topol[j][1], subtable[i] ) )
+					skipthisbranch[j][1] = 1;
+			}
+			foundthebranch = 0;
+			for( j=0; j<njob-1; j++ )
+			{
+				if( samemember( topol[j][0], subtable[i] ) || samemember( topol[j][1], subtable[i] ) )
+				{
+					foundthebranch = 1;
+					fprintf( stderr, " -> OK\n" );
+					break;
+				}
+			}
+			if( !foundthebranch )
+			{
+				system( "cp infile.tree GuideTree" ); // tekitou
+				fprintf( stderr, "\n" );
+				fprintf( stderr, "###############################################################################\n" );
+				fprintf( stderr, "# ERROR!\n" );
+				fprintf( stderr, "# Subalignment %d does not seem to form a monophyletic cluster\n", i+1 );
+				fprintf( stderr, "# in the guide tree ('GuideTree' in this directory) internally computed.\n" );
+				fprintf( stderr, "# If you really want to use this subalignment, pelase give a tree with --treein \n" );
+				fprintf( stderr, "# http://mafft.cbrc.jp/alignment/software/treein.html\n" );
+				fprintf( stderr, "# http://mafft.cbrc.jp/alignment/software/merge.html\n" );
+				if( subalignmentoffset )
+				{
+					fprintf( stderr, "#\n" );
+					fprintf( stderr, "# You specified seed alignment(s) consisting of %d sequences.\n", subalignmentoffset );
+					fprintf( stderr, "# In this case, the rule of numbering is:\n" );
+					fprintf( stderr, "#   The aligned seed sequences are numbered as 1 .. %d\n", subalignmentoffset );
+					fprintf( stderr, "#   The input sequences to be aligned are numbered as %d .. %d\n", subalignmentoffset+1, subalignmentoffset+njob );
+				}
+				fprintf( stderr, "############################################################################### \n" );
+				fprintf( stderr, "\n" );
+				exit( 1 );
+			}
+//			commongappick( seq[subtable[i]], subalignment[i] ); // irukamo
+		}
+#if 0
+		for( i=0; i<njob-1; i++ )
+		{
+			fprintf( stderr, "STEP %d\n", i+1 );
+			fprintf( stderr, "group1 = " );
+			for( j=0; topol[i][0][j] != -1; j++ )
+				fprintf( stderr, "%d ", topol[i][0][j]+1 );
+			fprintf( stderr, "\n" );
+			fprintf( stderr, "SKIP -> %d\n\n", skipthisbranch[i][0] );
+			fprintf( stderr, "group2 = " );
+			for( j=0; topol[i][1][j] != -1; j++ )
+				fprintf( stderr, "%d ", topol[i][1][j]+1 );
+			fprintf( stderr, "\n" );
+			fprintf( stderr, "SKIP -> %d\n\n", skipthisbranch[i][1] );
+		}
+#endif
+
+		for( i=0; i<njob; i++ ) 
+		{
+			if( insubtable[i] ) strcpy( bseq[i], seq[i] );
+			else gappick0( bseq[i], seq[i] );
+		}
+
+		for( i=0; i<nsubalignments; i++ ) 
+		{
+			for( j=0; subtable[i][j]!=-1; j++ ) subalnpt[i][j] = bseq[subtable[i][j]];
+			commongappick( j, subalnpt[i] );
+		}
+
+		FreeIntMtx( subtable );
+		free( insubtable );
+		for( i=0; i<nsubalignments; i++ ) free( subalnpt[i] );
+		free( subalnpt );
+		free( preservegaps );
+	}
+//--------------- kokomade ----
+
+
+
+
 	for( i=0; i<njob; i++ ) res_g[i][0] = 0;
 
 	for( iseg=0; iseg<nseg-1; iseg++ )
@@ -634,7 +829,7 @@ int main( int argc, char *argv[] )
 		fprintf( trap_g, "Segment %3d/%3d %4d-%4d\n", iseg+1, nseg-1, pos+1, pos+1+tmplen );
 	
 		cut = ocut;
-		returnvalue = TreeDependentIteration( njob, name, nlen, seq, bseq, topol, len, alloclen, localhomtable, singlerna, nkozo, kozoarivec );
+		returnvalue = TreeDependentIteration( njob, name, nlen, seq, bseq, topol, len, skipthisbranch, alloclen, localhomtable, singlerna, nkozo, kozoarivec );
 
 		for( i=0; i<njob; i++ )
 			strcat( res_g[i], bseq[i] );
@@ -643,7 +838,25 @@ int main( int argc, char *argv[] )
 	FreeIntCub( topol );
 	FreeDoubleMtx( len );
 	FreeDoubleMtx( eff );
+	FreeIntMtx( skipthisbranch );
+	free( kozoarivec );
 	if( constraint ) FreeLocalHomTable( localhomtable, njob );
+	if( rnakozo && rnaprediction == 'm' ) 
+	{
+		if( singlerna ) // nen no tame
+		{
+			for( i=0; i<njob; i++ ) 
+			{
+				for( j=0; singlerna[i][j]!=NULL; j++ )
+				{
+					if( singlerna[i][j] ) free( singlerna[i][j] );
+				}
+				if( singlerna[i] ) free( singlerna[i] );
+			}
+			free( singlerna );
+			singlerna = NULL;
+		}
+	}
 
 #if 0
 	Write( stdout, njob, name, nlen, bseq );
