@@ -14,8 +14,8 @@ extern void upg2( int nseq, double **eff, int ***topol, double **len );
 extern void veryfastsupg_int_realloc_nobk( int njob, int **mtx, int ***topol, double **len );
 extern void veryfastsupg( int nseq, double **oeff, int ***topol, double **len );
 extern void veryfastsupg_double( int nseq, double **oeff, int ***topol, double **len );
-extern void veryfastsupg_double_loadtree( int nseq, double **oeff, int ***topol, double **len );
-extern void veryfastsupg_double_loadtop( int nseq, double **oeff, int ***topol, double **len );
+extern void veryfastsupg_double_loadtree( int nseq, double **oeff, int ***topol, double **len, char **name );
+//extern void veryfastsupg_double_loadtop( int nseq, double **oeff, int ***topol, double **len );
 extern void veryfastsupg_int( int nseq, int **oeff, int ***topol, double **len );
 extern void fastsupg( int nseq, double **oeff, int ***topol, double **len );
 extern void supg( int nseq, double **oeff, int ***topol, double **len );
@@ -56,6 +56,8 @@ extern double DSPscore( int s, char **seq );
 extern int searchAnchors( int nseq, char **seq, Segment *seg );
 extern char *progName( char *str );
 extern void dontcalcimportance( int nseq, double *eff, char **seq, LocalHom **localhom );
+extern void dontcalcimportance_lastone( int nseq, double *eff, char **seq, LocalHom **localhom );
+extern void dontcalcimportance_firstone( int nseq, double *eff, char **seq, LocalHom **localhom );
 extern void calcimportance( int nseq, double *eff, char **seq, LocalHom **localhom );
 extern void weightimportance2( int nseq, double *eff, LocalHom **localhom );
 extern void weightimportance4( int clus1, int clus2, double *eff1, double *eff2, LocalHom ***localhom );
@@ -74,8 +76,7 @@ extern int fastconjuction( int *memlist, char **seq, char **aseq, double *peff,
 extern int fastconjuction_noname_kozo( int *memlist, char **seq, char **aseq, double *peff, double *eff, double *peff_kozo, double *eff_kozo, char *d );
 extern int fastconjuction_noname( int *memlist, char **seq, char **aseq, double *peff, double *eff, char *d );
 extern int fastconjuction_noweight( int *memlist, char **seq, char **aseq, double *peff, char *d );
-extern int conjuctionfortbfast( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char *d );
-extern int conjuctionfortbfast_kozo( double *tmptmp, char **pair, int s, char **seq, char **aseq, double *peff, double *eff, double *peff_kozo, double *eff_kozo, char *d );
+extern int conjuctionfortbfast_old( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char *d );
 extern int conjuction( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char **name, char **aname, char *d );
 extern void floatdelete( float **cpmx, int d, int len );
 extern void chardelete( char *seq, int d );
@@ -83,14 +84,16 @@ extern int RootBranchNode( int nseq, int ***topol, int step, int branch );
 extern void BranchLeafNode( int nseq, int ***topol, int *node, int step, int branch );
 extern void RootLeafNode( int nseq, int ***topol, int *node );
 extern void nodeFromABranch( int nseq, int *result, int **pairwisenode, int ***topol, double **len, int step, int num );
-extern void OneClusterAndTheOther( int locnjob, char **pair, int *s1, int *s2, int ***topol, int step, int branch );
+//extern void OneClusterAndTheOther( int locnjob, char **pair, int *s1, int *s2, int ***topol, int step, int branch );
+extern void OneClusterAndTheOther_fast( int locnjob, int *memlist1, int *memlist2, int *s1, int *s2, char *pairbuf, int ***topol, int step, int branch );
 extern void makeEffMtx( int nseq, double **mtx, double *vec );
 extern void node_eff( int nseq, double *eff, int *node );
 extern int shrinklocalhom( char **pair, int s1, int s2, LocalHom **localhom, LocalHom ***localhomshrink );
-extern int msshrinklocalhom( char **pair, int s1, int s2, LocalHom **localhom, LocalHom ***localhomshrink );
+extern int msshrinklocalhom_fast( int *memlist1, int *memlist2, LocalHom **localhom, LocalHom ***localhomshrink );
 extern int fastshrinklocalhom( int *mem1, int *mem2, LocalHom **localhom, LocalHom ***localhomshrink );
+extern int fastshrinklocalhom_one( int *mem1, int *mem2, int norg, LocalHom **localhom, LocalHom ***localhomshrink );
 extern int msfastshrinklocalhom( int *mem1, int *mem2, LocalHom **localhom, LocalHom ***localhomshrink );
-extern int TreeDependentIteration( int locnjob, char **name, int nlen[M], char **aseq, char **bseq, int ***topol, double **len, int alloclen, LocalHom **localhomtable, RNApair ***single, int nkozo, char *kozoarivec );
+extern int TreeDependentIteration( int locnjob, char **name, int nlen[M], char **aseq, char **bseq, int ***topol, double **len, int **skipthisbranch, int alloclen, LocalHom **localhomtable, RNApair ***single, int nkozo, char *kozoarivec );
 extern void checkMinusLength( int nseq, double **len );
 extern void negativeMember2( int *mem, int *query, int locnseq );
 extern int *negativeMember( int *query, int locnseq );
@@ -156,6 +159,7 @@ extern float partQ__align( char **seq1, char **seq2, double *eff1, double *eff2,
 extern float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp  );
 extern float G__align11_noalign( int mtx[0x80][0x80], int penal, int penal_ex, char **seq1, char **seq2, int alloclen );
 extern float L__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2pt );
+extern float L__align11_noalign( char **seq1, char **seq2 );
 extern float genL__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2pt );
 extern float genG__align11( char **seq1, char **seq2, int alloclen );
 extern float VAalign11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2pt, LocalHom *lhmpt );
@@ -168,8 +172,9 @@ extern void topolsort( int m, int s[] );
 extern void topolswap( int s1[], int s2[], int *mpt1, int *mpt2 );
 extern void reduc( double **mtx, int nseq, int im, int jm );
 extern void  nj( int nseq, double **omtx, int ***topol, double **dis );
-extern void JTTmtx( double **rsr, double *freq, char locamino[26], char locgrp[26], int isTM );
+extern void JTTmtx( double **rsr, double *freq, char locamino[0x80], char locgrp[0x80], int isTM );
 extern void BLOSUMmtx( int n, double **matrix, double *freq, char *amino, char *amino_grp );
+extern void extendedmtx( double **matrix, double *freq, char *amino, char *amino_grp );
 extern void putlocalhom2( char *al1, char *al2, LocalHom *localhompt, int off1, int off2, int opt, int overlapaa );
 extern void putlocalhom_str( char *al1, char *al2, double *equiv, double scale, LocalHom *localhompt, int off1, int off2, int opt, int overlapaa );
 extern void putlocalhom_ext( char *al1, char *al2, LocalHom *localhompt, int off1, int off2, int opt, int overlapaa );
@@ -218,12 +223,12 @@ extern void WriteHat2_pointer( FILE *hat2p, int locnjob, char **name, double **m
 extern void WriteHat2_part_pointer( FILE *hat2p, int locnjob, int nadd, char **name, double **mtx );
 extern int ReadFasta_sub( FILE *fp, double *dis, int nseq, char name[M][B] );
 extern int ReadSsearch( FILE *fp, double *dis, int nseq, char name[M][B] );
-extern int ReadBlastm7( FILE *fp, double *dis, int qmem, char name[M][B], LocalHom *localhomlist );
+extern int ReadBlastm7( FILE *fp, double *dis, int qmem, char **name, LocalHom *localhomlist );
 extern int ReadBlastm7_scoreonly( FILE *fp, double *dis, int nin );
 extern int ReadBlastm7_avscore( FILE *fp, double *dis, int nin );
-extern int ReadFasta34noalign( FILE *fp, double *dis, int qmem, char name[M][B], LocalHom *localhomlist );
-extern int ReadFasta34m10_nuc( FILE *fp, double *dis, int qmem, char name[M][B], LocalHom *localhomlist );
-extern int ReadFasta34m10( FILE *fp, double *dis, int qmem, char name[M][B], LocalHom *localhomlist );
+extern int ReadFasta34noalign( FILE *fp, double *dis, int qmem, char **name, LocalHom *localhomlist );
+extern int ReadFasta34m10_nuc( FILE *fp, double *dis, int qmem, char **name, LocalHom *localhomlist );
+extern int ReadFasta34m10( FILE *fp, double *dis, int qmem, char **name, LocalHom *localhomlist );
 extern int ReadFasta34m10_scoreonly_nuc( FILE *fp, double *dis, int nin );
 extern int ReadFasta34m10_scoreonly( FILE *fp, double *dis, int nin );
 extern int ReadFasta34( FILE *fp, double *dis, int nseq, char name[M][B], LocalHom *localhomlist );
@@ -235,15 +240,22 @@ extern int writePre( int nseq, char **name, int nlen[M], char **aseq, int force
 extern void readOtherOptions( int *ppidptr, int *fftThresholdptr, int *fftWinSizeptr );
 extern void initSignalSM( void );
 extern void initFiles( void );
-extern void WriteForFasta( FILE *fp, int locnjob, char name[][B], int nlen[M], char **aseq );
+extern void WriteForFasta( FILE *fp, int locnjob, char **name, int nlen[M], char **aseq );
 extern void readlocalhomtable( FILE*fp, int njob, LocalHom **localhomtable, char *kozoarivec );
 extern void readlocalhomtable2( FILE*fp, int njob, LocalHom **localhomtable, char *kozoarivec );
+extern void readlocalhomtable_part( FILE*fp, int njob, int nadd, LocalHom **localhomtable, char *kozoarivec );
+extern void readlocalhomtable_two( FILE*fp, int njob, int nadd, LocalHom **localhomtable, LocalHom **localhomtablex, char *kozoarivec );
+extern void readlocalhomtable_one( FILE*fp, int njob, int nadd, LocalHom **localhomtable, char *kozoarivec );
 extern void outlocalhom( LocalHom **localhom, int nseq );
+extern void outlocalhom_part( LocalHom **localhom, int norg, int nadd );
 extern void outlocalhompt( LocalHom ***localhom, int n1, int n2 );
 extern void FreeLocalHomTable( LocalHom **localhomtable, int n ) ;
+extern void FreeLocalHomTable_part( LocalHom **localhomtable, int n, int m ) ;
+extern void FreeLocalHomTable_two( LocalHom **localhomtable, int n, int m ) ;
+extern void FreeLocalHomTable_one( LocalHom **localhomtable, int n, int m ) ;
 extern void constants( int nseq, char **seq );
 extern void clustalout_pointer( FILE *fp, int nseq, int maxlen, char **seq, char **name, char *mark, char *comment, int *order, int namelen );
-extern void phylipout_pointer( FILE *fp, int nseq, int maxlen, char **seq, char **name, int *order );
+extern void phylipout_pointer( FILE *fp, int nseq, int maxlen, char **seq, char **name, int *order, int namelen );
 extern void writeData_reorder( FILE *fp, int locnjob, char name[][B], int nlen[], char **aseq, int *order );
 extern void writeData_reorder_pointer( FILE *fp, int locnjob, char **name, int *nlen, char **aseq, int *order );
 
@@ -291,12 +303,14 @@ extern void imp_rnaQ_gappick( int nseq1, int nseq2, char **seq1, char **seq2, do
 extern void foldalignedrna( int clus1, int clus2, char **mseq1, char **mseq2, double *effarr1, double *effarr2, RNApair *rnapairboth );
 void readmccaskill( FILE *fp, RNApair **pairprob, int length );
 void makegrouprna( RNApair ***group, RNApair ***all, int *memlist );
-void makegrouprnait( RNApair ***group, RNApair ***all, char **pair, int s );
+void makegrouprnait( RNApair ***group, RNApair ***all, char *pair, int s );
 extern void fixed_musclesupg_float_realloc_nobk_halfmtx( int nseq, float **eff, int ***topol, float **len, Treedep *, int progressout );
 extern void loadtree( int nseq, int ***topol, float **len, char **name, int *nlen, Treedep * );
-extern void loadtop( int nseq, float **eff, int ***topol, float **len );
+//extern void loadtop( int nseq, float **eff, int ***topol, float **len );
 extern void fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( int nseq, float **eff, int ***topol, float **len, char **name, int *nlen, Treedep * );
+extern void fixed_supg_float_realloc_nobk_halfmtx_treeout_constrained( int nseq, float **eff, int ***topol, float **len, char **name, int *nlen, Treedep *, int ncons, int **constraints );
 extern void fixed_musclesupg_double_treeout( int nseq, double **eff, int ***topol, double **len, char **name );
+extern void fixed_supg_double_treeout_constrained( int nseq, double **eff, int ***topol, double **len, char **name, int ncons, int **constraints );
 extern void imp_match_init_strict( float *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, double *eff1kozo, double*eff2kozo, LocalHom ***localhom, int forscore );
 extern void miyataout_reorder_pointer( FILE *fp, int locnjob, int nlenmax, char **name, int *nlen, char **aseq, int *order );
 extern void veryfastsupg_double_outtree( int nseq, double **eff, int ***topol, double **len, char **name );
@@ -319,6 +333,15 @@ extern void insertnewgaps( int njob, int *alreadyaligned, char **seq, int *ex1,
 extern void restorecommongaps( int n, char **seq, int *top0, int *top1, int *gaplen, int alloclen, char gapchar );
 extern int samemember( int *mem, int *cand );
 extern int includemember( int *mem, int *cand );
+extern int overlapmember( int *mem1, int *mem2 );
 extern void profilealignment( int n0, int n1, int n2, char **aln0, char **aln1, char **aln2, int alloclen, char alg );
+extern void profilealignment2( int n0, int n2, char **aln0, char **aln2, int alloclen, char alg );
 extern void sreverse( char *r, char *s );
-extern void addonetip( int njobc, int ***topolc, float **lenc, float **iscorec, int ***topol, float **len, Treedep *dep, int treeout, int iadd, char **name );
+extern int addonetip( int njobc, int ***topolc, float **lenc, float **iscorec, int ***topol, float **len, Treedep *dep, int treeout, Addtree *addtree, int iadd, char **name );
+extern void intcpy( int *s1, int *s2 );
+extern void intcat( int *s1, int *s2 );
+extern void readsubalignmentstable( int n, int **table, int *preservegaps, int *nsubpt, int *maxmempt );
+extern int myatoi( char * );
+extern void gapcount( double *freq, char **seq, int nseq, double *eff, int lgth );
+extern void gapcountf( float *freq, char **seq, int nseq, double *eff, int lgth );
+extern void outgapcount( float *freq, int nseq, char *gappat, double *eff );
