@@ -27,66 +27,6 @@ static void match_calc( float *match, char **s1, char **s2, int i1, int lgth2 )
 }
 #endif
 
-#if 0
-static void match_calc_bk( float *match, float **cpmx1, float **cpmx2, int i1, int lgth2, float **floatwork, int **intwork, int initialize )
-{
-	int j, k, l;
-	float scarr[26];
-	float **cpmxpd = floatwork;
-	int **cpmxpdn = intwork;
-	int count = 0;
-
-	if( initialize )
-	{
-		for( j=0; j<lgth2; j++ )
-		{
-			count = 0;
-			for( l=0; l<26; l++ )
-			{
-				if( cpmx2[l][j] )
-				{
-					cpmxpd[count][j] = cpmx2[l][j];
-					cpmxpdn[count][j] = l;
-					count++;
-				}
-			}
-			cpmxpdn[count][j] = -1;
-		}
-	}
-
-	for( l=0; l<26; l++ )
-	{
-		scarr[l] = 0.0;
-		for( k=0; k<26; k++ )
-			scarr[l] += n_dis[k][l] * cpmx1[k][i1];
-	}
-#if 0 
-	{
-		float *fpt, **fptpt, *fpt2;
-		int *ipt, **iptpt;
-		fpt2 = match;
-		iptpt = cpmxpdn;
-		fptpt = cpmxpd;
-		while( lgth2-- )
-		{
-			*fpt2 = 0.0;
-			ipt=*iptpt,fpt=*fptpt;
-			while( *ipt > -1 )
-				*fpt2 += scarr[*ipt++] * *fpt++;
-			fpt2++,iptpt++,fptpt++;
-		} 
-	}
-#else
-	for( j=0; j<lgth2; j++ )
-	{
-		match[j] = 0.0;
-		for( k=0; cpmxpdn[k][j]>-1; k++ )
-			match[j] += scarr[cpmxpdn[k][j]] * cpmxpd[k][j];
-	} 
-#endif
-}
-#endif
-
 static float gentracking( float *lasthorizontalw, float *lastverticalw, 
 						char **seq1, char **seq2, 
                         char **mseq1, char **mseq2, 
@@ -220,6 +160,38 @@ float genL__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *o
 	float localthr = -foffset;
 	float localthr2 = -foffset;
 
+	if( seq1 == NULL )
+	{
+		if( orlgth1 > 0 && orlgth2 > 0 )
+		{
+			orlgth1 = 0;
+			orlgth2 = 0;
+			free( mseq1 );
+			free( mseq2 );
+			FreeFloatVec( w1 );
+			FreeFloatVec( w2 );
+			FreeFloatVec( match );
+			FreeFloatVec( initverticalw );
+			FreeFloatVec( lastverticalw );
+
+			FreeFloatVec( m );
+			FreeIntVec( mp );
+			free( largeM );
+			free( Mp );
+
+			FreeCharMtx( mseq );
+
+			FreeFloatMtx( cpmx1 );
+			FreeFloatMtx( cpmx2 );
+
+			FreeFloatMtx( floatwork );
+			FreeIntMtx( intwork );
+
+		}
+		return( 0.0 );
+	}
+
+
 
 //	fprintf( stderr, "@@@@@@@@@@@@@ penalty_OP = %f, penalty_EX = %f, pelanty = %f\n", fpenalty_OP, fpenalty_EX, fpenalty );
 
@@ -280,11 +252,11 @@ float genL__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *o
 
 		mseq = AllocateCharMtx( njob, ll1+ll2 );
 
-		cpmx1 = AllocateFloatMtx( 26, ll1+2 );
-		cpmx2 = AllocateFloatMtx( 26, ll2+2 );
+		cpmx1 = AllocateFloatMtx( nalphabets, ll1+2 );
+		cpmx2 = AllocateFloatMtx( nalphabets, ll2+2 );
 
-		floatwork = AllocateFloatMtx( 26, MAX( ll1, ll2 )+2 ); 
-		intwork = AllocateIntMtx( 26, MAX( ll1, ll2 )+2 ); 
+		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
+		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 
 #if DEBUG
 		fprintf( stderr, "succeeded\n" );
