@@ -27,7 +27,7 @@ int intcmp( int *str1, int *str2 )
 
 char **arguments( int argc, char *argv[] )
 {
-    int c;
+    int c = 0;
 	
 	fmodel = 0;
 	nblosum = 62;
@@ -109,11 +109,11 @@ char **arguments( int argc, char *argv[] )
 					tbrweight = 0;
 					break;
 				case 'z':
-					fftThreshold = atoi( *++argv );
+					fftThreshold = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'w':
-					fftWinSize = atoi( *++argv );
+					fftWinSize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'Z':
@@ -135,25 +135,25 @@ char **arguments( int argc, char *argv[] )
                     --argc;
                     goto nextoption;
                 case 'k':
-                    kimuraR = atoi( *++argv );
+                    kimuraR = myatoi( *++argv );
                     fprintf( stderr, "kappa = %d\n", kimuraR );
                     --argc;
                     goto nextoption;
 				case 'b':
-					nblosum = atoi( *++argv );
+					nblosum = myatoi( *++argv );
 					scoremtx = 1;
 					fprintf( stderr, "blosum %d\n", nblosum );
 					--argc;
 					goto nextoption;
 				case 'j':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
 					fprintf( stderr, "jtt %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'm':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
 					fprintf( stderr, "tm %d\n", pamN );
