@@ -65,12 +65,12 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'I':
-					nadd = atoi( *++argv );
+					nadd = myatoi( *++argv );
 					fprintf( stderr, "nadd = %d\n", nadd );
 					--argc;
 					goto nextoption;
 				case 'C':
-					nthread = atoi( *++argv );
+					nthread = myatoi( *++argv );
 					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
 					goto nextoption;
@@ -90,12 +90,12 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'k':
-					kimuraR = atoi( *++argv );
+					kimuraR = myatoi( *++argv );
 					fprintf( stderr, "kappa = %d\n", kimuraR );
 					--argc;
 					goto nextoption;
 				case 'j':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
 					fprintf( stderr, "jtt/kimura %d\n", pamN );
@@ -375,7 +375,8 @@ static void	*directionthread( void *arg )
 //			strcpy( mseq2[0], seq[j] );
 			mseq1[0] = tmpseq;
 			mseq2[0] = seq[j];
-			res[j] = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, 0 );
+//			res[j] = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, 0 );
+			res[j] = L__align11_noalign( mseq1, mseq2 );
 		}
 		else
 			res[j] = commonsextet_p2( table1, spointt[j] );
@@ -384,7 +385,8 @@ static void	*directionthread( void *arg )
 	{
 		free( mseq1 );
 		free( mseq2 );
-		G__align11_noalign( NULL, 0, 0, NULL, NULL, 0 );
+//		G__align11_noalign( NULL, 0, 0, NULL, NULL, 0 );
+		L__align11_noalign( NULL, NULL );
 	}
 //	else
 //		if( nthread )  // inthread == 0 no toki free suru to, error. nazeda
@@ -510,8 +512,11 @@ int main( int argc, char *argv[] )
 		{
 			gappick0( mseq2[0], seq[i] );
 
-			res_forward = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1f, mseq2, 0 );
-			res_reverse = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1r, mseq2, 0 );
+//			res_forward = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1f, mseq2, 0 );
+//			res_reverse = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1r, mseq2, 0 );
+
+			res_forward = L__align11_noalign( mseq1f, mseq2 );
+			res_reverse = L__align11_noalign( mseq1r, mseq2 );
 #if 0
 
 			strcpy( mseq2[0], seq[i] );
