@@ -101,11 +101,14 @@ static void *athread( void *arg )
 		if( i == njob )
 		{
 			pthread_mutex_unlock( targ->mutex );
-			return( NULL );
+//			return( NULL );
+			break;
 		}
 		*jobpospt = i+1;
 		pthread_mutex_unlock( targ->mutex );
 
+		commongappick_record( 1, nogap+i, gapmap[i] );
+		if( strlen( nogap[i] ) == 0 ) continue;
 
 		sprintf( dirname, "_%d", i );
 		sprintf( com, "rm -rf %s", dirname );
@@ -114,7 +117,6 @@ static void *athread( void *arg )
 		system( com );
 
 		fprintf( stderr, "%d / %d (by thread %4d)\n", i+1, njob, thread_no );
-		commongappick_record( 1, nogap+i, gapmap[i] );
 		sprintf( com, "%s/_mccaskillinorg", dirname );
 		infp = fopen( com, "w" );
 //		fprintf( infp, ">in\n%s\n", nogap[i] );
@@ -152,6 +154,7 @@ static void *athread( void *arg )
 	}
 	free( dirname );
 	free( com );
+	return( NULL );
 }
 #endif
 
@@ -183,7 +186,7 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'C':
-					nthread = atoi( *++argv );
+					nthread = myatoi( *++argv );
 					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
 					goto nextoption;
@@ -334,6 +337,9 @@ int main( int argc, char *argv[] )
 		}
 		pthread_mutex_destroy( &mutex );
 
+		free( handle );
+		free( targ );
+
 
 		for( i=0; i<njob; i++ )
 		{
@@ -348,6 +354,12 @@ int main( int argc, char *argv[] )
 		{
 			fprintf( stderr, "%d / %d\n", i+1, njob );
 			commongappick_record( 1, nogap+i, gapmap[i] );
+			if( strlen( nogap[i] ) == 0 ) 
+			{
+				fprintf( stdout, ">%d\n", i );
+				continue;
+			}
+
 			infp = fopen( "_mccaskillinorg", "w" );
 //			fprintf( infp, ">in\n%s\n", nogap[i] );
 			fprintf( infp, ">in\n" );
