@@ -20,8 +20,8 @@
 #include <pthread.h>
 #endif
 
-#define VERSION "6.940b"
-#define SHOWVERSION fprintf( stderr, "%s (%s) Version " VERSION " alg=%c, model=%s\n%d thread(s)\n", progName( argv[0] ), (dorp=='d')?"nuc":"aa", alg, modelname, nthread )
+#define VERSION "7.123b"
+#define SHOWVERSION fprintf( stderr, "%s (%s) Version " VERSION " alg=%c, model=%s\n%d thread(s)\n", progName( argv[0] ), (dorp=='d')?"nuc":((nblosum==-2)?"text":"aa"), alg, modelname, nthread )
 
 #define FFT_THRESHOLD  80
 #define FFT_WINSIZE_P   20
@@ -58,12 +58,12 @@ extern char amino_grp[0x80];
 extern int amino_dis[0x80][0x80];
 extern int amino_disLN[0x80][0x80];
 extern double amino_dis_consweight_multi[0x80][0x80];
-extern int n_dis[26][26];
-extern int n_disFFT[26][26];
-extern float n_dis_consweight_multi[26][26];
-extern char amino[26];
-extern double polarity[20];
-extern double volume[20];
+extern int n_dis[0x80][0x80];
+extern int n_disFFT[0x80][0x80];
+extern float n_dis_consweight_multi[0x80][0x80];
+extern char amino[0x80];
+extern double polarity[0x80];
+extern double volume[0x80];
 extern int ribosumdis[37][37];
 
 extern int ppid;
@@ -274,6 +274,14 @@ typedef struct _Treedep
 	float distfromtip;
 } Treedep;
 
+typedef struct _Addtree
+{
+	int nearest;
+	float dist1;
+	char *neighbors;
+	float dist2;
+} Addtree;
+
 
 #include "fft.h"
 #include "dp.h"
@@ -300,4 +308,9 @@ extern int parallelizationstrategy;
 extern int scoreout;
 extern int outnumber;
 
+extern int legacygapcost;
+
 extern TLS char *newgapstr;
+
+extern int nalphabets;
+extern int nscoredalphabets;
