@@ -79,12 +79,12 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 't':
-					nhomologs = atoi( *++argv );
+					nhomologs = myatoi( *++argv );
 					fprintf( stderr, "nhomologs = %d\n", nhomologs );
 					--argc;
 					goto nextoption;
 				case 'o':
-					seedoffset = atoi( *++argv );
+					seedoffset = myatoi( *++argv );
 					fprintf( stderr, "seedoffset = %d\n", seedoffset );
 					--argc;
 					goto nextoption;
@@ -133,7 +133,7 @@ int countamino( char *s, int end )
 	return( val );
 }
 
-static void pairalign( char name[M][B], int nlen[M], char **seq, double *effarr, int alloclen )
+static void pairalign( char **name, int nlen[M], char **seq, double *effarr, int alloclen )
 {
 	int i, j;
 	FILE *hat3p;
@@ -298,7 +298,7 @@ static void WriteOptions( FILE *fp )
 int main( int argc, char *argv[] )
 {
 	static int  nlen[M];	
-	static char name[M][B], **seq;
+	static char **name, **seq;
 	static char **bseq;
 	static double *eff;
 	int i;
@@ -330,6 +330,7 @@ int main( int argc, char *argv[] )
 		exit( 1 );
 	}
 
+	name = AllocateCharMtx( njob, B+1 );
 	seq = AllocateCharMtx( njob, nlenmax*9+1 );
 	bseq = AllocateCharMtx( njob, nlenmax*9+1 );
 	alloclen = nlenmax*9;
@@ -339,7 +340,7 @@ int main( int argc, char *argv[] )
 #if 0
 	Read( name, nlen, seq );
 #else
-	readData( infp, name, nlen, seq );
+	readData_pointer( infp, name, nlen, seq );
 #endif
 	fclose( infp );
 
@@ -370,7 +371,8 @@ int main( int argc, char *argv[] )
 	for( i=0; i<njob; i++ ) gappick0( bseq[i], seq[i] );
 
 
-	for( i=0; i<njob; i++ ) fprintf( stdout, ">_seed_%s\n%s\n", name[i]+1, bseq[i] );
+//	for( i=0; i<njob; i++ ) fprintf( stdout, ">_seed_%s\n%s\n", name[i]+1, bseq[i] ); // CHUUI!!
+	for( i=0; i<njob; i++ ) fprintf( stdout, ">_seed_%s\n%s\n", name[i]+1, seq[i] );
 
 	pairalign( name, nlen, seq, eff, alloclen );
 
