@@ -828,25 +828,25 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'k':
-					kimuraR = atoi( *++argv );
+					kimuraR = myatoi( *++argv );
 //					fprintf( stderr, "kimuraR = %d\n", kimuraR );
 					--argc;
 					goto nextoption;
 				case 'b':
-					nblosum = atoi( *++argv );
+					nblosum = myatoi( *++argv );
 					scoremtx = 1;
 //					fprintf( stderr, "blosum %d\n", nblosum );
 					--argc;
 					goto nextoption;
 				case 'j':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
 					fprintf( stderr, "jtt %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'm':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
 					fprintf( stderr, "TM %d\n", pamN );
@@ -868,15 +868,15 @@ void arguments( int argc, char *argv[] )
 					--argc; 
 					goto nextoption;
 				case 't':
-					equivthreshold = atoi( *++argv );
+					equivthreshold = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'w':
-					equivwinsize = atoi( *++argv );
+					equivwinsize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'l':
-					equivshortestlen = atoi( *++argv );
+					equivshortestlen = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 's':
@@ -971,11 +971,11 @@ void arguments( int argc, char *argv[] )
 /* modification end. */
 #if 0
 				case 'z':
-					fftThreshold = atoi( *++argv );
+					fftThreshold = myatoi( *++argv );
 					--argc; 
 					goto nextoption;
 				case 'w':
-					fftWinSize = atoi( *++argv );
+					fftWinSize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'Z':
@@ -1021,7 +1021,7 @@ int countamino( char *s, int end )
 	return( val );
 }
 
-static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, char **mseq1, char **mseq2, double *equiv, double *effarr, char **strfiles, char **chainids, int alloclen )
+static void pairalign( char **name, int nlen[M], char **seq, char **aseq, char **mseq1, char **mseq2, double *equiv, double *effarr, char **strfiles, char **chainids, int alloclen )
 {
 	int i, j, ilim;
 	int clus1, clus2;
@@ -1114,8 +1114,8 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 
 			strcpy( aseq[i], seq[i] );
 			strcpy( aseq[j], seq[j] );
-			clus1 = conjuctionfortbfast( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
-			clus2 = conjuctionfortbfast( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
+			clus1 = conjuctionfortbfast_old( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
+			clus2 = conjuctionfortbfast_old( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
 	//		fprintf( stderr, "mseq1 = %s\n", mseq1[0] );
 	//		fprintf( stderr, "mseq2 = %s\n", mseq2[0] );
 	
@@ -1224,7 +1224,7 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 
 	hat2p = fopen( hat2file, "w" );
 	if( !hat2p ) ErrorExit( "Cannot open hat2." );
-	WriteHat2( hat2p, njob, name, distancemtx );
+	WriteHat2_pointer( hat2p, njob, name, distancemtx );
 	fclose( hat2p );
 
 	fprintf( stderr, "##### writing hat3\n" );
@@ -1314,7 +1314,7 @@ static void WriteOptions( FILE *fp )
 int main( int argc, char *argv[] )
 {
 	static int  nlen[M];	
-	static char name[M][B], **seq;
+	static char **name, **seq;
 	static char **mseq1, **mseq2;
 	static char **aseq;
 	static char **bseq;
@@ -1363,6 +1363,7 @@ int main( int argc, char *argv[] )
 
 	fclose( infp );
 
+	name = AllocateCharMtx( njob, B+1 );
 	aseq = AllocateCharMtx( njob, nlenmax*2+1 );
 	bseq = AllocateCharMtx( njob, nlenmax*2+1 );
 	eff = AllocateDoubleVec( njob );
