@@ -63,6 +63,8 @@ typedef struct _thread_arg
 	Jobtable *jobpospt;
 	char **name;
 	char **seq;
+	char **dseq;
+	int *thereisxineachseq;
 	LocalHom **localhomtable;
 	double **distancemtx;
 	double *selfscore;
@@ -106,6 +108,25 @@ static void t2u( char *seq )
 	}
 }
 
+static int removex( char *d, char *m )
+{
+	int val = 0;
+	while( *m != 0 )
+	{
+		if( *m == 'X' || *m == 'x' ) 
+		{
+			m++;
+			val++;
+		}
+		else 
+		{
+			*d++ = *m++;
+		}
+	}
+	*d = 0;
+	return( val );
+}
+
 static void putlocalhom_last( char *s1, char *s2, LocalHom *localhompt, Lastresx *lastresx )
 {
 	char *pt1, *pt2;
@@ -1639,30 +1660,31 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'k':
-					kimuraR = atoi( *++argv );
+					kimuraR = myatoi( *++argv );
 //					fprintf( stderr, "kimuraR = %d\n", kimuraR );
 					--argc;
 					goto nextoption;
 				case 'b':
-					nblosum = atoi( *++argv );
+					nblosum = myatoi( *++argv );
 					scoremtx = 1;
 //					fprintf( stderr, "blosum %d\n", nblosum );
 					--argc;
 					goto nextoption;
 				case 'j':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
 					fprintf( stderr, "jtt %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'm':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
 					fprintf( stderr, "TM %d\n", pamN );
 					--argc;
 					goto nextoption;
+#if 0
 				case 'l':
 					ppslocal = (int)( atof( *++argv ) * 1000 + 0.5 );
 					pslocal = (int)( 600.0 / 1000.0 * ppslocal + 0.5);
@@ -1670,6 +1692,12 @@ void arguments( int argc, char *argv[] )
 //					fprintf( stderr, "pslocal = %d\n", pslocal );
 					--argc;
 					goto nextoption;
+#else
+				case 'l':
+					if( atof( *++argv ) < 0.00001 ) store_localhom = 0;
+					--argc;
+					goto nextoption;
+#endif
 				case 'd':
 					whereispairalign = *++argv;
 					fprintf( stderr, "whereispairalign = %s\n", whereispairalign );
@@ -1681,22 +1709,22 @@ void arguments( int argc, char *argv[] )
 					--argc; 
 					goto nextoption;
 				case 'C':
-					nthread = atoi( *++argv );
+					nthread = myatoi( *++argv );
 					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
 					goto nextoption;
 				case 'I':
-					nadd = atoi( *++argv );
+					nadd = myatoi( *++argv );
 					fprintf( stderr, "nadd = %d\n", nadd );
 					--argc;
 					goto nextoption;
 				case 'w':
-					lastm = atoi( *++argv );
+					lastm = myatoi( *++argv );
 					fprintf( stderr, "lastm = %d\n", lastm );
 					--argc;
 					goto nextoption;
 				case 'e':
-					laste = atoi( *++argv );
+					laste = myatoi( *++argv );
 					fprintf( stderr, "laste = %d\n", laste );
 					--argc;
 					goto nextoption;
@@ -1822,11 +1850,11 @@ void arguments( int argc, char *argv[] )
 					goto nextoption;
 #if 0
 				case 'z':
-					fftThreshold = atoi( *++argv );
+					fftThreshold = myatoi( *++argv );
 					--argc; 
 					goto nextoption;
 				case 'w':
-					fftWinSize = atoi( *++argv );
+					fftWinSize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'Z':
@@ -1872,17 +1900,13 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 {
 	thread_arg_t *targ = (thread_arg_t *)arg;
 	int i, ilim, j, jst;
-	int clus1, clus2;
-	int off1, off2;
+	int off1, off2, dum1, dum2, thereisx;
 	int intdum;
 	double bunbo;
 	float pscore = 0.0; // by D.Mathog
-	double *effarr;
 	double *effarr1;
 	double *effarr2;
-	char **pair;
-	char *indication1, *indication2;
-	char **mseq1, **mseq2;
+	char **mseq1, **mseq2, **distseq1, **distseq2, **dumseq1, **dumseq2;
 	char **aseq;
 
 // thread_arg
@@ -1891,6 +1915,8 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 	Jobtable *jobpospt = targ->jobpospt;
 	char **name = targ->name;
 	char **seq = targ->seq;
+	char **dseq = targ->dseq;
+	int *thereisxineachseq = targ->thereisxineachseq;
 	LocalHom **localhomtable = targ->localhomtable;
 	double **distancemtx = targ->distancemtx;
 	double *selfscore = targ->selfscore;
@@ -1900,18 +1926,18 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 
 //	fprintf( stderr, "thread %d start!\n", thread_no );
 
-	effarr = AllocateDoubleVec( njob );
-	for( i=0; i<njob; i++ ) effarr[i] = 1.0;
-	effarr1 = AllocateDoubleVec( njob );
-	effarr2 = AllocateDoubleVec( njob );
-	indication1 = AllocateCharVec( 150 );
-	indication2 = AllocateCharVec( 150 );
-	pair = AllocateCharMtx( njob, njob );
-	for( i=0; i<njob; i++ ) for( j=0; j<njob; j++ ) pair[i][j] = 0;
-	for( i=0; i<njob; i++ ) pair[i][i] = 1;
+	effarr1 = AllocateDoubleVec( 1 );
+	effarr2 = AllocateDoubleVec( 1 );
 	mseq1 = AllocateCharMtx( njob, 0 );
 	mseq2 = AllocateCharMtx( njob, 0 );
-	aseq = AllocateCharMtx( njob, alloclen+10 );
+	if( alg == 'N' )
+	{
+		dumseq1 = AllocateCharMtx( 1, alloclen+10 );
+		dumseq2 = AllocateCharMtx( 1, alloclen+10 );
+	}
+	distseq1 = AllocateCharMtx( 1, 0 );
+	distseq2 = AllocateCharMtx( 1, 0 );
+	aseq = AllocateCharMtx( 2, alloclen+10 );
 
 	if( alg == 'Y' || alg == 'r' ) ilim = njob - nadd;
 	else ilim = njob - 1;
@@ -1940,17 +1966,22 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 				if( commonJP ) FreeIntMtx( commonJP );
 				commonJP = NULL;
 				Falign( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
+				G__align11( NULL, NULL, 0, 0, 0 ); // 20130603
 				G__align11_noalign( NULL, 0, 0, NULL, NULL, 0 );
 				L__align11( NULL, NULL, 0, NULL, NULL );
+				L__align11_noalign( NULL, NULL );
 				genL__align11( NULL, NULL, 0, NULL, NULL );
-				free( effarr );
 				free( effarr1 );
 				free( effarr2 );
-				free( indication1 );
-				free( indication2 );
-				FreeCharMtx( pair  );
 				free( mseq1 );
 				free( mseq2 );
+				if( alg == 'N' )
+				{
+					FreeCharMtx( dumseq1 );
+					FreeCharMtx( dumseq2 );
+				}
+				free( distseq1 );
+				free( distseq2 );
 				FreeCharMtx( aseq  );
 				return( NULL );
 			}
@@ -1971,22 +2002,35 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 		{
 			if( store_dist )
 			{
-				if( alg == 'Y' || alg == 'r' ) distancemtx[i][j-(njob-nadd)] = 2.0;
-				else distancemtx[i][j] = 2.0;
+				if( alg == 'Y' || alg == 'r' ) distancemtx[i][j-(njob-nadd)] = 3.0;
+				else distancemtx[i][j] = 3.0;
 			}
 			if( stdout_dist) 
 			{
 				pthread_mutex_lock( targ->mutex_stdout );
-				fprintf( stdout, "%d %d d=%.3f\n", i+1, j+1, 2.0 );
+				fprintf( stdout, "%d %d d=%.3f\n", i+1, j+1, 3.0 );
 				pthread_mutex_unlock( targ->mutex_stdout );
 			}
 			continue;
 		}
 
-		strcpy( aseq[i], seq[i] );
-		strcpy( aseq[j], seq[j] );
-		clus1 = conjuctionfortbfast( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
-		clus2 = conjuctionfortbfast( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
+		strcpy( aseq[0], seq[i] );
+		strcpy( aseq[1], seq[j] );
+//		clus1 = conjuctionfortbfast( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
+//		clus2 = conjuctionfortbfast( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
+//		fprintf( stderr, "Skipping conjuction..\n" );
+
+		effarr1[0] = 1.0;
+		effarr2[0] = 1.0;
+		mseq1[0] = aseq[0];
+		mseq2[0] = aseq[1];
+
+		thereisx = thereisxineachseq[i] + thereisxineachseq[j];
+//		strcpy( distseq1[0], dseq[i] ); // nen no tame
+//		strcpy( distseq2[0], dseq[j] ); // nen no tame
+		distseq1[0] = dseq[i];
+		distseq2[0] = dseq[j];
+
 //		fprintf( stderr, "mseq1 = %s\n", mseq1[0] );
 //		fprintf( stderr, "mseq2 = %s\n", mseq2[0] );
 	
@@ -2000,7 +2044,7 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 
 		if( use_fft )
 		{
-			pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum, NULL, 0, NULL );
+			pscore = Falign( mseq1, mseq2, effarr1, effarr2, 1, 1, alloclen, &intdum, NULL, 0, NULL );
 //			fprintf( stderr, "pscore (fft) = %f\n", pscore );
 			off1 = off2 = 0;
 		}
@@ -2024,25 +2068,55 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 					if( nadd && njob-nadd <= j && njob-nadd <= i ) // new sequence doushi ha mushi
 						pscore = 0.0;
 					else
-						pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+					{
+						if( store_localhom )
+						{
+							pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+							if( thereisx ) pscore = L__align11_noalign( distseq1, distseq2 ); // uwagaki
+						}
+						else
+							pscore = L__align11_noalign( distseq1, distseq2 );
+					}
+//					pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, distseq1, distseq2, alloclen ); // CHUUI!!!!!!
 					break;
 				case( 'Y' ):
 					if( nadd == 0 || ( i < njob-nadd && njob-nadd <= j ) ) // new sequence vs exiting sequence nomi keisan
-						pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+					{
+						if( store_localhom )
+						{
+							pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+							if( thereisx ) pscore = L__align11_noalign( distseq1, distseq2 ); // uwagaki
+						}
+						else
+							pscore = L__align11_noalign( distseq1, distseq2 );
+					}
 					else
 						pscore = 0.0;
 					break;
 				case( 'A' ):
-					pscore = G__align11( mseq1, mseq2, alloclen, outgap, outgap );
+					if( store_localhom )
+					{
+						pscore = G__align11( mseq1, mseq2, alloclen, outgap, outgap );
+						if( thereisx ) pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, distseq1, distseq2, alloclen ); // uwagaki
+					}
+					else
+						pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, distseq1, distseq2, alloclen ); // uwagaki
 					off1 = off2 = 0;
 					break;
 				case( 'N' ):
 //					pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
 					pscore = genL__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+					if( thereisx )
+					{
+						strcpy( dumseq1[0], distseq1[0] );
+						strcpy( dumseq2[0], distseq2[0] );
+						pscore = genL__align11( dumseq1, dumseq2, alloclen, &dum1, &dum2 ); // uwagaki
+					}
 					break;
 				case( 't' ):
-					pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
 					off1 = off2 = 0;
+//					pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
+					pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, distseq1, distseq2, alloclen ); // tsuneni distseq shiyou
 					break;
 				case( 's' ):
 					pscore = callmxscarna_giving_bpp( mseq1, mseq2, bpp[i], bpp[j], alloclen, i, j );
@@ -2083,16 +2157,19 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 #if SCOREOUT
 			fprintf( stderr, "score = %10.2f (%d,%d)\n", pscore, i, j );
 #endif
-			if( pscore > 0.0 && ( nadd == 0 || ( alg != 'Y' && alg != 'r' ) || ( i < njob-nadd && njob-nadd <= j ) ) )
+//			if( pscore > 0.0 && ( nadd == 0 || ( alg != 'Y' && alg != 'r' ) || ( i < njob-nadd && njob-nadd <= j ) ) ) x-ins-i de seido teika
+			if( ( nadd == 0 || ( alg != 'Y' && alg != 'r' ) || ( i < njob-nadd && njob-nadd <= j ) ) )
 			{
 				if( !store_localhom )
 					;
 				else if( alg == 'R' )
 					putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j, lastresx[i]+j );
 				else if( alg == 'r' )
-					putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j, lastresx[i]+j-(njob-nadd) );// ?????
+					putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), lastresx[i]+j-(njob-nadd) );// ?????
 				else if( alg == 'H' )
 					putlocalhom_ext( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+				else if( alg == 'Y' )
+					putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), off1, off2, (int)pscore, strlen( mseq1[0] ) );
 				else if( alg != 'S' && alg != 'V' )
 					putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
 			}
@@ -2141,13 +2218,11 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 }
 #endif
 
-static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **mseq1, char **mseq2, double *effarr, int alloclen, Lastresx **lastresx )
+static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **dseq, int *thereisxineachseq, char **mseq1, char **mseq2, int alloclen, Lastresx **lastresx )
 {
-	int i, j, ilim, jst;
-	int clus1, clus2;
-	int off1, off2;
+	int i, j, ilim, jst, jj;
+	int off1, off2, dum1, dum2, thereisx;
 	float pscore = 0.0; // by D.Mathog
-	static char *indication1, *indication2;
 	FILE *hat2p, *hat3p;
 	double **distancemtx;
 	double *selfscore;
@@ -2156,18 +2231,29 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 	char *pt;
 	char *hat2file = "hat2";
 	LocalHom **localhomtable = NULL, *tmpptr;
-	static char **pair;
 	int intdum;
 	double bunbo;
 	char ***bpp = NULL; // mxscarna no toki dake
+	char **distseq1, **distseq2;
+	char **dumseq1, **dumseq2;
 
 	if( store_localhom )
 	{
-		localhomtable = (LocalHom **)calloc( njob, sizeof( LocalHom *) );
-		for( i=0; i<njob; i++)
+		if( alg == 'Y' || alg == 'r' )
 		{
-			localhomtable[i] = (LocalHom *)calloc( njob, sizeof( LocalHom ) );
-			for( j=0; j<njob; j++)
+			ilim = njob - nadd;
+			jst = nadd;
+		}
+		else
+		{
+			ilim = njob;
+			jst = njob;
+		}
+		localhomtable = (LocalHom **)calloc( ilim, sizeof( LocalHom *) );
+		for( i=0; i<ilim; i++)
+		{
+			localhomtable[i] = (LocalHom *)calloc( jst, sizeof( LocalHom ) );
+			for( j=0; j<jst; j++)
 			{
 				localhomtable[i][j].start1 = -1;
 				localhomtable[i][j].end1 = -1;
@@ -2189,15 +2275,17 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 	}
 	else distancemtx = NULL;
 
+	if( alg == 'N' )
+	{
+		dumseq1 = AllocateCharMtx( 1, alloclen+10 );
+		dumseq2 = AllocateCharMtx( 1, alloclen+10 );
+	}
+	distseq1 = AllocateCharMtx( 1, 0 ); // muda
+	distseq2 = AllocateCharMtx( 1, 0 ); // muda
+
 	selfscore = AllocateDoubleVec( njob );
 	effarr1 = AllocateDoubleVec( njob );
 	effarr2 = AllocateDoubleVec( njob );
-	indication1 = AllocateCharVec( 150 );
-	indication2 = AllocateCharVec( 150 );
-#if 0
-#else
-	pair = AllocateCharMtx( njob, njob );
-#endif
 
 #if 0
 	fprintf( stderr, "##### fftwinsize = %d, fftthreshold = %d\n", fftWinSize, fftThreshold );
@@ -2211,9 +2299,6 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 
 //	writePre( njob, name, nlen, aseq, 0 );
 
-	for( i=0; i<njob; i++ ) for( j=0; j<njob; j++ ) pair[i][j] = 0;
-	for( i=0; i<njob; i++ ) pair[i][i] = 1;
-
 	if( alg == 'R' )
 	{
 		fprintf( stderr, "Calling last (http://last.cbrc.jp/)\n" );
@@ -2313,6 +2398,8 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 			targ[i].jobpospt = &jobpos;
 			targ[i].name = name;
 			targ[i].seq = seq;
+			targ[i].dseq = dseq;
+			targ[i].thereisxineachseq = thereisxineachseq;
 			targ[i].localhomtable = localhomtable;
 			targ[i].distancemtx = distancemtx;
 			targ[i].selfscore = selfscore;
@@ -2357,17 +2444,30 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 				{
 					if( store_dist ) 
 					{
-						if( alg == 'Y' || alg == 'r' ) distancemtx[i][j-(njob-nadd)] = 2.0;
-						else distancemtx[i][j] = 2.0;
+						if( alg == 'Y' || alg == 'r' ) distancemtx[i][j-(njob-nadd)] = 3.0;
+						else distancemtx[i][j] = 3.0;
 					}
-					if( stdout_dist) fprintf( stdout, "%d %d d=%.3f\n", i+1, j+1, 2.0 );
+					if( stdout_dist) fprintf( stdout, "%d %d d=%.3f\n", i+1, j+1, 3.0 );
 					continue;
 				}
 	
-				strcpy( aseq[i], seq[i] );
-				strcpy( aseq[j], seq[j] );
-				clus1 = conjuctionfortbfast( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
-				clus2 = conjuctionfortbfast( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
+				strcpy( aseq[0], seq[i] );
+				strcpy( aseq[1], seq[j] );
+//				clus1 = conjuctionfortbfast( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
+//				clus2 = conjuctionfortbfast( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
+//				fprintf( stderr, "Skipping conjuction..\n" );
+
+				effarr1[0] = 1.0;
+				effarr2[0] = 1.0;
+				mseq1[0] = aseq[0];
+				mseq2[0] = aseq[1];
+
+				thereisx = thereisxineachseq[i] + thereisxineachseq[j];
+//				strcpy( distseq1[0], dseq[i] ); // nen no tame
+//				strcpy( distseq2[0], dseq[j] ); // nen no tame
+				distseq1[0] = dseq[i];
+				distseq2[0] = dseq[j];
+
 	//			fprintf( stderr, "mseq1 = %s\n", mseq1[0] );
 	//			fprintf( stderr, "mseq2 = %s\n", mseq2[0] );
 		
@@ -2381,7 +2481,7 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 	
 				if( use_fft )
 				{
-					pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum, NULL, 0, NULL );
+					pscore = Falign( mseq1, mseq2, effarr1, effarr2, 1, 1, alloclen, &intdum, NULL, 0, NULL );
 //					fprintf( stderr, "pscore (fft) = %f\n", pscore );
 					off1 = off2 = 0;
 				}
@@ -2390,16 +2490,29 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 					switch( alg )
 					{
 						case( 't' ):
-							pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
+//							pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
+							pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, distseq1, distseq2, alloclen ); // tsuneni distseq shiyou
 							off1 = off2 = 0;
 							break;
 						case( 'A' ):
-							pscore = G__align11( mseq1, mseq2, alloclen, outgap, outgap );
+							if( store_localhom )
+							{
+								pscore = G__align11( mseq1, mseq2, alloclen, outgap, outgap );
+								if( thereisx ) pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, distseq1, distseq2, alloclen ); // uwagaki
+							}
+							else
+								pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, distseq1, distseq2, alloclen ); // uwagaki
 							off1 = off2 = 0;
 							break;
 						case( 'N' ):
 //							pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
 							pscore = genL__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+							if( thereisx )
+							{
+								strcpy( dumseq1[0], distseq1[0] );
+								strcpy( dumseq2[0], distseq2[0] );
+								pscore = genL__align11( dumseq1, dumseq2, alloclen, &dum1, &dum2 ); // uwagaki
+							}
 							break;
 						case( 'R' ):
 							if( nadd && njob-nadd <= j && njob-nadd <= i ) // new sequence doushi ha mushi
@@ -2417,16 +2530,33 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 							if( nadd && njob-nadd <= j && njob-nadd <= i ) // new sequence doushi ha mushi
 								pscore = 0.0;
 							else
-							pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 ); // all pair
+							{
+								if( store_localhom )
+								{
+									pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 ); // all pair
+									if( thereisx ) pscore = L__align11_noalign( distseq1, distseq2 ); // all pair
+								}
+								else
+									pscore = L__align11_noalign( distseq1, distseq2 ); // all pair
+							}
+//							pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, distseq1, distseq2, alloclen ); // CHUUI!!!!!!
 							break;
 						case( 'Y' ):
 							if( nadd == 0 || ( i < njob-nadd && njob-nadd <= j ) ) // new sequence vs exiting sequence nomi keisan
-								pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+							{
+								if( store_localhom )
+								{
+									pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+									if( thereisx ) pscore = L__align11_noalign( distseq1, distseq2 ); // uwagaki
+								}
+								else
+									pscore = L__align11_noalign( distseq1, distseq2 );
+							}
 							else
 								pscore = 0.0;
 							break;
 						case( 'a' ):
-							pscore = Aalign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen );
+							pscore = Aalign( mseq1, mseq2, effarr1, effarr2, 1, 1, alloclen );
 							off1 = off2 = 0;
 							break;
 #if 0
@@ -2464,21 +2594,23 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 #if SCOREOUT
 					fprintf( stderr, "score = %10.2f (%d,%d)\n", pscore, i, j );
 #endif
-					if( pscore > 0.0 && ( nadd == 0 || ( alg != 'Y' && alg != 'r' ) || ( i < njob-nadd && njob-nadd <= j ) ) )
+//					if( pscore > 0.0 && ( nadd == 0 || ( alg != 'Y' && alg != 'r' ) || ( i < njob-nadd && njob-nadd <= j ) ) ) // x-ins-i de seido teika
+					if( ( nadd == 0 || ( alg != 'Y' && alg != 'r' ) || ( i < njob-nadd && njob-nadd <= j ) ) )
 					{
 						if( !store_localhom )
 							;
 						else if( alg == 'R' )
 							putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j, lastresx[i]+j );
 						else if( alg == 'r' )
-							putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j, lastresx[i]+j-(njob-nadd) );// ?????
+							putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), lastresx[i]+j-(njob-nadd) );// ?????
 						else if( alg == 'H' )
 							putlocalhom_ext( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+						else if( alg == 'Y' )
+							putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), off1, off2, (int)pscore, strlen( mseq1[0] ) );
 						else if( alg != 'S' && alg != 'V' )
 							putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
 					}
 
-	
 					if( (bunbo=MIN( selfscore[i], selfscore[j] )) == 0.0 )
 						pscore = 2.0;
 					else if( bunbo < pscore ) // mondai ari
@@ -2530,13 +2662,27 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 	if( store_localhom )
 	{
 		fprintf( stderr, "\n\n##### writing hat3\n" );
-		ilim = njob-1;	
+		if( alg == 'Y' || alg == 'r' )
+			ilim = njob-nadd;	
+		else
+			ilim = njob-1;	
 		for( i=0; i<ilim; i++ ) 
 		{
-			for( j=i+1; j<njob; j++ )
+			if( alg == 'Y' || alg == 'r' )
 			{
-				for( tmpptr=localhomtable[i]+j; tmpptr; tmpptr=tmpptr->next )
+				jst = njob-nadd;
+				jj = 0;
+			}
+			else
+			{
+				jst = i+1;
+				jj = i+1;
+			}
+			for( j=jst; j<njob; j++, jj++ )
+			{
+				for( tmpptr=localhomtable[i]+jj; tmpptr; tmpptr=tmpptr->next )
 				{
+//					fprintf( stderr, "j=%d, jj=%d\n", j, jj );
 					if( tmpptr->opt == -1.0 ) continue;
 // tmptmptmptmptmp
 //					if( alg == 'B' || alg == 'T' )
@@ -2550,7 +2696,10 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 #if DEBUG
 		fprintf( stderr, "calling FreeLocalHomTable\n" );
 #endif
-		FreeLocalHomTable( localhomtable, njob );
+		if( alg == 'Y' || alg == 'r' )
+			FreeLocalHomTable_part( localhomtable, (njob-nadd), nadd );
+		else
+			FreeLocalHomTable( localhomtable, njob );
 #if DEBUG
 		fprintf( stderr, "done. FreeLocalHomTable\n" );
 #endif
@@ -2576,8 +2725,13 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 	free( selfscore );
 	free( effarr1 );
 	free( effarr2 );
-	free( indication1 );
-	free( indication2 );
+	if( alg == 'N' )
+	{
+		FreeCharMtx( dumseq1 );
+		FreeCharMtx( dumseq2 );
+	}
+	free( distseq1 );
+	free( distseq2 );
 	if( store_dist ) FreeDoubleMtx( distancemtx );
 }
 
@@ -2585,12 +2739,12 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **m
 
 int main( int argc, char *argv[] )
 {
-	int  *nlen;	
+	int  *nlen, *thereisxineachseq;
 	char **name, **seq;
 	char **mseq1, **mseq2;
 	char **aseq;
 	char **bseq;
-	double *eff;
+	char **dseq;
 	int i, j, k;
 	FILE *infp;
 	char c;
@@ -2639,13 +2793,14 @@ int main( int argc, char *argv[] )
 
 	alloclen = nlenmax*2;
 	seq = AllocateCharMtx( njob, alloclen+10 );
-	aseq = AllocateCharMtx( njob, alloclen+10 );
+	aseq = AllocateCharMtx( 2, alloclen+10 );
 	bseq = AllocateCharMtx( njob, alloclen+10 );
+	dseq = AllocateCharMtx( njob, alloclen+10 );
 	mseq1 = AllocateCharMtx( njob, 0 );
 	mseq2 = AllocateCharMtx( njob, 0 );
 	name = AllocateCharMtx( njob, B );
-	eff = AllocateDoubleVec( njob );
 	nlen = AllocateIntVec( njob );
+	thereisxineachseq = AllocateIntVec( njob );
 
 	if( alg == 'R' )
 	{
@@ -2713,12 +2868,14 @@ int main( int argc, char *argv[] )
 
 //	writePre( njob, name, nlen, seq, 0 );
 
-	for( i=0; i<njob; i++ ) eff[i] = 1.0;
 
+	for( i=0; i<njob; i++ ) 
+	{
+		gappick0( bseq[i], seq[i] );
+		thereisxineachseq[i] = removex( dseq[i], bseq[i] );
+	}
 
-	for( i=0; i<njob; i++ ) gappick0( bseq[i], seq[i] );
-
-	pairalign( name, nlen, bseq, aseq, mseq1, mseq2, eff, alloclen, lastresx );
+	pairalign( name, nlen, bseq, aseq, dseq, thereisxineachseq, mseq1, mseq2, alloclen, lastresx );
 
 	fprintf( trap_g, "done.\n" );
 #if DEBUG
@@ -2766,10 +2923,12 @@ int main( int argc, char *argv[] )
 	FreeCharMtx( seq );
 	FreeCharMtx( aseq );
 	FreeCharMtx( bseq );
+	FreeCharMtx( dseq );
 	FreeCharMtx( name );
 	free( mseq1 );
 	free( mseq2 );
-	free( eff );
+	free( nlen );
+	free( thereisxineachseq );
 
 	return( 0 );
 }
