@@ -372,18 +372,20 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 {
 #if FASTMATCHCALC
 	int j, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
 	float *matchpt, *cpmxpdpt, **cpmxpdptpt;
 	int *cpmxpdnpt, **cpmxpdnptpt;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 	if( initialize )
 	{
 		int count = 0;
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[l][j] )
 				{
@@ -397,10 +399,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 	}
 
 	{
-		for( l=0; l<26; l++ )
+		for( l=0; l<nalphabets; l++ )
 		{
 			scarr[l] = 0.0;
-			for( j=0; j<26; j++ )
+			for( j=0; j<nalphabets; j++ )
 //				scarr[l] += n_dis[j][l] * cpmx1[j][i1];
 				scarr[l] += n_dis_consweight_multi[j][l] * cpmx1[j][i1];
 		}
@@ -417,11 +419,14 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 			matchpt++;
 		} 
 	}
+	free( scarr );
 #else
 	int j, k, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 	// simple
 	if( initialize )
 	{
@@ -429,7 +434,7 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[l][j] )
 				{
@@ -441,10 +446,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 			cpmxpdn[count][j] = -1;
 		}
 	}
-	for( l=0; l<26; l++ )
+	for( l=0; l<nalphabets; l++ )
 	{
 		scarr[l] = 0.0;
-		for( k=0; k<26; k++ )
+		for( k=0; k<nalphabets; k++ )
 //			scarr[l] += n_dis[k][l] * cpmx1[k][i1];
 			scarr[l] += n_dis_consweight_multi[k][l] * cpmx1[k][i1];
 	}
@@ -454,6 +459,7 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		for( k=0; cpmxpdn[k][j]>-1; k++ )
 			match[j] += scarr[cpmxpdn[k][j]] * cpmxpd[k][j];
 	} 
+	free( scarr );
 #endif
 }
 
@@ -466,8 +472,10 @@ static void Atracking_localhom( float *impwmpt, float *lasthorizontalw, float *l
 						int *gapmap1, int *gapmap2 )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k;
-	char gap[] = "-";
+//	char gap[] = "-";
+	char *gap;
 	float wm;
+	gap = newgapstr;
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
 
@@ -577,8 +585,10 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
                         int **ijp, int icyc, int jcyc )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k, lastk;
-	char gap[] = "-";
+//	char gap[] = "-";
+	char *gap;
 	float wm = 0.0;
+	gap = newgapstr;
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
 
@@ -859,15 +869,15 @@ float partQ__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 		gapz_n1 = AllocateFloatVec( ll1+2 );
 		gapz_n2 = AllocateFloatVec( ll2+2 );
 
-		cpmx1 = AllocateFloatMtx( 26, ll1+2 );
-		cpmx2 = AllocateFloatMtx( 26, ll2+2 );
+		cpmx1 = AllocateFloatMtx( nalphabets, ll1+2 );
+		cpmx2 = AllocateFloatMtx( nalphabets, ll2+2 );
 
 #if FASTMATCHCALC
-		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 26 ); 
-		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 26 ); 
+		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
+		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets ); 
 #else
-		floatwork = AllocateFloatMtx( 26, MAX( ll1, ll2 )+2 ); 
-		intwork = AllocateIntMtx( 26, MAX( ll1, ll2 )+2 ); 
+		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
+		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 #endif
 
 #if DEBUG
