@@ -431,18 +431,20 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 {
 #if FASTMATCHCALC
 	int j, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
 	float *matchpt, *cpmxpdpt, **cpmxpdptpt;
 	int *cpmxpdnpt, **cpmxpdnptpt;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 	if( initialize )
 	{
 		int count = 0;
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[l][j] )
 				{
@@ -456,10 +458,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 	}
 
 	{
-		for( l=0; l<26; l++ )
+		for( l=0; l<nalphabets; l++ )
 		{
 			scarr[l] = 0.0;
-			for( j=0; j<26; j++ )
+			for( j=0; j<nalphabets; j++ )
 				scarr[l] += n_dis_consweight_multi[j][l] * cpmx1[j][i1];
 //				scarr[l] += n_dis[j][l] * cpmx1[j][i1];
 		}
@@ -476,11 +478,14 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 			matchpt++;
 		} 
 	}
+	free( scarr );
 #else
 	int j, k, l;
-	float scarr[26];
+//	float scarr[26];
 	float **cpmxpd = floatwork;
 	int **cpmxpdn = intwork;
+	float *scarr;
+	scarr = calloc( nalphabets, sizeof( float ) );
 	// simple
 	if( initialize )
 	{
@@ -488,7 +493,7 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 		for( j=0; j<lgth2; j++ )
 		{
 			count = 0;
-			for( l=0; l<26; l++ )
+			for( l=0; l<nalphabets; l++ )
 			{
 				if( cpmx2[l][j] )
 				{
@@ -500,10 +505,10 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 			cpmxpdn[count][j] = -1;
 		}
 	}
-	for( l=0; l<26; l++ )
+	for( l=0; l<nalphabets; l++ )
 	{
 		scarr[l] = 0.0;
-		for( k=0; k<26; k++ )
+		for( k=0; k<nalphabets; k++ )
 			scarr[l] += n_dis_consweight_multi[k][l] * cpmx1[k][i1];
 //			scarr[l] += n_dis[k][l] * cpmx1[k][i1];
 	}
@@ -774,6 +779,8 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 	static TLS float *fgcp2;
 	static TLS float **cpmx1;
 	static TLS float **cpmx2;
+	static TLS float *gapfreq1;
+	static TLS float *gapfreq2;
 	static TLS int **intwork;
 	static TLS float **floatwork;
 	static TLS int orlgth1 = 0, orlgth2 = 0;
@@ -785,6 +792,12 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 	float *ogcp2pt;
 	float fgcp1va;
 	float ogcp1va;
+	float *gf2pt;
+	float *gf2ptpre;
+	float gf1va;
+	float gf1vapre;
+	float headgapfreq1;
+	float headgapfreq2;
 
 
 	if( seq1 == NULL )
@@ -819,6 +832,9 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 			FreeFloatMtx( cpmx1 );
 			FreeFloatMtx( cpmx2 );
 
+			FreeFloatVec( gapfreq1 );
+			FreeFloatVec( gapfreq2 );
+
 			FreeFloatMtx( floatwork );
 			FreeIntMtx( intwork );
 
@@ -833,8 +849,8 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
 #if 1
-	if( lgth1 == 0 ) fprintf( stderr, "WARNING: lgth1=0 in partA__align\n" );
-	if( lgth2 == 0 ) fprintf( stderr, "WARNING: lgth2=0 in partA__align\n" );
+//	if( lgth1 == 0 ) fprintf( stderr, "WARNING: lgth1=0 in partA__align\n" );
+//	if( lgth2 == 0 ) fprintf( stderr, "WARNING: lgth2=0 in partA__align\n" );
 
 	if( lgth1 == 0 && lgth2 == 0 )
 		return( 0.0 );
@@ -845,7 +861,7 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 		{
 			j = lgth2;
 			seq1[i][j] = 0;
-			while( j ) seq1[i][--j] = '-';
+			while( j ) seq1[i][--j] = *newgapstr;
 //			fprintf( stderr, "seq1[i] = %s\n", seq1[i] );
 		}
 		return( 0.0 );
@@ -857,7 +873,7 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 		{
 			j = lgth1;
 			seq2[i][j] = 0;
-			while( j ) seq2[i][--j] = '-';
+			while( j ) seq2[i][--j] = *newgapstr;
 //			fprintf( stderr, "seq2[i] = %s\n", seq2[i] );
 		}
 		return( 0.0 );
@@ -903,6 +919,9 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 			FreeFloatMtx( cpmx1 );
 			FreeFloatMtx( cpmx2 );
 
+			FreeFloatVec( gapfreq1 );
+			FreeFloatVec( gapfreq2 );
+
 			FreeFloatMtx( floatwork );
 			FreeIntMtx( intwork );
 		}
@@ -931,15 +950,18 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 		fgcp1 = AllocateFloatVec( ll1+2 );
 		fgcp2 = AllocateFloatVec( ll2+2 );
 
-		cpmx1 = AllocateFloatMtx( 26, ll1+2 );
-		cpmx2 = AllocateFloatMtx( 26, ll2+2 );
+		cpmx1 = AllocateFloatMtx( nalphabets, ll1+2 );
+		cpmx2 = AllocateFloatMtx( nalphabets, ll2+2 );
+
+		gapfreq1 = AllocateFloatVec( ll1+2 );
+		gapfreq2 = AllocateFloatVec( ll2+2 );
 
 #if FASTMATCHCALC
-		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, 26 ); 
-		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, 26 ); 
+		floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
+		intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets ); 
 #else
-		floatwork = AllocateFloatMtx( 26, MAX( ll1, ll2 )+2 ); 
-		intwork = AllocateIntMtx( 26, MAX( ll1, ll2 )+2 ); 
+		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
+		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 #endif
 
 #if DEBUG
@@ -991,6 +1013,10 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 		new_OpeningGapCount( ogcp2, jcyc, seq2, eff2, lgth2, sgap2 );
 		new_FinalGapCount( fgcp1, icyc, seq1, eff1, lgth1, egap1 );
 		new_FinalGapCount( fgcp2, jcyc, seq2, eff2, lgth2, egap2 );
+		outgapcount( &headgapfreq1, icyc, sgap1, eff1 );
+		outgapcount( &headgapfreq2, jcyc, sgap2, eff2 );
+		outgapcount( gapfreq1+lgth1, icyc, egap1, eff1 );
+		outgapcount( gapfreq2+lgth2, jcyc, egap2, eff2 );
 	}
 	else
 	{
@@ -998,17 +1024,38 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 		st_OpeningGapCount( ogcp2, jcyc, seq2, eff2, lgth2 );
 		st_FinalGapCount( fgcp1, icyc, seq1, eff1, lgth1 );
 		st_FinalGapCount( fgcp2, jcyc, seq2, eff2, lgth2 );
+		headgapfreq1 = 0.0;
+		headgapfreq2 = 0.0;
+		gapfreq1[lgth1] = 0.0;
+		gapfreq2[lgth2] = 0.0;
+	}
+
+	if( legacygapcost == 0 )
+	{
+		gapcountf( gapfreq1, seq1, icyc, eff1, lgth1 );
+		gapcountf( gapfreq2, seq2, jcyc, eff2, lgth2 );
+		for( i=0; i<lgth1+1; i++ ) gapfreq1[i] = 1.0 - gapfreq1[i];
+		for( i=0; i<lgth2+1; i++ ) gapfreq2[i] = 1.0 - gapfreq2[i];
+		headgapfreq1 = 1.0 - headgapfreq1;
+		headgapfreq2 = 1.0 - headgapfreq2;
+	}
+	else
+	{
+		for( i=0; i<lgth1+1; i++ ) gapfreq1[i] = 1.0;
+		for( i=0; i<lgth2+1; i++ ) gapfreq2[i] = 1.0;
+		headgapfreq1 = 1.0;
+		headgapfreq2 = 1.0;
 	}
 
 	for( i=0; i<lgth1; i++ ) 
 	{
-		ogcp1[i] = 0.5 * ( 1.0 - ogcp1[i] ) * fpenalty;
-		fgcp1[i] = 0.5 * ( 1.0 - fgcp1[i] ) * fpenalty;
+		ogcp1[i] = 0.5 * ( 1.0 - ogcp1[i] ) * fpenalty * ( gapfreq1[i] );
+		fgcp1[i] = 0.5 * ( 1.0 - fgcp1[i] ) * fpenalty * ( gapfreq1[i] );
 	}
 	for( i=0; i<lgth2; i++ ) 
 	{
-		ogcp2[i] = 0.5 * ( 1.0 - ogcp2[i] ) * fpenalty;
-		fgcp2[i] = 0.5 * ( 1.0 - fgcp2[i] ) * fpenalty;
+		ogcp2[i] = 0.5 * ( 1.0 - ogcp2[i] ) * fpenalty * ( gapfreq2[i] );
+		fgcp2[i] = 0.5 * ( 1.0 - fgcp2[i] ) * fpenalty * ( gapfreq2[i] );
 	}
 #if 0
 	for( i=0; i<lgth1; i++ ) 
@@ -1037,11 +1084,13 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 	{
 		for( i=1; i<lgth1+1; i++ )
 		{
-			initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] ) ;
+//			initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] ) ;
+			initverticalw[i] += ( ogcp1[0] * headgapfreq2 + fgcp1[i-1] * gapfreq2[0] ) ;
 		}
 		for( j=1; j<lgth2+1; j++ )
 		{
-			currentw[j] += ( ogcp2[0] + fgcp2[j-1] ) ;
+//			currentw[j] += ( ogcp2[0] + fgcp2[j-1] ) ;
+			currentw[j] += ( ogcp2[0] * headgapfreq1 + fgcp2[j-1] * gapfreq1[0] ) ;
 		}
 	}
 #if OUTGAP0TRY
@@ -1056,7 +1105,8 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 
 	for( j=1; j<lgth2+1; ++j ) 
 	{
-		m[j] = currentw[j-1] + ogcp1[1]; mp[j] = 0;
+//		m[j] = currentw[j-1] + ogcp1[1]; mp[j] = 0;
+		m[j] = currentw[j-1] + ogcp1[1] * gapfreq2[j-1]; mp[j] = 0;;
 	}
 
 	lastverticalw[0] = currentw[lgth2-1];
@@ -1133,7 +1183,8 @@ fprintf( stderr, "\n" );
 		currentw[0] = initverticalw[i];
 
 
-		mi = previousw[0] + ogcp2[1]; mpi = 0;
+//		mi = previousw[0] + ogcp2[1]; mpi = 0;
+		mi = previousw[0] + ogcp2[1] * gapfreq1[i-1]; mpi=0;
 
 		ijppt = ijp[i] + 1;
 		mjpt = m + 1;
@@ -1144,6 +1195,11 @@ fprintf( stderr, "\n" );
 		ogcp2pt = ogcp2+1;
 		fgcp1va = fgcp1[i-1];
 		ogcp1va = ogcp1[i];
+		gf1va = gapfreq1[i];
+		gf1vapre = gapfreq1[i-1];
+		gf2pt = gapfreq2+1;
+		gf2ptpre = gapfreq2;
+
 		for( j=1; j<lastj; j++ )
 		{
 #ifdef xxxenablemultithread
@@ -1161,17 +1217,14 @@ fprintf( stderr, "\n" );
 #if 0
 			fprintf( stderr, "%5.0f->", wm );
 #endif
-			g = mi + *fgcp2pt;
-#if 0
-			fprintf( stderr, "%5.0f?", g );
-#endif
-			if( g > wm )
+//			g = mi + *fgcp2pt * gapfreq1[i];
+			if( (g = mi + *fgcp2pt * gf1va) > wm )
 			{
 				wm = g;
 				*ijppt = -( j - mpi );
 			}
-			g = *prept + *ogcp2pt;
-			if( g >= mi )
+//			g = *prept + *ogcp2pt * gapfreq1[i-1];
+			if( (g = *prept + *ogcp2pt * gf1vapre) >= mi )
 			{
 				mi = g;
 				mpi = j-1;
@@ -1180,17 +1233,14 @@ fprintf( stderr, "\n" );
 			mi += fpenalty_ex;
 #endif
 
-			g = *mjpt + fgcp1va;
-#if 0 
-			fprintf( stderr, "%5.0f?", g );
-#endif
-			if( g > wm )
+//			g = *mjpt + fgcp1va * gapfreq2[j];
+			if( (g = *mjpt + fgcp1va * *gf2pt) > wm )
 			{
 				wm = g;
 				*ijppt = +( i - *mpjpt );
 			}
-			g = *prept + ogcp1va;
-			if( g >= *mjpt )
+//			g = *prept + ogcp1va * gapfreq2[j-1];
+			if( (g = *prept + ogcp1va * *gf2ptpre) >= *mjpt )
 			{
 				*mjpt = g;
 				*mpjpt = i-1;
@@ -1210,6 +1260,8 @@ fprintf( stderr, "\n" );
 			curpt++;
 			fgcp2pt++;
 			ogcp2pt++;
+			gf2ptpre++;
+			gf2pt++;
 		}
 		lastverticalw[i] = currentw[lgth2-1];
 
