@@ -89,73 +89,6 @@ int dcompare( const Scores *a, const Scores *b )
 	}
 }
 
-#if 0
-static void	gappickandx0( char *out, char *in )
-{
-	char c;
-	if( scoremtx == -1 )
-	{
-		while( *in )
-		{
-			if( (c=*in++) == '-' )
-				;	
-			else if( c == 'u' )
-				*out++ = 't';
-			else if( amino_n[c] < 4 && amino_n[c] > -1 )
-				*out++ = c;
-			else
-				*out++ = 'n';
-		}
-	}
-	else
-	{
-		while( *in )
-		{
-			if( (c=*in++) == '-' )
-				;	
-			else if( amino_n[c] < 20 && amino_n[c] > -1 )
-				*out++ = c;
-			else
-				*out++ = 'X';
-		}
-	}
-	*out = 0;
-}	
-
-static int getkouho( int *pickkouho, double prob, int nin, Scores *scores, char **seq ) // 0 < prob < 1
-{
-	int nkouho = 0;
-	int i, j;
-	int *iptr = pickkouho;
-	for( i=1; i<nin; i++ )
-	{
-		if( ( nkouho==0 || rnd() < prob ) && ( scores[i].shimon != scores->shimon || strcmp( seq[scores->numinseq], seq[scores[i].numinseq] ) ) )
-		{
-#if 0
-			for( j=0; j<nkouho; j++ )
-			{
-				if( scores[i].shimon == scores[pickkouho[j]].shimon || !strcmp( seq[scores[pickkouho[j]].numinseq], seq[scores[i].numinseq] ) ) 
-					break;
-			}
-			if( j == nkouho )
-#endif
-			{
-				*iptr++ = i;
-				nkouho++;
-//				fprintf( stderr, "ok! nkouho=%d\n", nkouho );
-			}
-		}
-		else
-		{
-			;
-//			fprintf( stderr, "no! %d-%d\n", 0, scores[i].numinseq );
-		}
-	}
-	fprintf( stderr, "\ndone\n\n"  );
-	return nkouho;
-}
-
-#endif
 
 static void getfastascoremtx( int **tmpaminodis )
 {
@@ -652,6 +585,7 @@ void arguments( int argc, char *argv[] )
 	classsize = NOTSPECIFIED;
 	picksize = NOTSPECIFIED;
 	tokyoripara = NOTSPECIFIED;
+	legacygapcost = 0;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -660,12 +594,12 @@ void arguments( int argc, char *argv[] )
             switch( c )
             {
 				case 'p':
-					picksize = atoi( *++argv );
+					picksize = myatoi( *++argv );
 					fprintf( stderr, "picksize = %d\n", picksize );
 					--argc;
 					goto nextoption;
 				case 's':
-					classsize = atoi( *++argv );
+					classsize = myatoi( *++argv );
 					fprintf( stderr, "groupsize = %d\n", classsize );
 					--argc;
 					goto nextoption;
@@ -690,25 +624,25 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'k':
-					kimuraR = atoi( *++argv );
+					kimuraR = myatoi( *++argv );
 					fprintf( stderr, "kimuraR = %d\n", kimuraR );
 					--argc;
 					goto nextoption;
 				case 'b':
-					nblosum = atoi( *++argv );
+					nblosum = myatoi( *++argv );
 					scoremtx = 1;
 //					fprintf( stderr, "blosum %d\n", nblosum );
 					--argc;
 					goto nextoption;
 				case 'j':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
 					fprintf( stderr, "jtt %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'm':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
 					fprintf( stderr, "tm %d\n", pamN );
@@ -732,7 +666,7 @@ void arguments( int argc, char *argv[] )
 				case 'Z':
 					fromaln = 1;
 					break;
-				case 'L':
+				case 'U':
 					doalign = 1;
 					break;
 				case 'x':
@@ -756,6 +690,9 @@ void arguments( int argc, char *argv[] )
 				case 'O':
 					fftNoAnchStop = 1;
 					break;
+				case 'L':
+					legacygapcost = 1;
+					break;
 #if 0
 				case 'R':
 					fftRepeatStop = 1;
@@ -814,11 +751,11 @@ void arguments( int argc, char *argv[] )
 					treemethod = 'q'; // minimum
 					break;
 				case 'z':
-					fftThreshold = atoi( *++argv );
+					fftThreshold = myatoi( *++argv );
 					--argc; 
 					goto nextoption;
 				case 'w':
-					fftWinSize = atoi( *++argv );
+					fftWinSize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
                 default:
@@ -854,6 +791,7 @@ void arguments( int argc, char *argv[] )
 
 static int maxl;
 static int tsize;
+static int nunknown = 0;
 
 int seq_grp_nuc( int *grp, char *seq )
 {
@@ -865,7 +803,7 @@ int seq_grp_nuc( int *grp, char *seq )
 		if( tmp < 4 )
 			*grp++ = tmp;
 		else
-			fprintf( stderr, "WARNING : Unknown character %c\r", *(seq-1) );
+			nunknown++;
 	}
 	*grp = END_OF_VEC;
 	return( grp-grpbk );
@@ -881,7 +819,7 @@ int seq_grp( int *grp, char *seq )
 		if( tmp < 6 )
 			*grp++ = tmp;
 		else
-			fprintf( stderr, "WARNING : Unknown character %c\r", *(seq-1) );
+			nunknown++;
 	}
 	*grp = END_OF_VEC;
 	return( grp-grpbk );
@@ -1053,6 +991,7 @@ static void pairalign( int nseq, int *nlen, char **seq, int *mem1, int *mem2, do
 		fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode\n", len1, len2 );
 		alg = 'M';
 		if( commonIP ) FreeIntMtx( commonIP );
+		commonIP = 0;
 		commonAlloc1 = 0;
 		commonAlloc2 = 0;
 	}
@@ -1322,16 +1261,24 @@ static int splitseq_mq( Scores *scores, int nin, int *nlen, char **seq, char **o
 			}
 			free( tmptree );
 	
-			*tree = (char *)calloc( treelen + nin + 5, sizeof( char ) );
-			if( nin > 1 ) **tree = '(';
-			else		  **tree = '\0';
-//			**tree = '\0';
+			*tree = (char *)calloc( treelen + nin + 15, sizeof( char ) );
+			**tree = '\n';
+			if( nin > 1 ) 
+			{
+				*(*tree+1) = '(';
+				*(*tree+2) = '\0';
+			}
+			else
+			{
+				*(*tree+1) = '\0';
+			}
 			for( j=0; j<nin-1; j++ )
 			{
 				sprintf( *tree+strlen( *tree ), "%d,", scores[j].numinseq+1 );
 			}
 			sprintf( *tree+strlen( *tree ), "%d", scores[j].numinseq+1 );
-			if( nin > 1 ) strcat( *tree, ")" );
+			if( nin > 1 ) strcat( *tree, ")\n" );
+			else strcat( *tree, "\n" );
 //			fprintf( stdout, "*tree = %s\n", *tree );
 		}
 
@@ -2898,6 +2845,7 @@ int main( int argc, char *argv[] )
 //		fprintf( stdout, ">%s\n", name[i] );
 //		fprintf( stdout, "%s\n", seq[i] );
 	}
+	if( nunknown ) fprintf( stderr, "\nWARNING : %d unknown characters\n", nunknown );
 //	exit( 1 );
 
 #if 0
