@@ -11,6 +11,8 @@ static int treeout;
 static int distout;
 static int noalign;
 static int multidist;
+static int subalignment;
+static int subalignmentoffset;
 
 #ifdef enablemultithread
 typedef struct _jobtable
@@ -46,6 +48,7 @@ typedef struct _treebasethread_arg
 	RNApair ***singlerna;
 	double *effarr_kozo;
 	int *fftlog;
+	char *mergeoralign;
 	pthread_mutex_t *mutex;
 	pthread_cond_t *treecond;
 } treebasethread_arg_t;
@@ -112,6 +115,9 @@ void arguments( int argc, char *argv[] )
 	consweight_multi = 1.0;
 	consweight_rna = 0.0;
 	multidist = 0;
+	subalignment = 0;
+	subalignmentoffset = 0;
+	legacygapcost = 0;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -125,7 +131,7 @@ void arguments( int argc, char *argv[] )
 					--argc;
                     goto nextoption;
 				case 'I':
-					nadd = atoi( *++argv );
+					nadd = myatoi( *++argv );
 					fprintf( stderr, "nadd = %d\n", nadd );
 					--argc;
 					goto nextoption;
@@ -153,25 +159,25 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'k':
-					kimuraR = atoi( *++argv );
+					kimuraR = myatoi( *++argv );
 					fprintf( stderr, "kappa = %d\n", kimuraR );
 					--argc;
 					goto nextoption;
 				case 'b':
-					nblosum = atoi( *++argv );
+					nblosum = myatoi( *++argv );
 					scoremtx = 1;
 					fprintf( stderr, "blosum %d / kimura 200\n", nblosum );
 					--argc;
 					goto nextoption;
 				case 'j':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
 					fprintf( stderr, "jtt/kimura %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'm':
-					pamN = atoi( *++argv );
+					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
 					fprintf( stderr, "tm %d\n", pamN );
@@ -192,10 +198,15 @@ void arguments( int argc, char *argv[] )
 					--argc;
 					goto nextoption;
 				case 'C':
-					nthread = atoi( *++argv );
+					nthread = myatoi( *++argv );
 					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
 					goto nextoption;
+				case 'H':
+					subalignment = 1;
+					subalignmentoffset = myatoi( *++argv );
+					--argc;
+					goto nextoption;
 				case 'R':
 					rnaprediction = 'r';
 					break;
@@ -225,6 +236,9 @@ void arguments( int argc, char *argv[] )
 				case 'P':
 					dorp = 'p';
 					break;
+				case 'L':
+					legacygapcost = 1;
+					break;
 #if 1
 				case 'O':
 					outgap = 0;
@@ -267,13 +281,13 @@ void arguments( int argc, char *argv[] )
 				case 'a':
 					alg = 'a';
 					break;
+				case 'H':
+					alg = 'H';
+					break;
 #endif
 				case 'Q':
 					alg = 'Q';
 					break;
-				case 'H':
-					alg = 'H';
-					break;
 				case 'A':
 					alg = 'A';
 					break;
@@ -319,11 +333,11 @@ void arguments( int argc, char *argv[] )
 					break;
 /* modification end. */
 				case 'z':
-					fftThreshold = atoi( *++argv );
+					fftThreshold = myatoi( *++argv );
 					--argc; 
 					goto nextoption;
 				case 'w':
-					fftWinSize = atoi( *++argv );
+					fftWinSize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
 				case 'Z':
@@ -389,7 +403,7 @@ static void *distancematrixthread2( void *arg )
 
 		ssi = selfscore[i];
 		if( i % 10 == 0 ) fprintf( stderr, "\r% 5d / %d (thread %4d)", i, njob, thread_no );
-		for( j=i+1; j<njob; j++)
+		for( j=i+1; j<njob; j++ )
 		{
 			ssj = selfscore[j];
 			bunbo = MIN( ssi, ssj );
@@ -465,6 +479,7 @@ static void *treebasethread( void *arg )
 	RNApair ***singlerna = targ->singlerna;
 	double *effarr_kozo = targ->effarr_kozo;
 	int *fftlog = targ->fftlog;
+	char *mergeoralign = targ->mergeoralign;
 
 	char **mseq1, **mseq2;
 	char **localcopy;
@@ -481,7 +496,7 @@ static void *treebasethread( void *arg )
 	int m1, m2;
 	float dumfl = 0.0;
 	int ffttry;
-	RNApair ***grouprna1, ***grouprna2;
+	RNApair ***grouprna1 = NULL, ***grouprna2 = NULL;
 
 
 	mseq1 = AllocateCharMtx( njob, 0 );
@@ -507,7 +522,7 @@ static void *treebasethread( void *arg )
 	if( constraint )
 	{
 		localhomshrink = (LocalHom ***)calloc( njob, sizeof( LocalHom ** ) );
-		for( i=0; i<njob; i++)
+		for( i=0; i<njob; i++ )
 			localhomshrink[i] = (LocalHom **)calloc( njob, sizeof( LocalHom *) );
 	}
 #endif
@@ -546,6 +561,7 @@ static void *treebasethread( void *arg )
 			if( commonIP ) FreeIntMtx( commonIP );
 			commonIP = NULL;
 			Falign( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
+			Falign_udpari_long( NULL, NULL, NULL, NULL, 0, 0, 0, NULL );
 			A__align( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
 			free( mseq1 );
 			free( mseq2 );
@@ -556,11 +572,24 @@ static void *treebasethread( void *arg )
 			free( effarr2_kozo );
 			free( indication1 );
 			free( indication2 );
+			if( rnakozo && rnaprediction == 'm' )
+			{
+				if( grouprna1 ) free( grouprna1 ); // nakami ha?
+				if( grouprna2 ) free( grouprna2 ); // nakami ha?
+				grouprna1 = grouprna2 = NULL;
+			}
 			if( constraint )
 			{
-				for( i=0; i<njob; i++)
-					free( localhomshrink[i] );
-				free( localhomshrink );
+				if( localhomshrink ) // nen no tame
+				{
+					for( i=0; i<njob; i++ )
+					{
+						free( localhomshrink[i] );
+						localhomshrink[i] = NULL;
+					}
+					free( localhomshrink );
+					localhomshrink = NULL;
+				}
 			}
 			return( NULL );
 		}
@@ -582,12 +611,28 @@ static void *treebasethread( void *arg )
 
 //		pthread_mutex_unlock( targ->mutex );
 
+		if( mergeoralign[l] == 'n' )
+		{
+//			fprintf( stderr, "SKIP!\n" );
+			dep[l].done = 1;
+			(*nrunpt)--;
+			pthread_cond_broadcast( targ->treecond );
+			free( topol[l][0] );
+			free( topol[l][1] );
+			free( topol[l] );
+			pthread_mutex_unlock( targ->mutex );
+			continue;
+		}
+
+
 
 		m1 = topol[l][0][0];
 		m2 = topol[l][1][0];
 
 //		pthread_mutex_lock( targ->mutex );
 
+
+
         len1 = strlen( aseq[m1] );
         len2 = strlen( aseq[m2] );
         if( *alloclen <= len1 + len2 )
@@ -611,6 +656,8 @@ static void *treebasethread( void *arg )
 
 		pthread_mutex_unlock( targ->mutex );
 
+
+
 		if( effarr_kozo )
 		{
 			clus1 = fastconjuction_noname_kozo( topol[l][0], localcopy, mseq1, effarr1, effarr, effarr1_kozo, effarr_kozo, indication1 );
@@ -682,6 +729,7 @@ static void *treebasethread( void *arg )
 			fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode.\n", len1, len2 );
 			alg = 'M';
 			if( commonIP ) FreeIntMtx( commonIP );
+			commonIP = NULL;
 			commonAlloc1 = 0;
 			commonAlloc2 = 0;
 		}
@@ -830,7 +878,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 	static int *alreadyaligned;
 	float dumfl = 0.0;
 	int ffttry;
-	RNApair ***grouprna1, ***grouprna2;
+	RNApair ***grouprna1 = NULL, ***grouprna2 = NULL;
 
 	if( rnakozo && rnaprediction == 'm' )
 	{
@@ -857,7 +905,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 		if( constraint )
 		{
 			localhomshrink = (LocalHom ***)calloc( njob, sizeof( LocalHom ** ) );
-			for( i=0; i<njob; i++)
+			for( i=0; i<njob; i++ )
 				localhomshrink[i] = (LocalHom **)calloc( njob, sizeof( LocalHom *) );
 		}
 #endif
@@ -884,6 +932,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 
 	if( constraint )
 		calcimportance( njob, effarr, aseq, localhomtable );
+//		dontcalcimportance( njob, effarr, aseq, localhomtable ); // CHUUIII!!!!!
 
 
 //	writePre( njob, name, nlen, aseq, 0 );
@@ -935,7 +984,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 			clus2 = fastconjuction_noname( topol[l][1], aseq, mseq2, effarr2, effarr, indication2 );
 		}
 
-		if( mergeoralign[l] == '1' || mergeoralign[l] == '2' )
+		if( mergeoralign[l] == '1' || mergeoralign[l] == '2' ) // only in serial version
 		{
 			newgapstr = "=";
 		}
@@ -1015,6 +1064,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 			fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode.\n", len1, len2 );
 			alg = 'M';
 			if( commonIP ) FreeIntMtx( commonIP );
+			commonIP = NULL;
 			commonAlloc1 = 0;
 			commonAlloc2 = 0;
 		}
@@ -1145,6 +1195,25 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 #if SCOREOUT
 	fprintf( stderr, "totalscore = %10.2f\n\n", tscore );
 #endif
+	if( rnakozo && rnaprediction == 'm' )
+	{
+		if( grouprna1 ) free( grouprna1 ); // nakami ha?
+		if( grouprna2 ) free( grouprna2 ); // nakami ha?
+		grouprna1 = grouprna2 = NULL;
+	}
+	if( constraint )
+	{
+		if( localhomshrink ) // nen no tame
+		{
+			for( i=0; i<njob; i++ )
+			{
+				free( localhomshrink[i] );
+				localhomshrink[i] = NULL;
+			}
+			free( localhomshrink );
+			localhomshrink = NULL;
+		}
+	}
 }
 
 static void WriteOptions( FILE *fp )
@@ -1236,11 +1305,16 @@ int main( int argc, char *argv[] )
 	int alignmentlength;
 	char *mergeoralign;
 	int foundthebranch;
-	
+	int nsubalignments, maxmem;
+	int **subtable;
+	int *insubtable;
+	int *preservegaps;
+	char ***subalnpt;
+
 	char c;
 	int alloclen;
 	LocalHom **localhomtable = NULL;
-	RNApair ***singlerna;
+	RNApair ***singlerna = NULL;
 	float ssi, ssj, bunbo;
 	static char *kozoarivec;
 	int nkozo;
@@ -1250,6 +1324,8 @@ int main( int argc, char *argv[] )
 	nthread = 0;
 #endif
 
+	if( fastathreshold < 0.0001 ) constraint = 0; 
+
 	if( inputfile )
 	{
 		infp = fopen( inputfile, "r" );
@@ -1275,6 +1351,20 @@ int main( int argc, char *argv[] )
 		exit( 1 );
 	}
 
+	if( subalignment )
+	{
+		readsubalignmentstable( njob, NULL, NULL, &nsubalignments, &maxmem );
+		fprintf( stderr, "nsubalignments = %d\n", nsubalignments );
+		fprintf( stderr, "maxmem = %d\n", maxmem );
+		subtable = AllocateIntMtx( nsubalignments, maxmem+1 );
+		insubtable = AllocateIntVec( njob );
+		for( i=0; i<njob; i++ ) insubtable[i] = 0;
+		preservegaps = AllocateIntVec( njob );
+		for( i=0; i<njob; i++ ) preservegaps[i] = 0;
+		subalnpt = AllocateCharCub( nsubalignments, maxmem, 0 );
+		readsubalignmentstable( njob, subtable, preservegaps, NULL, NULL );
+	}
+
 	seq = AllocateCharMtx( njob, nlenmax+1 );
 	mseq1 = AllocateCharMtx( njob, 0 );
 	mseq2 = AllocateCharMtx( njob, 0 );
@@ -1297,10 +1387,10 @@ int main( int argc, char *argv[] )
 	if( constraint )
 	{
 		localhomtable = (LocalHom **)calloc( njob, sizeof( LocalHom *) );
-		for( i=0; i<njob; i++)
+		for( i=0; i<njob; i++ )
 		{
 			localhomtable[i] = (LocalHom *)calloc( njob, sizeof( LocalHom ) );
-			for( j=0; j<njob; j++)
+			for( j=0; j<njob; j++ )
 			{
 				localhomtable[i][j].start1 = -1;
 				localhomtable[i][j].end1 = -1;
@@ -1536,11 +1626,17 @@ int main( int argc, char *argv[] )
 		fflush( stderr );
 		if( topin )
 		{
-			fprintf( stderr, "Loading a topology ... " );
-			loadtop( njob, iscore, topol, len );
-			fprintf( stderr, "\ndone.\n\n" );
+			fprintf( stderr, "--topin has been disabled\n" );
+			exit( 1 );
+//			fprintf( stderr, "Loading a topology ... " );
+//			loadtop( njob, iscore, topol, len );
+//			fprintf( stderr, "\ndone.\n\n" );
 		}
-		else if( treeout )
+		else if( subalignment ) // merge error no tame
+		{
+			fixed_supg_float_realloc_nobk_halfmtx_treeout_constrained( njob, iscore, topol, len, name, nlen, dep, nsubalignments, subtable );
+		}
+		else if( treeout ) // merge error no tame
 		{
 			fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( njob, iscore, topol, len, name, nlen, dep );
 		}
@@ -1594,7 +1690,7 @@ int main( int argc, char *argv[] )
 #if 0
 		utree = 0; counteff( njob, topol, len, eff ); utree = 1;
 #else
-		counteff_simple_float( njob, topol, len, eff );
+		counteff_simple_float_nostatic( njob, topol, len, eff );
 		for( i=njob-nadd; i<njob; i++ ) eff[i] /= (double)100;
 #if 0
 		fprintf( stderr, "######  WEIGHT = \n" );
@@ -1646,6 +1742,7 @@ int main( int argc, char *argv[] )
 	alloclen = nlenmax*2+1; //chuui!
 	bseq = AllocateCharMtx( njob, alloclen );
 
+
 	if( nadd )
 	{
 		alignmentlength = strlen( seq[0] );
@@ -1767,6 +1864,126 @@ int main( int argc, char *argv[] )
 		commongappick( njob-nadd, seq );
 		for( i=0; i<njob-nadd; i++ ) strcpy( bseq[i], seq[i] );
 	}
+//--------------- kokokara ----
+	else if( subalignment )
+	{
+		for( i=0; i<njob-1; i++ ) mergeoralign[i] = 'a';
+		for( i=0; i<nsubalignments; i++ )
+		{
+			fprintf( stderr, "Checking subalignment %d:\n", i+1 );
+			alignmentlength = strlen( seq[subtable[i][0]] );
+//			for( j=0; subtable[i][j]!=-1; j++ )
+//				fprintf( stderr, " %d. %-30.30s\n", subtable[i][j]+1, name[subtable[i][j]]+1 );
+			for( j=0; subtable[i][j]!=-1; j++ )
+			{
+				if( subtable[i][j] >= njob )
+				{
+					fprintf( stderr, "No such sequence, %d.\n", subtable[i][j]+1 );
+					exit( 1 );
+				}
+				if( alignmentlength != strlen( seq[subtable[i][j]] ) )
+				{
+					fprintf( stderr, "\n" );
+					fprintf( stderr, "###############################################################################\n" );
+					fprintf( stderr, "# ERROR!\n" );
+					fprintf( stderr, "# Subalignment %d must be aligned.\n", i+1 );
+					fprintf( stderr, "# Please check the alignment lengths of following sequences.\n" );
+					fprintf( stderr, "#\n" );
+					fprintf( stderr, "# %d. %-10.10s -> %d letters (including gaps)\n", subtable[i][0]+1, name[subtable[i][0]]+1, alignmentlength );
+					fprintf( stderr, "# %d. %-10.10s -> %d letters (including gaps)\n", subtable[i][j]+1, name[subtable[i][j]]+1, (int)strlen( seq[subtable[i][j]] ) );
+					fprintf( stderr, "#\n" );
+					fprintf( stderr, "# See http://mafft.cbrc.jp/alignment/software/merge.html for details.\n" );
+					if( subalignmentoffset )
+					{
+						fprintf( stderr, "#\n" );
+						fprintf( stderr, "# You specified seed alignment(s) consisting of %d sequences.\n", subalignmentoffset );
+						fprintf( stderr, "# In this case, the rule of numbering is:\n" );
+						fprintf( stderr, "#   The aligned seed sequences are numbered as 1 .. %d\n", subalignmentoffset );
+						fprintf( stderr, "#   The input sequences to be aligned are numbered as %d .. %d\n", subalignmentoffset+1, subalignmentoffset+njob );
+					}
+					fprintf( stderr, "###############################################################################\n" );
+					fprintf( stderr, "\n" );
+					exit( 1 );
+				}
+				insubtable[subtable[i][j]] = 1;
+			}
+			for( j=0; j<njob-1; j++ )
+			{
+				if( includemember( topol[j][0], subtable[i] ) && includemember( topol[j][1], subtable[i] ) )
+				{
+					mergeoralign[j] = 'n';
+				}
+			}
+			foundthebranch = 0;
+			for( j=0; j<njob-1; j++ )
+			{
+				if( samemember( topol[j][0], subtable[i] ) || samemember( topol[j][1], subtable[i] ) )
+				{
+					foundthebranch = 1;
+					fprintf( stderr, " -> OK\n" );
+					break;
+				}
+			}
+			if( !foundthebranch )
+			{
+				system( "cp infile.tree GuideTree" ); // tekitou
+				fprintf( stderr, "\n" );
+				fprintf( stderr, "###############################################################################\n" );
+				fprintf( stderr, "# ERROR!\n" );
+				fprintf( stderr, "# Subalignment %d does not form a monophyletic cluster\n", i+1 );
+				fprintf( stderr, "# in the guide tree ('GuideTree' in this directory) internally computed.\n" );
+				fprintf( stderr, "# If you really want to use this subalignment, pelase give a tree with --treein \n" );
+				fprintf( stderr, "# http://mafft.cbrc.jp/alignment/software/treein.html\n" );
+				fprintf( stderr, "# http://mafft.cbrc.jp/alignment/software/merge.html\n" );
+				if( subalignmentoffset )
+				{
+					fprintf( stderr, "#\n" );
+					fprintf( stderr, "# You specified seed alignment(s) consisting of %d sequences.\n", subalignmentoffset );
+					fprintf( stderr, "# In this case, the rule of numbering is:\n" );
+					fprintf( stderr, "#   The aligned seed sequences are numbered as 1 .. %d\n", subalignmentoffset );
+					fprintf( stderr, "#   The input sequences to be aligned are numbered as %d .. %d\n", subalignmentoffset+1, subalignmentoffset+njob );
+				}
+				fprintf( stderr, "############################################################################### \n" );
+				fprintf( stderr, "\n" );
+				exit( 1 );
+			}
+//			commongappick( seq[subtable[i]], subalignment[i] ); // irukamo
+		}
+#if 0
+		for( i=0; i<njob-1; i++ )
+		{
+			fprintf( stderr, "STEP %d\n", i+1 );
+			fprintf( stderr, "group1 = " );
+			for( j=0; topol[i][0][j] != -1; j++ )
+				fprintf( stderr, "%d ", topol[i][0][j]+1 );
+			fprintf( stderr, "\n" );
+			fprintf( stderr, "group2 = " );
+			for( j=0; topol[i][1][j] != -1; j++ )
+				fprintf( stderr, "%d ", topol[i][1][j]+1 );
+			fprintf( stderr, "\n" );
+			fprintf( stderr, "%d -> %c\n\n", i, mergeoralign[i] );
+		}
+#endif
+
+		for( i=0; i<njob; i++ ) 
+		{
+			if( insubtable[i] ) strcpy( bseq[i], seq[i] );
+			else gappick0( bseq[i], seq[i] );
+		}
+
+		for( i=0; i<nsubalignments; i++ ) 
+		{
+			for( j=0; subtable[i][j]!=-1; j++ ) subalnpt[i][j] = bseq[subtable[i][j]];
+			if( !preservegaps[i] ) commongappick( j, subalnpt[i] );
+		}
+
+		FreeIntMtx( subtable );
+		free( insubtable );
+		for( i=0; i<nsubalignments; i++ ) free( subalnpt[i] );
+		free( subalnpt );
+		free( preservegaps );
+	}
+//--------------- kokomade ----
 	else
 	{
 		for( i=0; i<njob; i++ ) gappick0( bseq[i], seq[i] );
@@ -1782,13 +1999,15 @@ int main( int argc, char *argv[] )
 		for( i=0; i<njob; i++ )
 		{
 			nogaplen = strlen( bseq[i] );
-			singlerna[i] = (RNApair **)calloc( nogaplen, sizeof( RNApair * ) );
+			singlerna[i] = (RNApair **)calloc( nogaplen+1, sizeof( RNApair * ) );
 			for( j=0; j<nogaplen; j++ )
 			{
 				singlerna[i][j] = (RNApair *)calloc( 1, sizeof( RNApair ) );
 				singlerna[i][j][0].bestpos = -1;
 				singlerna[i][j][0].bestscore = -1.0;
 			}
+			singlerna[i][nogaplen] =  NULL;
+//			fprintf( stderr, "### reading bpp %d ...\n", i );
 			readmccaskill( prep, singlerna[i], nogaplen );
 		}
 		fclose( prep );
@@ -1826,6 +2045,7 @@ int main( int argc, char *argv[] )
 
 		if( constraint )
 			calcimportance( njob, eff, bseq, localhomtable );
+//			dontcalcimportance( njob, eff, bseq, localhomtable ); // CHUUUUIIII!!!
 
 		for( i=0; i<nthread_yoyu; i++ )
 		{
@@ -1843,6 +2063,7 @@ int main( int argc, char *argv[] )
 			targ[i].singlerna = singlerna;
 			targ[i].effarr_kozo = eff_kozo_mapped;
 			targ[i].fftlog = fftlog;
+			targ[i].mergeoralign = mergeoralign;
 			targ[i].mutex = &mutex;
 			targ[i].treecond = &treecond;
 
@@ -1861,6 +2082,7 @@ int main( int argc, char *argv[] )
 	}
 	else
 #endif
+
 		treebase( nlen, bseq, nadd, mergeoralign, mseq1, mseq2, topol, eff, &alloclen, localhomtable, singlerna, eff_kozo_mapped );
 	fprintf( stderr, "\ndone.\n" );
 	if( scoreout )
@@ -1875,9 +2097,9 @@ int main( int argc, char *argv[] )
 	if( constraint )
 	{
 		LocalHom *tmppt1, *tmppt2;
-		for( i=0; i<njob; i++)
+		for( i=0; i<njob; i++ )
 		{
-			for( j=0; j<njob; j++)
+			for( j=0; j<njob; j++ )
 			{
 				tmppt1 = localhomtable[i]+j;
 				while( tmppt2 = tmppt1->next )
@@ -1896,6 +2118,22 @@ int main( int argc, char *argv[] )
 	fprintf( trap_g, "done.\n" );
 	fclose( trap_g );
 	free( mergeoralign );
+	if( rnakozo && rnaprediction == 'm' ) 
+	{
+		if( singlerna ) // nen no tame
+		{
+			for( i=0; i<njob; i++ ) 
+			{
+				for( j=0; singlerna[i][j]!=NULL; j++ )
+				{
+					if( singlerna[i][j] ) free( singlerna[i][j] );
+				}
+				if( singlerna[i] ) free( singlerna[i] );
+			}
+			free( singlerna );
+			singlerna = NULL;
+		}
+	}
 
 	writeData_pointer( prep_g, njob, name, nlen, bseq );
 #if 0
