@@ -134,7 +134,7 @@ void cpmx_calc( char **seq, float **cpmx, double *eff, int lgth, int clus )
 	double totaleff = 0.0;
 
 	for( i=0; i<clus; i++ ) totaleff += eff[i]; 
-	for( i=0; i<26; i++ ) for( j=0; j<lgth; j++ ) cpmx[i][j] = 0.0;
+	for( i=0; i<nalphabets; i++ ) for( j=0; j<lgth; j++ ) cpmx[i][j] = 0.0;
 	for( j=0; j<lgth; j++ ) for( k=0; k<clus; k++ )
 			cpmx[(int)amino_n[(int)seq[k][j]]][j] += (float)eff[k] / totaleff;
 }
@@ -145,7 +145,7 @@ void cpmx_calc_new_bk( char **seq, float **cpmx, double *eff, int lgth, int clus
     int  i, j, k;
     float feff;
 
-    for( i=0; i<26; i++ ) for( j=0; j<lgth; j++ ) cpmx[i][j] = 0.0;
+    for( i=0; i<nalphabets; i++ ) for( j=0; j<lgth; j++ ) cpmx[i][j] = 0.0;
     for( k=0; k<clus; k++ )
     {
         feff = (float)eff[k];
@@ -163,7 +163,7 @@ void cpmx_calc_new( char **seq, float **cpmx, double *eff, int lgth, int clus )
 	float *cpmxpt, **cpmxptpt;
 	char *seqpt;
 
-	j = 26;
+	j = nalphabets;
 	cpmxptpt = cpmx;
 	while( j-- )
 	{
@@ -195,7 +195,7 @@ void MScpmx_calc_new( char **seq, float **cpmx, double *eff, int lgth, int clus
 	while( j-- )
 	{
 		cpmxpt = *cpmxptpt++;
-		i = 26;
+		i = nalphabets;
 		while( i-- )
 			*cpmxpt++ = 0.0;
 	}
@@ -209,7 +209,7 @@ void MScpmx_calc_new( char **seq, float **cpmx, double *eff, int lgth, int clus
 			(*cpmxptpt++)[(int)amino_n[(int)*seqpt++]] += feff;
 	}
 #if 0
-	for( j=0; j<lgth; j++ ) for( i=0; i<26; i++ ) cpmx[j][i] = 0.0;
+	for( j=0; j<lgth; j++ ) for( i=0; i<nalphabets; i++ ) cpmx[j][i] = 0.0;
 	for( k=0; k<clus; k++ )
 	{
 		feff = (float)eff[k];
@@ -355,17 +355,6 @@ void makegrouprna( RNApair ***group, RNApair ***all, int *memlist )
 	}
 }
 
-void makegrouprnait( RNApair ***group, RNApair ***all, char **pair, int s )
-{
-	int k, m;
-	for( m=s, k=0; m<njob; m++ )
-	{
-		if( pair[s][m] != 0 )
-		{
-			group[k++] = all[m];
-		}
-	}
-}
 
 int fastconjuction_noweight( int *memlist, char **seq, char **aseq, double *peff, char *d )
 {
@@ -532,64 +521,8 @@ int fastconjuction( int *memlist, char **seq, char **aseq, double *peff, double
 }
 
 
-int conjuctionfortbfast_kozo( double *tmptmptmp, char **pair, int s, char **seq, char **aseq, double *peff, double *eff, double *peff_kozo, double *eff_kozo, char *d )
-{
-	int m, k;
-	char b[B];
-	double total;
-	double total_kozo;
 
-#if DEBUG
-	fprintf( stderr, "s = %d\n", s );
-#endif
-
-	total = 0.0;
-//	total_kozo = 0.0;
-	total_kozo = *tmptmptmp; // masaka
-	d[0] = 0;
-	for( m=s, k=0; m<njob; m++ )
-	{
-		if( pair[s][m] != 0 )
-		{
-			sprintf( b, " %d", m+1 ); 
-#if 1
-			if( strlen( d ) < 100 ) strcat( d, b );
-#else
-			strcat( d, b );
-#endif
-			aseq[k] = seq[m];
-			peff[k] = eff[m];
-			peff_kozo[k] = eff_kozo[m];
-			total += peff[k];
-			total_kozo += peff_kozo[k];
-#if 0
-			strcpy( aname[k], name[m] );
-#endif
-			k++;
-		}
-	}
-#if 1
-	for( m=0; m<k; m++ )
-		peff[m] /= total;
-	if( total_kozo > 0.0 )
-	{
-		for( m=0; m<k; m++ ) 
-		{
-			peff_kozo[m] /= total_kozo;
-			if( peff_kozo[m] > 0.0 ) peff_kozo[m] += peff[m];
-		}
-	}
-	else //iranai
-	{
-		for( m=0; m<k; m++ ) peff_kozo[m] = 0.0;
-	}
-#endif
-//	fprintf( stderr, "\n\ndvtditr_total_kozo = %f\n\n", total_kozo );
-	*tmptmptmp = total_kozo;
-	return( k );
-}
-
-int conjuctionfortbfast( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char *d )
+int conjuctionfortbfast_old( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char *d )
 {
 	int m, k;
 	char *b;
@@ -628,6 +561,7 @@ int conjuctionfortbfast( char **pair, int s, char **seq, char **aseq, double *pe
 	free( b );
 	return( k );
 }
+
 int conjuction( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char **name, char **aname, char *d )
 {
 	int m, k;
@@ -672,7 +606,7 @@ void floatdelete( float **cpmx, int d, int len )
 
 	for( i=d; i<len-1; i++ )
 	{
-		for( j=0; j<26; j++ )
+		for( j=0; j<nalphabets; j++ )
 		{
 			cpmx[j][i] = cpmx[j][i+1]; 
 		}
@@ -798,34 +732,36 @@ void nodeFromABranch( int nseq, int *result, int **pairwisenode, int ***topol, d
 					
 
 				
-		
-
-#if 0
-void OneClusterAndTheOther( int locnjob, char pair[njob][njob], int *s1, int *s2, int ***topol, int step, int branch )
-#else
-void OneClusterAndTheOther( int locnjob, char **pair, int *s1, int *s2, int ***topol, int step, int branch )
-#endif
+void OneClusterAndTheOther_fast( int locnjob, int *memlist1, int *memlist2, int *s1, int *s2, char *pair, int ***topol, int step, int branch )
 {
-	int i;
+	int i, k;
 	int r1;
-	
-    *s1 = topol[step][branch][0];
-    for( i=0; (r1=topol[step][branch][i])>-1; i++ ) 
-        pair[*s1][r1] = 1;
-    for( i=0; i<locnjob; i++ ) 
+//	char *pair;
+
+//	pair = calloc( locnjob, sizeof( char ) );
+
+	for( i=0; i<locnjob; i++ ) pair[i] = 0;
+    for( i=0, k=0; (r1=topol[step][branch][i])>-1; i++ ) 
+	{
+        pair[r1] = 1;
+		memlist1[k++] = r1;
+	}
+	memlist1[k] = -1;
+
+    for( i=0, k=0; i<locnjob; i++ ) 
     {
-        if( !pair[*s1][i] ) 
+        if( !pair[i] ) 
         {
-            *s2 = i;
-            break;
+			memlist2[k++] = i;
         }
     }
-    for( i=*s2; i<locnjob; i++ ) 
-    {
-        if( !pair[*s1][i] )
-            pair[*s2][i] = 1;
-    }
+	memlist2[k] = -1;
+
+	*s1 = memlist1[0];
+	*s2 = memlist2[0];
+//	free( pair );
 }
+		
 
 void makeEffMtx( int nseq, double **mtx, double *vec )
 {
@@ -873,27 +809,42 @@ int shrinklocalhom( char **pair, int s1, int s2, LocalHom **localhom, LocalHom *
 	}
 	return( 0 );
 }
-int msshrinklocalhom( char **pair, int s1, int s2, LocalHom **localhom, LocalHom ***localhomshrink )
-{
-	int m1, k1, n1, m2, k2, n2;
 
-	for( m1=s1, k1=0; m1<njob; m1++ )
+int msshrinklocalhom_fast( int *memlist1, int *memlist2, LocalHom **localhom, LocalHom ***localhomshrink )
+{
+	int m1, k1, m2, k2;
+
+	for( k1=0; (m1=memlist1[k1])!=-1; k1++ )
 	{
-		if( pair[s1][m1] != 0 )
+		for( k2=0; (m2=memlist2[k2])!=-1; k2++ )
 		{
-			for( m2=s2, k2=0; m2<njob; m2++ )
+			if( localhom[m1][m2].opt == -1 )
+				localhomshrink[k1][k2] = NULL;
+			else
+				localhomshrink[k1][k2] = localhom[m1]+m2;
+		}
+	}
+	return( 0 );
+}
+int fastshrinklocalhom_one( int *mem1, int *mem2, int norg, LocalHom **localhom, LocalHom ***localhomshrink )
+{
+	int k1, k2;
+	int *intpt1, *intpt2;
+
+	
+	for( intpt1=mem1, k1=0; *intpt1!=-1; intpt1++, k1++ )
+	{
+		for( intpt2=mem2, k2=0; *intpt2!=-1; intpt2++, k2++ )
+		{
+			if( *intpt2 != norg ) 
 			{
-				if( pair[s2][m2] != 0 )
-				{
-					n1 = MIN(m1,m2); n2=MAX(m1,m2);
-					if( localhom[m1][m2].opt == -1 )
-						localhomshrink[k1][k2] = NULL;
-					else
-						localhomshrink[k1][k2] = localhom[n1]+n2;
-					k2++;
-				}
+				fprintf( stderr, "ERROR! *intpt2 = %d\n", *intpt2 );
+				exit( 1 );
 			}
-			k1++;
+			if( localhom[*intpt1][0].opt == -1 )
+				localhomshrink[k1][k2] = NULL;
+			else
+				localhomshrink[k1][k2] = localhom[*intpt1];
 		}
 	}
 	return( 0 );
